; Game engine code --------------------------------------------------------------

; Arcade Game Designer.
; (C) 2008 - 2020 Jonathan Cauldwell.
; Amstrad CPC Engine v0.7.9

; Global definitions ------------------------------------------------------------

SHRAPN equ 40630           ; shrapnel table, just below map.
MAP    equ 40960           ; properties map buffer.
MAPHI  equ 160             ; high byte of MAP.


; Block characteristics.

PLATFM equ 1               ; platform.
WALL   equ PLATFM + 1      ; solid wall.
LADDER equ WALL + 1        ; ladder.
FODDER equ LADDER + 1      ; fodder block.
DEADLY equ FODDER + 1      ; deadly block.
CUSTOM equ DEADLY + 1      ; custom block.
WATER  equ CUSTOM + 1      ; water block.
COLECT equ WATER + 1       ; collectable block.
NUMTYP equ COLECT + 1      ; number of types.

; Sprites.

NUMSPR equ 12              ; number of sprites.
TABSIZ equ 17              ; size of each entry.
SPRBUF equ NUMSPR * TABSIZ ; size of entire table.
NMESIZ equ 4               ; bytes stored in nmetab for each sprite.
X      equ 8               ; new x coordinate of sprite.
Y      equ X + 1           ; new y coordinate of sprite.
PAM1ST equ 5               ; first sprite parameter, old x (ix+5).

; Particle engine.

NUMSHR equ 55              ; pieces of shrapnel.
SHRSIZ equ 6               ; bytes per particle.


; Game starts here.

       org 2000

start  equ $

       ld hl,(57)          ; fetch interrupt address.
       ld (firmad),hl      ; store firmware address.
       call 48127          ; initialise screen to default settings.
       call 48148          ; clear the screen.
       xor a               ; mode 0, 16 colours.
       call 48142          ; set screen mode.

start0 call game           ; start the game.
       jr start0

joyval defb 0              ; joystick reading.
frmno  defb 0              ; selected frame.

; Don't change the order of these four.  Menu routine relies on winlft following wintop.

wintop defb WINDOWTOP      ; top of window.
winlft defb WINDOWLFT      ; left edge.
winhgt defb WINDOWHGT      ; window height.
winwid defb WINDOWWID      ; window width.

numob  defb NUMOBJ         ; number of objects in game.

; Variables start here.
; Pixel versions of wintop, winlft, winhgt, winwid.

wntopx defb 8 * WINDOWTOP
wnlftx defb 5 * WINDOWLFT
wnbotx defb WINDOWTOP * 8 + WINH - 16
wnrgtx defb WINDOWLFT * 5 + WINW - 10
scno   defb 0              ; present screen number.
numlif defb 3              ; number of lives.
vara   defb 0              ; general-purpose variable.
varb   defb 0              ; general-purpose variable.
varc   defb 0              ; general-purpose variable.
vard   defb 0              ; general-purpose variable.
vare   defb 0              ; general-purpose variable.
varf   defb 0              ; general-purpose variable.
varg   defb 0              ; general-purpose variable.
varh   defb 0              ; general-purpose variable.
vari   defb 0              ; general-purpose variable.
varj   defb 0              ; general-purpose variable.
vark   defb 0              ; general-purpose variable.
varl   defb 0              ; general-purpose variable.
varm   defb 0              ; general-purpose variable.
varn   defb 0              ; general-purpose variable.
varo   defb 0              ; general-purpose variable.
varp   defb 0              ; general-purpose variable.
varq   defb 0              ; general-purpose variable.
varr   defb 0              ; general-purpose variable.
vars   defb 0              ; general-purpose variable.
vart   defb 0              ; general-purpose variable.
varu   defb 0              ; general-purpose variable.
varv   defb 0              ; general-purpose variable.
varw   defb 0              ; general-purpose variable.
varz   defb 0              ; general-purpose variable.
contrl defb 0              ; control, 0 = keyboard, 1 = Joystick 1, 2 = Joystick 2.
charx  defb 0              ; cursor x position.
chary  defb 0              ; cursor y position.
clock  defb 0              ; last clock reading.
varrnd defb 255            ; last random number.
varobj defb 254            ; last object number.
varopt defb 255            ; last option chosen from menu.
varblk defb 255            ; block type.
nexlev defb 0              ; next level flag.
restfl defb 0              ; restart screen flag.
deadf  defb 0              ; dead flag.
gamwon defb 0              ; game won flag.
dispx  defb 0              ; cursor x position.
dispy  defb 0              ; cursor y position.
loopa  defb 0              ; loop counter system variable.
loopb  defb 0              ; loop counter system variable.
loopc  defb 0              ; loop counter system variable.

; Make sure pointers are arranged in the same order as the data itself.

frmptr defw frmlst         ; sprite frames.
proptr defw bprop          ; address of char properties.
scrptr defw scdat          ; address of screens.
nmeptr defw nmedat         ; enemy start positions.

; Assorted game routines which can go in contended memory.

; Modify for inventory.

minve  ld hl,invdis        ; routine address.
       ld (mod0+1),hl      ; set up menu routine.
       ld (mod2+1),hl      ; set up count routine.
       ld hl,fopt          ; find option from available objects.
       ld (mod1+1),hl      ; set up routine.
       jr dbox             ; do menu routine.

; Modify for menu.

mmenu  ld hl,always        ; routine address.
       ld (mod0+1),hl      ; set up routine.
       ld (mod2+1),hl      ; set up count routine.
       ld hl,fstd          ; standard option selection.
       ld (mod1+1),hl      ; set up routine.

; Drop through into box routine.

; Work out size of box for message or menu.

;dbox   ld hl,nummsg        ; total messages.
;       cp (hl)             ; does this one exist?
;       ret nc              ; no, nothing to display.
dbox   ld hl,msgdat        ; pointer to messages.
       call getwrd         ; get message number.
       push hl             ; store pointer to message.
       ld d,1              ; height.
       xor a               ; start at object zero.
       ld (combyt),a       ; store number of object in combyt.
       ld e,a              ; maximum width.
dbox5  ld b,0              ; this line's width.
mod2   call always         ; item in player's possession?
       jr nz,dbox6         ; not in inventory, skip this line.
       inc d               ; add to tally.
dbox6  ld a,(hl)           ; get character.
       inc hl              ; next character.
       cp ','              ; reached end of line?
       jr z,dbox3          ; yes.
       cp 13               ; reached end of line?
       jr z,dbox3          ; yes.
       inc b               ; add to this line's width.
       and a               ; end of message?
       jp m,dbox4          ; yes, end count.
       jr dbox6            ; repeat until we find the end.
dbox3  ld a,e              ; maximum line width.
       cp b                ; have we exceeded longest so far?
       jr nc,dbox5         ; no, carry on looking.
       ld e,b              ; make this the widest so far.
       jr dbox5            ; keep looking.
dbox4  ld a,e              ; maximum line width.
       cp b                ; have we exceeded longest so far?
       jr nc,dbox8         ; no, carry on looking.
       ld e,b              ; final line is the longest so far.
dbox8  dec d               ; decrement items found.
       jp z,dbox15         ; total was zero.
       ld a,e              ; longest line.
       and a               ; was it zero?
       jp z,dbox15         ; total was zero.
       ld (bwid),de        ; set up size.

; That's set up our box size.

       ld a,(winhgt)       ; window height in characters.
       sub d               ; subtract height of box.
       rra                 ; divide by 2.
       ld hl,wintop        ; top edge of window.
       add a,(hl)          ; add displacement.
       ld (btop),a         ; set up box top.
       ld a,(winwid)       ; window width in characters.
       sub e               ; subtract box width.
       rra                 ; divide by 2.
       inc hl              ; left edge of window.
       add a,(hl)          ; add displacement.
       ld (blft),a         ; box left.
       pop hl              ; restore message pointer.
       ld a,(btop)         ; box top.
       ld (dispx),a        ; set display coordinate.
       xor a               ; start at object zero.
       ld (combyt),a       ; store number of object in combyt.
dbox2  ld a,(combyt)       ; get object number.
mod0   call always         ; check inventory for display.
       jp nz,dbox13        ; not in inventory, skip this line.

       ld a,(blft)         ; box left.
       ld (dispy),a        ; set left display position.
       ld a,(bwid)         ; box width.
       ld b,a              ; store width.
dbox0  ld a,(hl)           ; get character.
       cp ','              ; end of line?
       jr z,dbox1          ; yes, next one.
       cp 13               ; end of option?
       jr z,dbox1          ; yes, on to next.
       dec b               ; one less to display.
       and 127             ; remove terminator.
       push bc             ; store characters remaining.
       push hl             ; store address on stack.
       call ptxta          ; display on screen and advance.
       pop hl              ; retrieve address of next character.
       pop bc              ; chars left for this line.
       ld a,(hl)           ; get character.
       inc hl              ; next character.
       cp 128              ; end of message?
       jp nc,dbox7         ; yes, job done.
       ld a,b              ; chars remaining.
       and a               ; are any left?
       jr nz,dbox0         ; yes, continue.

; Reached limit of characters per line.

dbox9  ld a,(hl)           ; get character.
       inc hl              ; next one.
       cp ','              ; another line?
       jr z,dbox10         ; yes, do next line.
       cp 13               ; another line?
       jr z,dbox10         ; yes, on to next.
       cp 128              ; end of message?
       jr nc,dbox11        ; yes, finish message.
       jr dbox9

; Fill box to end of line.

dboxf  push hl             ; store address on stack.
       push bc             ; store characters remaining.
       ld a,32             ; space character.
       call ptxta          ; display character and advance.
       pop bc              ; retrieve character count.
       pop hl              ; retrieve address of next character.
       djnz dboxf          ; repeat for remaining chars on line.
       ret
dbox1  inc hl              ; skip character.
       call dboxf          ; fill box out to right side.
dbox10 ld a,(dispx)        ; x coordinate.
       inc a               ; down a line.
       ld (dispx),a        ; next position.
       jp dbox2            ; next line.
dbox7  ld a,b              ; chars remaining.
       and a               ; are any left?
       jr z,dbox11         ; no, nothing to draw.
       call dboxf          ; fill message to line.

; Drawn the box menu, now select option.

dbox11 ld a,(btop)         ; box top.
       ld (dispx),a        ; set bar position.
dbox14 call joykey         ; get controls.
       and 63              ; anything pressed?
       jr nz,dbox14        ; yes, debounce it.
       call dbar           ; draw bar.
dbox12 call joykey         ; get controls.
       and 19              ; anything pressed?
       jr z,dbox12         ; no, nothing.
       and 16              ; fire button pressed?
mod1   jp nz,fstd          ; yes, job done.
       call dbar           ; delete bar.
       ld a,(joyval)       ; joystick reading.
       and 1               ; going up?
       jr nz,dboxu         ; yes, go up.
       ld a,(dispx)        ; vertical position of bar.
       inc a               ; look down.
       ld hl,btop          ; top of box.
       sub (hl)            ; find distance from top.
       dec hl              ; point to height.
       cp (hl)             ; are we at end?
       jp z,dbox14         ; yes, go no further.
       ld hl,dispx         ; coordinate.
       inc (hl)            ; move bar.
       jr dbox14           ; continue.
dboxu  ld a,(dispx)        ; vertical position of bar.
       ld hl,btop          ; top of box.
       cp (hl)             ; are we at the top?
       jp z,dbox14         ; yes, go no further.
       ld hl,dispx         ; coordinate.
       dec (hl)            ; move bar.
       jr dbox14           ; continue.
fstd   ld a,(dispx)        ; bar position.
       ld hl,btop          ; top of menu.
       sub (hl)            ; find selected option.
       ld (varopt),a       ; store the option.
       jp redraw           ; redraw the screen.

; Option not available.  Skip this line.

dbox13 ld a,(hl)           ; get character.
       inc hl              ; next one.
       cp ','              ; another line?
       jp z,dbox2          ; yes, do next line.
       cp 13               ; another line?
       jp z,dbox2          ; yes, on to next line.
       and a               ; end of message?
       jp m,dbox11         ; yes, finish message.
       jr dbox13
dbox15 pop hl              ; pop message pointer from the stack.
       ret

dbar   ld a,(blft)         ; box left.
       ld (dispy),a        ; set display coordinate.
       call gprad          ; get printing address.
       ex de,hl            ; flip into hl register pair.

       ld b,8              ; 8 pixel rows to invert.
dbar1  push bc             ; store pixel count on stack.
       push hl             ; store address.
       ld a,(bwid)         ; box width.
       ld e,a              ; copy to e register.
       rlca                ; multiple of 2.
       rlca                ; multiple of 4.
       add a,e             ; multiple of 5.
       srl a               ; divide by 2.
       ld b,a              ; loop counter in b.
dbar0  ld a,(hl)           ; get screen byte.
       cpl                 ; reverse all bits.
       ld (hl),a           ; write back to screen.
       inc hl              ; next bar.
       djnz dbar0          ; repeat for whole line.
       pop hl              ; restore screen address.
       call nline2         ; look down one line.
       pop bc              ; restore row count.
       djnz dbar1          ; repeat for 8 pixel rows.
       ret

invdis push hl             ; store message text pointer.
       push de             ; store de pair for line count.
       ld hl,combyt        ; object number.
       ld a,(hl)           ; get object number.
       inc (hl)            ; ready for next one.
       call gotob          ; check if we have object.
       pop de              ; retrieve de pair from stack.
       pop hl              ; retrieve text pointer.
       ret
;always xor a               ; set zero flag.
;       ret

; Find option selected.

fopt   ld a,(dispx)
       ld hl,btop          ; top of menu.
       sub (hl)            ; find selected option.
       inc a               ; object 0 needs one iteration, 1 needs 2 and so on.
       ld b,a              ; option selected in b register.
       ld hl,combyt        ; object number.
       ld (hl),0           ; set to first item.
fopt0  push bc             ; store option counter in b register.
       call fobj           ; find next object in inventory.
       pop bc              ; restore option counter.
       djnz fopt0          ; repeat for relevant steps down the list.
       ld a,(combyt)       ; get option.
       dec a               ; one less, due to where we increment combyt.
       ld (varopt),a       ; store the option.
       jp redraw           ; redraw the screen.

fobj   ld hl,combyt        ; object number.
       ld a,(hl)           ; get object number.
       inc (hl)            ; ready for next item.
       ret z               ; in case we loop back to zero.
       call gotob          ; do we have this item?
       ret z               ; yes, it's on the list.
       jr fobj             ; repeat until we find next item in pockets.

bwid   defb 0              ; box/menu width.
blen   defb 0              ; box/menu height.
btop   defb 0              ; box coordinates.
blft   defb 0

; Wait for keypress.

prskey call debkey         ; debounce key.
prsky0 call vsync          ; vertical synch.
       call 47881          ; keyboard state.
       jr nc,prsky0        ; repeat until key pressed.

; Debounce keypress.

debkey call vsync          ; update scrolling, sounds etc.
       call 47881          ; read keyboard.
       jr c,debkey         ; key pressed - loop until key is released.
       ret

; Delay routine.

delay  push bc             ; store loop counter.
       call vsync          ; pause for a frame.
       call plsnd          ; time to play sound.
       pop bc              ; restore counter.
       dec b               ; one less iteration.
       ret z               ; done.
       push bc             ; store loop counter again.
       call vsync          ; pause for a frame.
       pop bc              ; restore counter.
       djnz delay          ; repeat.
       ret

; Clear sprite table.

xspr   ld hl,sprtab        ; sprite table.
       ld b,SPRBUF         ; length of table.
xspr0  ld (hl),255         ; clear one byte.
       inc hl              ; move to next byte.
       djnz xspr0          ; repeat for rest of table.
       ret

silenc call silen1         ; silence channel 1.
       call silen2         ; silence channel 2.
       call silen3         ; silence channel 3.
       jp plsnd            ; play all channels to switch them off.

; Initialise all objects.

iniob  ld ix,objdta        ; objects table.
       ld a,(numob)        ; number of objects in the game.
       ld b,a              ; loop counter.
       ld de,70            ; distance between objects.
iniob0 ld a,(ix+67)        ; start screen.
       ld (ix+64),a        ; set start screen.
       ld a,(ix+68)        ; find start x.
       ld (ix+65),a        ; set start x.
       ld a,(ix+69)        ; get initial y.
       ld (ix+66),a        ; set y coord.
       add ix,de           ; point to next object.
       djnz iniob0         ; repeat.
       ret

; Screen synchronisation.

vsync  call joykey         ; read joystick/keyboard.
       ld hl,framec        ; frame counter.
       ld a,(hl)           ; clock reading.
       inc (hl)            ; post-increment frame.
       rra                 ; rotate bit into carry.
       call c,vsync5       ; time to play sound and do shrapnel/ticker stuff.
       ld bc,clock         ; last clock reading.
vsync0 call 48397          ; get current clock.
       ld a,(bc)           ; previous clock.
       sub l               ; new reading.
       cp 251              ; difference of 6?
       jr nc,vsync0        ; no, wait until clock changes.
       ld a,l              ; clock reading in accumulator.
       ld (bc),a           ; set new clock reading.
       ret
vsync5 call plsnd          ; play sound.
       jp proshr           ; shrapnel and stuff.

sndtyp defb 0
framec defb 0              ; frame counter.
firmad defw 0

; Redraw the screen.

; Remove old copy of all sprites for redraw.

redraw push ix             ; place sprite pointer on stack.
       call droom          ; show screen layout.
       call shwob          ; draw objects.
numsp0 ld b,NUMSPR         ; sprites to draw.
       ld ix,sprtab        ; sprite table.
redrw0 ld a,(ix+0)         ; old sprite type.
       inc a               ; is it enabled?
       jr z,redrw1         ; no, find next one.
       ld a,(ix+3)         ; sprite x.
       cp 185              ; beyond maximum?
       jr nc,redrw1        ; yes, nothing to draw.
       push bc             ; store sprite counter.
       call sspria         ; show single sprite.
       pop bc              ; retrieve sprite counter.
redrw1 ld de,TABSIZ        ; distance to next odd/even entry.
       add ix,de           ; next sprite.
       djnz redrw0         ; repeat for remaining sprites.
rpblc1 call rbloc          ; redraw blocks if in adventure mode.
       call dshrp          ; redraw shrapnel.
       pop ix              ; retrieve sprite pointer.
       ret

; Clear screen routine.

;cls    jp 48148            ; clear screen.
cls    ld h,192            ; screen base is 256 * 192 = 49152.
       ld b,8              ; 8 rows per character.
cls0   push bc             ; store loop counter.
       call cls1           ; clear 25 lines.
       pop bc              ; restore loop counter.
       djnz cls0           ; repeat.
       ret
cls1   ld bc,1999          ; bytes to write.
       ld a,(papmsk)       ; get paper colour.
cls2   ld l,0              ; lines start at multiples of 256.
       ld d,h              ; copy hl to de.
       ld e,1              ; add 1 to de so it's pointing to next byte.
       ld (hl),a           ; write colour.
       ldir
       inc h               ; point hl at next block of lines.
       ret

; Set palette routine and data.
; Palette.
; 0  Black
; 1  Blue
; 2  Bright blue
; 3  Red
; 4  Magenta
; 5  Mauve
; 6  Bright red
; 7  Purple
; 8  Bright magenta
; 9  Green
; 10 Cyan
; 11 Sky blue
; 12 Yellow
; 13 White
; 14 Pastel blue
; 15 Orange
; 16 Pink
; 17 Pastel magenta
; 18 Bright green
; 19 Sea green
; 20 Bright cyan
; 21 Lime
; 22 Pastel green
; 23 Pastel cyan
; 24 Bright yellow
; 25 Pastel yellow
; 26 Bright white

setpal ld a,16             ; colours to set.
       ld hl,palett+15     ; palette.
setpa0 ld c,(hl)           ; get colour.
       dec hl              ; next in list.
       ld b,c              ; make sure it's in b and c.
       dec a
       push af             ; store pen number.
       push hl             ; store palette pointer.
       call 48178          ; set palette colour.
       pop hl              ; restore palette pointer.
       pop af              ; restore pen.
       jr nz,setpa0        ; repeat until all colours are set.
       ret

fdchk  ld a,(hl)           ; fetch cell.
       cp FODDER           ; is it fodder?
       ret nz              ; no.
       ld (hl),0           ; rewrite block type.
       push hl             ; store pointer to block.
       ld de,MAP           ; address of map.
       and a               ; clear carry flag for subtraction.
       sbc hl,de           ; find simple displacement for block.
       ld a,l              ; low byte is y coordinate.
       and 31              ; column position 0 - 31.
       ld (dispy),a        ; set up y position.
       add hl,hl           ; multiply displacement by 8.
       add hl,hl
       add hl,hl
       ld a,h              ; x coordinate now in h.
       ld (dispx),a        ; set the display coordinate.
       xor a               ; block to write.
       call pattr          ; write block.
       pop hl              ; restore block pointer.
       ret

; Routine to colour a sprite is redundant on CPC.

cspr   ret

; Scrolly text and puzzle variables.

txtbit defb 128            ; bit to write.
txtwid defb 16             ; width of ticker message.
txtpos defw msgdat
txtini defw msgdat
txtscr defw 16406

; Specialist routines.
; Process shrapnel.

proshr ld ix,SHRAPN        ; table.
       ld b,NUMSHR         ; shrapnel pieces to process.
       ld de,SHRSIZ        ; distance to next.
prosh0 ld a,(ix+0)         ; on/off marker.
       rla                 ; check its status.
proshx call nc,prosh1      ; on, so process it.
       add ix,de           ; point there.
       djnz prosh0         ; round again.
       jp scrly
prosh1 push bc             ; store counter.
       call plot           ; delete the pixel.
       ld a,(ix+0)         ; restore shrapnel type.
       ld hl,shrptr        ; shrapnel routine pointers.
       call prosh2         ; run the routine.
       call chkxy          ; check x and y are good before we redisplay.
       pop bc              ; restore counter.
       ld de,SHRSIZ        ; distance to next.
       ret
prosh2 rlca                ; 2 bytes per address.
       ld e,a              ; copy to de.
       add hl,de           ; point to address of routine.
       ld a,(hl)           ; get address low.
       inc hl              ; point to second byte.
       ld h,(hl)           ; fetch high byte from table.
       ld l,a              ; put low byte in l.
       jp (hl)             ; jump to routine.

shrptr defw laser          ; laser.
       defw trail          ; vapour trail.
       defw shrap          ; shrapnel from explosion.
       defw dotl           ; horizontal starfield left.
       defw dotr           ; horizontal starfield right.
       defw dotu           ; vertical starfield up.
       defw dotd           ; vertical starfield down.
       defw ptcusr         ; user particle.

; Explosion shrapnel.

shrap  ld e,(ix+1)         ; get the angle.
       ld d,0              ; no high byte.
       ld hl,shrsin        ; shrapnel sine table.
       add hl,de           ; point to sine.

       ld e,(hl)           ; fetch value from table.
       inc hl              ; next byte of table.
       ld d,(hl)           ; fetch value from table.
       inc hl              ; next byte of table.
       ld c,(hl)           ; fetch value from table.
       inc hl              ; next byte of table.
       ld b,(hl)           ; fetch value from table.
       ld l,(ix+2)         ; x coordinate in hl.
       ld h,(ix+3)
       add hl,de           ; add sine.
       ld (ix+2),l         ; store new coordinate.
       ld (ix+3),h
       ld l,(ix+4)         ; y coordinate in hl.
       ld h,(ix+5)
       add hl,bc           ; add cosine.
       ld (ix+4),l         ; store new coordinate.
       ld (ix+5),h
       ret

dotl   dec (ix+5)          ; move left.
       ret
dotr   inc (ix+5)          ; move left.
       ret
dotu   dec (ix+3)          ; move up.
       ret
dotd   inc (ix+3)          ; move down.
       ret

; Check coordinates are good before redrawing at new position.

chkxy  ld hl,wntopx        ; window top.
       ld a,(ix+3)         ; fetch shrapnel coordinate.
       cp (hl)             ; compare with top window limit.
       jr c,kilshr         ; out of window, kill shrapnel.
       inc hl              ; left edge.
       ld a,(ix+5)         ; fetch shrapnel coordinate.
       cp (hl)             ; compare with left window limit.
       jr c,kilshr         ; out of window, kill shrapnel.

       inc hl              ; point to bottom.
       ld a,(hl)           ; fetch window limit.
       add a,15            ; add height of sprite.
       cp (ix+3)           ; compare with shrapnel x coordinate.
       jr c,kilshr         ; off screen, kill shrapnel.
       inc hl              ; point to right edge.
       ld a,(hl)           ; fetch shrapnel y coordinate.
       add a,9             ; add width of sprite.
       cp (ix+5)           ; compare with window limit.
       jr c,kilshr         ; off screen, kill shrapnel.

; Drop through.
; Display shrapnel.

plot   ld l,(ix+3)         ; x integer.
       ld h,(ix+5)         ; y integer.
       ld (dispx),hl       ; workspace coordinates.
       ld a,(ix+0)         ; type.
       and a               ; is it a laser?
       jr z,plot2          ; yes, draw laser instead.
plot0  ld a,h              ; which pixel within byte do we
       rra                 ; want to set first?
       ld e,170            ; default to left pixel.
       jr nc,plot1         ; jump to set pixel code.
       ld e,85             ; draw right pixel instead.
plot1  call scadd          ; screen address.
       ld a,(hl)           ; see what's already there.
       xor e               ; merge with pixels.
       ld (hl),a           ; put back on screen.
       ret
plot2  ld a,h              ; get horizontal position.
       rra                 ; start at left or right pixel?
       push af             ; preserve carry.
       call scadd          ; screen address.
       pop af              ; restore carry.
       jr c,plot3          ; start at right pixel.
       ld a,(hl)           ; fetch byte there.
       cpl                 ; toggle 2 pixels.
       ld (hl),a           ; new byte.
       inc hl              ; next byte.
       ld a,(hl)           ; fetch byte there.
       cpl                 ; toggle 2 pixels.
       ld (hl),a           ; write the new byte.
       inc hl              ; next byte.
       ld a,(hl)           ; fetch byte there.
       xor 170             ; merge pixel.
       ld (hl),a           ; set new screen byte value.
       ret
plot3  ld a,(hl)           ; fetch byte there.
       xor 85              ; merge pixel.
       ld (hl),a           ; write new byte.
       inc hl              ; next byte.
       ld a,(hl)           ; fetch byte there.
       cpl                 ; toggle 2 pixels.
       ld (hl),a           ; write the new byte.
       inc hl              ; next byte.
       ld a,(hl)           ; fetch byte there.
       cpl                 ; toggle 2 pixels.
       ld (hl),a           ; write the new byte.
       ret

kilshr ld (ix+0),128       ; switch off shrapnel.
       ret

;explc  defb 0              ; explosion counter.

shrsin defw 0,1024,391,946,724,724,946,391
       defw 1024,0,946,65144,724,64811,391,64589
       defw 0,64512,65144,64589,64811,64811,64589,65144
       defw 64512,0,64589,391,64811,724,65144,946

trail  dec (ix+1)          ; time remaining.
       jp z,trailk         ; time to switch it off.
       call qrand          ; get a random number.
       rra                 ; x or y axis?
       jr c,trailv         ; use x.
       rra                 ; which direction?
       jr c,traill         ; go left.
       inc (ix+5)          ; go right.
       ret
traill dec (ix+5)          ; go left.
       ret
trailv rra                 ; which direction?
       jr c,trailu         ; go up.
       inc (ix+3)          ; go down.
       ret
trailu dec (ix+3)          ; go up.
       ret
trailk ld (ix+3),200       ; set off-screen to kill vapour trail.
       ret

laser  ld a,(ix+1)         ; direction.
       rra                 ; left or right?
       jr nc,laserl        ; move left.
       ld b,5              ; distance to travel.
       jr laserm           ; move laser.
laserl ld b,251            ; distance to travel.
laserm ld a,(ix+5)         ; y position.
       add a,b             ; add distance.
       ld (ix+5),a         ; set new y coordinate.

; Test new block.

       ld (dispy),a        ; set y for block collision detection purposes.
       ld a,(ix+3)         ; get x.
       ld (dispx),a        ; set coordinate for collision test.
       call tstbl          ; get block type there.
       cp WALL             ; is it solid?
       jr z,trailk         ; yes, it cannot pass.
       cp FODDER           ; is it fodder?
       ret nz              ; no, ignore it.
       call fdchk          ; remove fodder block.
       jr trailk           ; destroy laser.

; Plot, preserving de.

plotde push de             ; put de on stack.
       call plot           ; plot pixel.
       pop de              ; restore de from stack.
       ret

; Shoot a laser.

shoot  ld c,a              ; store direction in c register.
       ld a,(ix+8)         ; x coordinate.
shoot1 add a,7             ; down 7 pixels.
       ld l,a              ; put x coordinate in l.
       ld h,(ix+9)         ; y coordinate in h.
       push ix             ; store pointer to sprite.
       call fpslot         ; find particle slot.
       jr nc,vapou2        ; failed, restore ix.
       ld (ix+0),0         ; set up a laser.
       ld (ix+1),c         ; set the direction.
       ld (ix+3),l         ; set x coordinate.
       rr c                ; check direction we want.
       jr c,shootr         ; shoot right.
       ld a,h              ; y position.
;       dec a               ; left a pixel.
shoot0 and 248             ; align on character boundary.
       ld (ix+5),a         ; set y coordinate.
       jr vapou0           ; draw first image.
shootr ld a,h              ; y position.
       add a,9             ; look right.
       jr shoot0           ; align and continue.

; Create a bit of vapour trail.

vapour push ix             ; store pointer to sprite.
       ld l,(ix+8)         ; x coordinate.
       ld h,(ix+9)         ; y coordinate.
vapou3 ld de,5*256+7       ; mid-point of sprite.
       add hl,de           ; point to centre of sprite.
       call fpslot         ; find particle slot.
       jr c,vapou1         ; no, we can use it.
vapou2 pop ix              ; restore sprite pointer.
       ret                 ; out of slots, can't generate anything.

vapou1 ld (ix+3),l         ; set up x.
       ld (ix+5),h         ; set up y coordinate.
       call qrand          ; get quick random number.
       and 15              ; random time.
       add a,15            ; minimum time on screen.
       ld (ix+1),a         ; set time on screen.
       ld (ix+0),1         ; define particle as vapour trail.
vapou0 call chkxy          ; plot first position.
       jr vapou2

; Create a user particle.

ptusr  ld c,a              ; store timer.
       ld l,(ix+8)         ; x coordinate.
       ld h,(ix+9)         ; y coordinate.
       ld de,5*256+7       ; mid-point of sprite.
       add hl,de           ; point to centre of sprite.
       call fpslot         ; find particle slot.
       jr c,ptusr1         ; free slot.
       ret                 ; out of slots, can't generate anything.

ptusr1 ld (ix+3),l         ; set up x.
       ld (ix+5),h         ; set up y coordinate.
       ld a,c              ; restore timer.
       ld (ix+1),a         ; set time on screen.
       ld (ix+0),7         ; define particle as user particle.
       jp chkxy            ; plot first position.


; Create a vertical or horizontal star.

star   push ix             ; store pointer to sprite.
       call fpslot         ; find particle slot.
       jp c,star7          ; found one we can use.
star0  pop ix              ; restore sprite pointer.
       ret                 ; out of slots, can't generate anything.

star7  ld a,c              ; direction.
       and 3               ; is it left?
       jr z,star1          ; yes, it's horizontal.
       dec a               ; is it right?
       jr z,star2          ; yes, it's horizontal.
       dec a               ; is it up?
       jr z,star3          ; yes, it's vertical.

       ld a,(wntopx)       ; get edge of screen.
       inc a               ; down one pixel.
star8  ld (ix+3),a         ; set x coord.
       call qrand          ; get quick random number.
star9  ld (ix+5),a         ; set y position.
       ld a,c              ; direction.
       and 3               ; zero to three.
       add a,3             ; 3 to 6 for starfield.
       ld (ix+0),a         ; define particle as star.
       call chkxy          ; plot first position.
       jp star0
star1  call qrand          ; get quick random number.
       ld (ix+3),a         ; set x coord.
       ld a,(wnrgtx)       ; get edge of screen.
       add a,8             ; add width of sprite minus 1.
       jp star9
star2  call qrand          ; get quick random number.
       ld (ix+3),a         ; set x coord.
       ld a,(wnlftx)       ; get edge of screen.
       jp star9
star3  ld a,(wnbotx)       ; get edge of screen.
       add a,15            ; height of sprite minus one pixel.
       jp star8


; Find particle slot for lasers or vapour trail.
; Can't use alternate accumulator.

fpslot ld ix,SHRAPN        ; shrapnel table.
       ld de,SHRSIZ        ; size of each particle.
       ld b,NUMSHR         ; number of pieces in table.
fpslt0 ld a,(ix+0)         ; get type.
       rla                 ; is this slot in use?
       ret c               ; no, we can use it.
       add ix,de           ; point to more shrapnel.
       djnz fpslt0         ; repeat for all shrapnel.
       ret                 ; out of slots, can't generate anything.

; Create an explosion at sprite position.

explod ld c,a              ; particles to create.
       push ix             ; store pointer to sprite.
       ld l,(ix+8)         ; x coordinate.
       ld h,(ix+9)         ; y coordinate.
       ld ix,SHRAPN        ; shrapnel table.
       ld de,SHRSIZ        ; size of each particle.
       ld b,NUMSHR         ; number of pieces in table.
expld0 ld a,(ix+0)         ; get type.
       rla                 ; is this slot in use?
       jr c,expld1         ; no, we can use it.
expld2 add ix,de           ; point to more shrapnel.
       djnz expld0         ; repeat for all shrapnel.
expld3 pop ix              ; restore sprite pointer.
       ret                 ; out of slots, can't generate any more.
expld1 ld a,c              ; shrapnel counter.
       and 15              ; 0 to 15.
       add a,l             ; add to x.
       ld (ix+3),a         ; x coord.
       ld a,(seed3)        ; crap random number.
       and 7               ; 0 to 7.
       inc a               ; 1 to 8.
       add a,h             ; add to y.
       ld (ix+5),a         ; y coord.
       ld (ix+0),2         ; switch it on.
       push hl             ; store coordinates.
       call chkxy          ; plot first position.
       call qrand          ; quick random angle.
       and 60              ; keep within range.
       ld (ix+1),a         ; angle.
       pop hl              ; restore coordinates.
       ld de,SHRSIZ        ; restore size of each particle.
       dec c               ; one piece of shrapnel fewer to generate.
       jr nz,expld2        ; back to main explosion loop.
       jr expld3           ; restore sprite pointer and exit.
qrand  ld a,(seed3)        ; random seed.
       ld l,a              ; low byte.
       ld h,0              ; no high byte.
       ld a,r              ; r register.
       xor (hl)            ; combine with seed.
       ld (seed3),a        ; new seed.
       ret
seed3  defb 0

; Display all shrapnel.

dshrp  ld hl,plotde        ; display routine.
       ld (proshx+1),hl    ; modify routine.
       call proshr         ; process shrapnel.
       ld hl,prosh1        ; processing routine.
       ld (proshx+1),hl    ; modify the call.
       ret

; Particle engine.

inishr ld hl,SHRAPN        ; table.
       ld b,NUMSHR         ; shrapnel pieces to process.
       ld de,SHRSIZ        ; distance to next.
inish0 ld (hl),255         ; kill the shrapnel.
       add hl,de           ; point there.
       djnz inish0         ; round again.
       ret

; Check for collision between laser and sprite.

lcol   ld hl,SHRAPN        ; shrapnel table.
       ld de,SHRSIZ        ; size of each particle.
       ld b,NUMSHR         ; number of pieces in table.
lcol0  ld a,(hl)           ; get type.
       and a               ; is this slot a laser?
       jr z,lcol1          ; yes, check collision.
lcol3  add hl,de           ; point to more shrapnel.
       djnz lcol0          ; repeat for all shrapnel.
       ret                 ; no collision, carry not set.
lcol1  push hl             ; store pointer to laser.
       inc hl              ; direction.
       inc hl              ; not used.
       inc hl              ; x position.
       ld a,(hl)           ; get x.
       sub (ix+X)          ; subtract sprite x.
lcolh  cp 16               ; within range?
       jr nc,lcol2         ; no, missed.
       inc hl              ; not used.
       inc hl              ; y position.
       ld a,(hl)           ; get y.
       add a,4             ; add width of laser.
       sub (ix+Y)          ; subtract sprite y.
       cp 14               ; within range?
       jr c,lcol4          ; yes, collision occurred.
lcol2  pop hl              ; restore laser pointer from stack.
       jr lcol3
lcol4  pop hl              ; restore laser pointer.
       ret                 ; return with carry set for collision.

; Main game engine code starts here.

game   equ $

       call setpal         ; set up palette.
rpblc2 call inishr         ; initialise particle engine.
evintr call evnt12         ; call intro/menu event.

       ld hl,MAP           ; block properties.
       ld de,MAP+1         ; next byte.
       ld bc,799           ; size of properties map = 32 x 25.
       ld (hl),WALL        ; write default property.
       ldir
       call iniob          ; initialise objects.
       xor a               ; put zero in accumulator.
       ld (gamwon),a       ; reset game won flag.

       ld hl,score         ; score.
       call inisc          ; init the score.
mapst  ld a,(stmap)        ; start position on map.
       ld (roomtb),a       ; set up position in table, if there is one.
inipbl call ibloc          ; set up first screen.
       ld ix,ssprit        ; default to spare sprite in table.
evini  call evnt13         ; initialisation.

; Two restarts.
; First restart - clear all sprites and initialise everything.

rstrt  call rsevt          ; restart events.
       call xspr           ; clear sprite table.
       call sprlst         ; fetch pointer to screen sprites.
       call ispr           ; initialise sprite table.
       jr rstrt0

; Second restart - clear all but player, and don't initialise him.

rstrtn call rsevt          ; restart events.
       call nspr           ; clear all non-player sprites.
       call sprlst         ; fetch pointer to screen sprites.
       call kspr           ; initialise sprite table, no more players.


; Set up the player and/or enemy sprites.

rstrt0 xor a               ; zero in accumulator.
       ld (nexlev),a       ; reset next level flag.
       ld (restfl),a       ; reset restart flag.
       ld (deadf),a        ; reset dead flag.
       call droom          ; show screen layout.
rpblc0 call rbloc          ; redraw blocks if in adventure mode.
       call inishr         ; initialise particle engine.
       call shwob          ; draw objects.
       ld ix,sprtab        ; address of sprite table, even sprites.
       call dspr           ; display sprites.
       ld ix,sprtab+TABSIZ ; address of first odd sprite.
       call dspr           ; display sprites.

mloop  call vsync          ; synchronise with display.

       ld ix,sprtab        ; address of sprite table, even sprites.
       call dspr           ; display even sprites.

       call plsnd          ; play sounds.
       call vsync          ; synchronise with display.
       ld ix,sprtab+TABSIZ ; address of first odd sprite.
       call dspr           ; display odd sprites.
       ld ix,ssprit        ; point to spare sprite for spawning purposes.
evlp1  call evnt10         ; called once per main loop.
       call pspr           ; process sprites.

; Main loop events.

       ld ix,ssprit        ; point to spare sprite for spawning purposes.
evlp2  call evnt11         ; called once per main loop.

bsortx call bsort          ; sort sprites.
       ld a,(nexlev)       ; finished level flag.
       and a               ; has it been set?
       jr nz,newlev        ; yes, go to next level.
       ld a,(gamwon)       ; finished game flag.
       and a               ; has it been set?
       jr nz,evwon         ; yes, finish the game.
       ld a,(restfl)       ; finished level flag.
       dec a               ; has it been set?
       jr z,rstrt          ; yes, go to next level.
       dec a               ; has it been set?
       jr z,rstrtn         ; yes, go to next level.

       ld a,(deadf)        ; dead flag.
       and a               ; is it non-zero?
       jr nz,pdead         ; yes, player dead.

       ld hl,frmno         ; game frame.
       inc (hl)            ; advance the frame.

; Back to start of main loop.

qoff   jp mloop            ; switched to a jp nz,mloop during test mode.
       ret
newlev ld a,(scno)         ; current screen.
       ld hl,numsc         ; total number of screens.
       inc a               ; next screen.
       cp (hl)             ; reached the limit?
       jr nc,evwon         ; yes, game finished.
       ld (scno),a         ; set new level number.
       jp rstrt            ; restart, clearing all aliens.
evwon  call evnt18         ; game completed.
       jp tidyup           ; tidy up and return to BASIC/calling routine.

; Player dead.

pdead  xor a               ; zeroise accumulator.
       ld (deadf),a        ; reset dead flag.
evdie  call evnt16         ; death subroutine.
       ld a,(numlif)       ; number of lives.
       and a               ; reached zero yet?
       jp nz,rstrt         ; restart game.
evfail call evnt17         ; failure event.
tidyup ld hl,hiscor        ; high score.
       ld de,score         ; player's score.
       ld b,6              ; digits to check.
tidyu2 ld a,(de)           ; get score digit.
       cp (hl)             ; are we larger than high score digit?
       jr c,tidyu0         ; high score is bigger.
       jr nz,tidyu1        ; score is greater, record new high score.
       inc hl              ; next digit of high score.
       inc de              ; next digit of score.
       djnz tidyu2         ; repeat for all digits.
tidyu0 call 47875          ; reset key manager, empty buffer.
       ld hl,(firmad)      ; firmware interrupt address.
       ld (57),hl          ; restore interrupts.
       ld hl,score         ; return pointing to score so programmer can store high-score.
       ret
tidyu1 ld hl,score         ; score.
       ld de,hiscor        ; high score.
       ld bc,6             ; digits to copy.
       ldir                ; copy score to high score.
evnewh call evnt19         ; new high score event.
       jr tidyu0           ; tidy up.

; Restart event.

rsevt  ld ix,ssprit        ; default to spare element in table.
evrs   jp evnt14           ; call restart event.

; Copy number passed in a to string position bc, right-justified.

num2ch ld l,a              ; put accumulator in l.
       ld h,0              ; blank high byte of hl.
       ld a,32             ; leading spaces.
numdg3 ld de,100           ; hundreds column.
       call numdg          ; show digit.
numdg2 ld de,10            ; tens column.
       call numdg          ; show digit.
       or 16               ; last digit is always shown.
       ld de,1             ; units column.
numdg  and 48              ; clear carry, clear digit.
numdg1 sbc hl,de           ; subtract from column.
       jr c,numdg0         ; nothing to show.
       or 16               ; something to show, make it a digit.
       inc a               ; increment digit.
       jr numdg1           ; repeat until column is zero.
numdg0 add hl,de           ; restore total.
       cp 32               ; leading space?
       ret z               ; yes, don't write that.
       ld (bc),a           ; write digit to buffer.
       inc bc              ; next buffer position.
       ret
num2dd ld l,a              ; put accumulator in l.
       ld h,0              ; blank high byte of hl.
       ld a,32             ; leading spaces.
       ld de,100           ; hundreds column.
       call numdg          ; show digit.
       or 16               ; second digit is always shown.
       jr numdg2
num2td ld l,a              ; put accumulator in l.
       ld h,0              ; blank high byte of hl.
       ld a,48             ; leading spaces.
       jr numdg3


inisc  ld b,6              ; digits to initialise.
inisc0 ld (hl),'0'         ; write zero digit.
       inc hl              ; next column.
       djnz inisc0         ; repeat for all digits.
       ret


; Multiply h by d and return in hl.

imul   ld e,d              ; HL = H * D
       ld c,h              ; make c first multiplier.
imul0  ld hl,0             ; zeroise total.
       ld d,h              ; zeroise high byte.
       ld b,8              ; repeat 8 times.
imul1  rr c                ; rotate rightmost bit into carry.
       jr nc,imul2         ; wasn't set.
       add hl,de           ; bit was set, so add de.
       and a               ; reset carry.
imul2  rl e                ; shift de 1 bit left.
       rl d
       djnz imul1          ; repeat 8 times.
       ret

; Divide d by e and return in d, remainder in a.

idiv   xor a
       ld b,8              ; bits to shift.
idiv0  sla d               ; multiply d by 2.
       rla                 ; shift carry into remainder.
       cp e                ; test if e is smaller.
       jr c,idiv1          ; e is greater, no division this time.
       sub e               ; subtract it.
       inc d               ; rotate into d.
idiv1  djnz idiv0
       ret

; Initialise a sound.

isnd   ld de,(ch1ptr)      ; first pointer.
       ld a,(de)           ; get first byte.
       inc a               ; reached the end?
       jr z,isnd1          ; that'll do.
       ld de,(ch2ptr)      ; second pointer.
       ld a,(de)           ; get first byte.
       inc a               ; reached the end?
       jr z,isnd2          ; that'll do.
       ld de,(ch3ptr)      ; final pointer.
       ld a,(de)           ; get first byte.
       inc a               ; reached the end?
       jr z,isnd3          ; that'll do.
       ret
isnd1  ld (ch1ptr),hl      ; set up the sound.
       ret
isnd2  ld (ch2ptr),hl      ; set up the sound.
       ret
isnd3  ld (ch3ptr),hl      ; set up the sound.
       ret


ch1ptr defw nosnd
ch2ptr defw nosnd
ch3ptr defw nosnd

; Write the contents of our AY buffer to the AY registers.

w8912  ld hl,snddat        ; start of AY-3-8912 register data.
       ld de,14*256        ; start with register 0, 14 to write.
       ld c,253            ; low byte of port to write.
w8912a ld b,255            ; port 65533=select soundchip register.
       ld a,(hl)           ; value to write.
       ld c,e              ; register to write.

       ld b,244            ; setup PSG register number on PPI port A.
       out (c),c           ;
       ld bc,63168         ; tell PSG to select register from data on PPI port A.
       out (c),c           ;
       ld bc,62976         ; put PSG into inactive state.
       out (c),c           ;
       ld b,244            ; setup register data on PPI port A.
       out (c),a           ;
       ld bc,63104         ; tell PSG to write data on PPI port A into selected register.
       out (c),c           ;
       ld bc,62976         ; put PSG into inactive state.
       out (c),c           ;

       inc e               ; next sound chip register.
       inc hl              ; next byte to write.
       dec d               ; decrement loop counter.
       jp nz,w8912a        ; repeat until done.
       ret

snddat defw 0              ; tone registers, channel A.
       defw 0              ; channel B tone registers.
       defw 0              ; as above, channel C.
sndwnp defb 0              ; white noise period.
sndmix defb 60             ; tone/noise mixer control.
sndv1  defb 0              ; channel A amplitude/envelope generator.
sndv2  defb 0              ; channel B amplitude/envelope.
sndv3  defb 0              ; channel C amplitude/envelope.
       defw 0              ; duration of each note.
       defb 0

plwn   inc hl              ; next byte of sound.
       and 56              ; check if we're bothering with white noise.
       ret nz              ; we're not.
       ld a,(hl)           ; fetch byte.
       ld (sndwnp),a       ; set white noise period.
       ret


plsnd2 call cksnd2         ; check sound for first channel.
       cp 255              ; reached end?
       jr z,silen2         ; silence this channel.
       and 15              ; sound bits.
       ld (sndv2),a        ; set volume for channel.
       ld a,(sndmix)       ; mixer byte.
       and 237             ; remove bits for this channel.
       ld b,a              ; store in b register.
       call plmix          ; fetch mixer details.
       and 18              ; mixer bits we want.
       or b                ; combine with mixer bits.
       ld (sndmix),a       ; new mixer value.
       call plwn           ; white noise check.
       inc hl              ; tone low.
       ld e,(hl)           ; fetch value.
       inc hl              ; tone high.
       ld d,(hl)           ; fetch value.
       ld (snddat+2),de    ; set tone.
       inc hl              ; next bit of sound.
       ld (ch2ptr),hl      ; set pointer.
       ret

plsnd3 call cksnd3         ; check sound for first channel.
       cp 255              ; reached end?
       jr z,silen3         ; silence last channel.
       and 15              ; sound bits.
       ld (sndv3),a        ; set volume for channel.
       ld a,(sndmix)       ; mixer byte.
       and 219             ; remove bits for this channel.
       ld b,a              ; store in b register.
       call plmix          ; fetch mixer details.
       and 36              ; mixer bits we want.
       or b                ; combine with mixer bits.
       ld (sndmix),a       ; new mixer value.
       call plwn           ; white noise check.
       inc hl              ; tone low.
       ld e,(hl)           ; fetch value.
       inc hl              ; tone high.
       ld d,(hl)           ; fetch value.
       ld (snddat+4),de    ; set tone.
       inc hl              ; next bit of sound.
       ld (ch3ptr),hl      ; set pointer.
       ret

plmix  ld a,(hl)           ; fetch mixer byte.
       and 192             ; mix bits are d6 and d7.
       rlca                ; rotate into d0 and d1.
       rlca
       ld e,a              ; displacement in de.
       ld d,0
       push hl             ; store pointer on stack.
       ld hl,mixtab        ; mixer table.
       add hl,de           ; point to mixer byte.
       ld a,(hl)           ; fetch mixer value.
       pop hl              ; restore pointer.
       ret
mixtab defb 63,56,7,0      ; mixer byte settings.

silen1 xor a               ; zero.
       ld (sndv1),a        ; sound off.
       ld a,(sndmix)       ; mixer byte.
       or 9                ; mix bits off.
       ld (sndmix),a       ; mixer setting for channel.
       ret
silen2 xor a               ; zero.
       ld (sndv2),a        ; sound off.
       ld a,(sndmix)       ; mixer byte.
       or 18               ; mix bits off.
       ld (sndmix),a       ; mixer setting for channel.
       ret
silen3 xor a               ; zero.
       ld (sndv3),a        ; sound off.
       ld a,(sndmix)       ; mixer byte.
       or 36               ; mix bits off.
       ld (sndmix),a       ; mixer setting for channel.
       ret
cksnd1 ld hl,(ch1ptr)      ; pointer to sound.
       ld a,(hl)           ; fetch mixer/flag.
       ret
cksnd2 ld hl,(ch2ptr)      ; pointer to sound.
       ld a,(hl)           ; fetch mixer/flag.
       ret
cksnd3 ld hl,(ch3ptr)      ; pointer to sound.
       ld a,(hl)           ; fetch mixer/flag.
       ret

plsnd1 call cksnd1         ; check sound for first channel.
       cp 255              ; reached end?
       jr z,silen1         ; silence first channel.
       and 15              ; sound bits.
       ld (sndv1),a        ; set volume for channel.
       ld a,(sndmix)       ; mixer byte.
       and 246             ; remove bits for this channel.
       ld b,a              ; store in b register.
       call plmix          ; fetch mixer details.
       and 9               ; mixer bits we want.
       or b                ; combine with mixer bits.
       ld (sndmix),a       ; new mixer value.
       call plwn           ; white noise check.
       inc hl              ; tone low.
       ld e,(hl)           ; fetch value.
       inc hl              ; tone high.
       ld d,(hl)           ; fetch value.
       ld (snddat),de      ; set tone.
       inc hl              ; next bit of sound.
       ld (ch1ptr),hl      ; set pointer.
       ret


; Objects handling.
; 32 bytes for image
; 1 for colour
; 3 for room, x and y
; 3 for starting room, x and y.
; 254 = disabled.
; 255 = object in player's pockets.

; Show items present.

shwob  ld hl,objdta        ; objects table.
       ld de,64            ; distance to room number.
       add hl,de           ; point to room data.
       ld a,(numob)        ; number of objects in the game.
       ld b,a              ; loop counter.
shwob0 push bc             ; store count.
       push hl             ; store item pointer.
       ld a,(scno)         ; current location.
       cp (hl)             ; same as an item?
       call z,dobj         ; yes, display object.
       pop hl              ; restore pointer.
       pop bc              ; restore counter.
       ld de,70            ; distance to next item.
       add hl,de           ; point to it.
       djnz shwob0         ; repeat for others.
       ret

; Display object.
; hl must point to object's room number.

dobj   inc hl              ; point to x.
dobj0  ld de,dispx         ; coordinates.
       ldi                 ; transfer x coord.
       ldi                 ; transfer y too.
       ld de,65469         ; minus 67.
       add hl,de           ; point to image.
dobj1  jp sprite           ; draw this sprite.

; Remove an object.

remob  ld hl,numob         ; number of objects in game.
       cp (hl)             ; are we checking past the end?
       ret nc              ; yes, can't get non-existent item.
       push af             ; remember object.
       call getob          ; pick it up if we haven't already got it.
       pop af              ; retrieve object number.
       call gotob          ; get its address.
       ld (hl),254         ; remove it.
       ret

; Pick up object number held in the accumulator.

getob  ld hl,numob         ; number of objects in game.
       cp (hl)             ; are we checking past the end?
       ret nc              ; yes, can't get non-existent item.
       call gotob          ; check if we already have it.
       ret z               ; we already do.
       ex de,hl            ; object address in de.
       ld hl,scno          ; current screen.
       cp (hl)             ; is it on this screen?
       ex de,hl            ; object address back in hl.
       jr nz,getob0        ; not on screen, so nothing to delete.
       ld (hl),255         ; pick it up.
       inc hl              ; point to x coord.
getob1 ld e,(hl)           ; x coord.
       inc hl              ; back to y coord.
       ld d,(hl)           ; y coord.
       ld (dispx),de       ; set display coords.
       ld de,65470         ; minus graphic size.
       add hl,de           ; point to graphics.
       jp dobj1            ; delete object sprite.
getob0 ld (hl),255         ; pick it up.
       ret

; Got object check.
; Call with object in accumulator, returns zero set if in pockets.

gotob  ld hl,numob         ; number of objects in game.
       cp (hl)             ; are we checking past the end?
       jr nc,gotob0        ; yes, we can't have a non-existent object.
       call findob         ; find the object.
gotob1 cp 255              ; in pockets?
       ret
gotob0 ld a,254            ; missing.
       jr gotob1

findob ld hl,objdta        ; objects.
       ld de,70            ; size of each object.
       and a               ; is it zero?
       jr z,fndob1         ; yes, skip loop.
       ld b,a              ; loop counter in b.
fndob2 add hl,de           ; point to next one.
       djnz fndob2         ; repeat until we find address.
fndob1 ld e,64             ; distance to room it's in.
       add hl,de           ; point to room.
       ld a,(hl)           ; fetch status.
       ret

; Drop object number at (dispx, dispy).

drpob  ld hl,numob         ; number of objects in game.
       cp (hl)             ; are we checking past the end?
       ret nc              ; yes, can't drop non-existent item.
       call gotob          ; make sure object is in inventory.
       ld a,(scno)         ; screen number.
       cp (hl)             ; already on this screen?
       ret z               ; yes, nothing to do.
       ld (hl),a           ; bring onto screen.
       inc hl              ; point to x coord.
       ld a,(dispx)        ; sprite x coordinate.
       ld (hl),a           ; set x coord.
       inc hl              ; point to object y.
       ld a,(dispy)        ; sprite y coordinate.
       ld (hl),a           ; set the y position.
       ld de,65470         ; minus graphic size.
       add hl,de           ; point to graphics.
       jp dobj1            ; delete object sprite.

; Seek objects at sprite position.

skobj  ld hl,objdta        ; pointer to objects.
       ld de,64            ; distance to room number.
       add hl,de           ; point to room data.
       ld de,70            ; size of each object.
       ld a,(numob)        ; number of objects in game.
       ld b,a              ; set up the loop counter.
skobj0 ld a,(scno)         ; current room number.
       cp (hl)             ; is object in here?
       call z,skobj1       ; yes, check coordinates.
       add hl,de           ; point to next object in table.
       djnz skobj0         ; repeat for all objects.
       ld a,255            ; end of list and nothing found, return 255.
       ret
skobj1 inc hl              ; point to x coordinate.
       ld a,(hl)           ; get coordinate.
       sub (ix+8)          ; subtract sprite x.
       add a,15            ; add sprite height minus one.
       cp 31               ; within range?
       jp nc,skobj2        ; no, ignore object.
       inc hl              ; point to y coordinate now.
       ld a,(hl)           ; get coordinate.
       sub (ix+9)          ; subtract the sprite y.
       add a,8             ; add sprite width minus one.
       cp 18               ; within range?
       jp nc,skobj3        ; no, ignore object.
       pop de              ; remove return address from stack.
       ld a,(numob)        ; objects in game.
       sub b               ; subtract loop counter.
       ret                 ; accumulator now points to object.
skobj3 dec hl              ; back to y position.
skobj2 dec hl              ; back to room.
       ret


; Spawn a new sprite.

spawn  ld hl,sprtab        ; sprite table.
       push bc             ; store parameters on stack.
       ld b,NUMSPR         ; number of sprites.
       ld de,TABSIZ        ; size of each entry.
spaw0  ld a,(hl)           ; get sprite type.
       inc a               ; is it an unused slot?
       jr z,spaw1          ; yes, we can use this one.
       add hl,de           ; point to next sprite in table.
       djnz spaw0          ; keep going until we find a slot.

; Didn't find one but drop through and set up a dummy sprite instead.

spaw1  pop bc              ; take image and type back off the stack.
       push ix             ; existing sprite address on stack.
       ld (spptr),hl       ; store spawned sprite address.
       ld (hl),c           ; set the type.
       inc hl              ; point to image.
       ld (hl),b           ; set the image.
       inc hl              ; next byte.
       ld (hl),0           ; frame zero.
       inc hl              ; next byte.
       ld a,(ix+X)         ; x coordinate.
       ld (hl),a           ; set sprite coordinate.
       inc hl              ; next byte.
       ld a,(ix+Y)         ; y coordinate.
       ld (hl),a           ; set sprite coordinate.
       inc hl              ; next byte.
       ex de,hl            ; swap address into de.
       ld hl,(spptr)       ; restore address of details.
       ld bc,5             ; number of bytes to duplicate.
       ldir                ; copy first version to new version.
       ex de,hl            ; swap address into de.
       ld a,(ix+10)        ; direction of original.
       ld (hl),a           ; set the direction.
       inc hl              ; next byte.
       ld (hl),b           ; reset parameter.
       inc hl              ; next byte.
       ld (hl),b           ; reset parameter.
       inc hl              ; next byte.
       ld (hl),b           ; reset parameter.
       inc hl              ; next byte.
       ld (hl),b           ; reset parameter.
rtssp  ld ix,(spptr)       ; address of new sprite.
evis1  call evnt09         ; call sprite initialisation event.
       ld ix,(spptr)       ; address of new sprite.
       call sspria         ; display the new sprite.
       pop ix              ; address of original sprite.
       ret

spptr  defw 0              ; spawned sprite pointer.
seed   defb 0              ; seed for random numbers.
seed2  defb 0              ; second seed.
score  defb '000000'       ; player's score.
hiscor defb '000000'       ; high score.
bonus  defb '000000'       ; bonus.

checkx ld a,e              ; x position.
       cp 24               ; off screen?
       ret c               ; no, it's okay.
       pop hl              ; remove return address from stack.
       ret

; Displays the current score.

dscor  call preprt         ; set up font and print position.
       call checkx         ; make sure we're in a printable range.
       ld a,(prtmod)       ; get print mode.
       and a               ; standard size text?
       jp nz,bscor0        ; no, show double-height.
dscor0 push bc             ; place counter onto the stack.
       push hl
       ld a,(hl)           ; fetch character.
       call ptxta          ; display character and advance one.
       pop hl
       inc hl              ; next score column.
       pop bc              ; retrieve character counter.
       djnz dscor0         ; repeat for all digits.
dscor2 ld hl,(dispx)       ; general coordinates.
       ld (charx),hl       ; set up display coordinates.
       ret

; Displays the current score in double-height characters.

bscor0 push bc             ; place counter onto the stack.
       push hl
       ld a,(hl)           ; fetch character.
       call btxt           ; display big char.
       pop hl
       inc hl              ; next score column.
       pop bc              ; retrieve character counter.
       djnz bscor0         ; repeat for all digits.
       jp dscor2           ; tidy up line and column variables.

; Adds number in the hl pair to the score.

addsc  ld de,score+1       ; ten thousands column.
       ld bc,10000         ; amount to add each time.
       call incsc          ; add to score.
       inc de              ; thousands column.
       ld bc,1000          ; amount to add each time.
       call incsc          ; add to score.
       inc de              ; hundreds column.
       ld bc,100           ; amount to add each time.
       call incsc          ; add to score.
       inc de              ; tens column.
       ld bc,10            ; amount to add each time.
       call incsc          ; add to score.
       inc de              ; units column.
       ld bc,1             ; units.
incsc  push hl             ; store amount to add.
       and a               ; clear the carry flag.
       sbc hl,bc           ; subtract from amount to add.
       jr c,incsc0         ; too much, restore value.
       pop af              ; delete the previous amount from the stack.
       push de             ; store column position.
       call incsc2         ; do the increment.
       pop de              ; restore column.
       jp incsc            ; repeat until all added.
incsc0 pop hl              ; restore previous value.
       ret
incsc2 ld a,(de)           ; get amount.
       inc a               ; add one to column.
       ld (de),a           ; write new column total.
       cp '9'+1            ; gone beyond range of digits?
       ret c               ; no, carry on.
       ld a,'0'            ; mae it zero.
       ld (de),a           ; write new column total.
       dec de              ; back one column.
       jr incsc2

; Add bonus to score.

addbo  ld de,score+5       ; last score digit.
       ld hl,bonus+5       ; last bonus digit.
       and a               ; clear carry.
       ld bc,6*256+48      ; 6 digits to add, ASCII '0' in c.
addbo0 ld a,(de)           ; get score.
       adc a,(hl)          ; add bonus.
       sub c               ; 0 to 18.
       ld (hl),c           ; zeroise bonus.
       dec hl              ; next bonus.
       cp 58               ; carried?
       jr c,addbo1         ; no, do next one.
       sub 10              ; subtract 10.
addbo1 ld (de),a           ; write new score.
       dec de              ; next score digit.
       ccf                 ; set carry for next digit.
       djnz addbo0         ; repeat for all 6 digits.
       ret

; Swap score and bonus.

swpsb  ld de,score         ; first score digit.
       ld hl,bonus         ; first bonus digit.
       ld b,6              ; digits to add.
swpsb0 ld a,(de)           ; get score and bonus digits.
       ld c,(hl)
       ex de,hl            ; swap pointers.
       ld (hl),c           ; write bonus and score digits.
       ld (de),a
       inc hl              ; next score and bonus.
       inc de
       djnz swpsb0         ; repeat for all 6 digits.
       ret

; Get print address.
; Returns screen address in de.

gprad  push hl             ; store hl pair.
       ld a,(dispx)        ; get vertical position.
       rlca                ; multiply by 8, max is 192.
       rlca
       rlca
       ld l,a              ; put into hl.
       ld h,0              ; still 8 bits, no high byte.
       add hl,hl           ; hl is now coordinate * 16.
       ld d,h              ; make de 16 * coordinate also.
       ld e,l
       add hl,hl           ; hl is coordinate * 32.
       add hl,hl           ; hl is coordinate * 64.
       add hl,de           ; now it's coordinate * 80.
       ld de,49152         ; screen base address.
       add hl,de           ; add x displacement.

       ld a,(dispy)        ; y coordinate.
       ld d,0              ; no displacement for odd position.
       rra                 ; shift odd char position into carry.
       jr nc,gprad0        ; at even position.
       ld d,2              ; odd position is two bytes along.
gprad0 and 15              ; character position / 2.
       ld e,a              ; copy to e.
       rlca                ; multiple of 2.
       rlca                ; multiple of 4.
       add a,e             ; multiple of 5.
       add a,d             ; add on any odd displacement.
       ld e,a              ; copy to de.
       ld d,0              ; high byte of displacement is zero.
       add hl,de           ; point to even char position.
       ex de,hl            ; flip result into de.
       pop hl              ; retrieve hl.
       ret

; Get property buffer address of char at (dispx, dispy) in hl.

pradd  ld a,(dispx)        ; x coordinate.
       rrca                ; multiply by 32.
       rrca
       rrca
       ld l,a              ; store shift in l.
       and 3               ; high byte bits.
       add a,MAPHI         ; 160 * 256 = 40960, start of properties map.
       ld h,a              ; that's our high byte.
       ld a,l              ; restore shift result.
       and 224             ; only want low bits.
       ld l,a              ; put into low byte.
       ld a,(dispy)        ; fetch y coordinate.
       and 31              ; should be in range 0 - 31.
       add a,l             ; add to low byte.
       ld l,a              ; new low byte.
       ret

; Display character block on screen.

pchar  call gchadd         ; get character address.
pchark call gprad          ; get screen address.
       ld a,(dispy)        ; get horizontal position.
       rra                 ; are we displaying block at an odd position?
       jp c,pchar0         ; yes, need to shift everything then.
       call pchare         ; even pixel position.
       call pchare
       call pchare
       call pchare
       call pchare
       call pchare
       call pchare

pchare ldi                 ; transfer byte.
       ldi                 ; transfer byte.
       ld a,(de)           ; get data on screen.
       and 85              ; lose the left pixel colours.
       or (hl)             ; merge in third byte of data.
       inc hl              ; postincrement to next byte of data.
       ld (de),a           ; write third byte.
       ex de,hl            ; flip source and target.
       ld bc,2046          ; distance to next line.
       add hl,bc           ; look down one pixel.
       ex de,hl            ; source and target back again.
       ret

pchar0 ex de,hl            ; screen in hl, block image in de.
       call pcharo         ; odd pixel position.
       call pcharo
       call pcharo
       call pcharo
       call pcharo
       call pcharo
       call pcharo

pcharo ld a,(hl)           ; get data from screen.
       and 170             ; lose the right pixel colours.
       ld b,a              ; store result.
       ld a,(de)           ; get first byte of data to write.
       rra                 ; shift into right position.
       and 85              ; get right pixel colour.
       or b                ; merge onto screen.
       ld (hl),a           ; write that byte.

       inc hl              ; next screen byte.
       ld a,(de)           ; get first byte of image again.
       rla                 ; shift second pixel into left position.
       and 170             ; remove right bits.
       ld b,a              ; store for now.
       inc de              ; point to next byte of data.
       ld a,(de)           ; get third pixel.
       rra                 ; shift into right pixel position.
       and 85              ; remove left pixel.
       or b                ; merge with previous pixel.
       ld (hl),a           ; write to screen.
       inc hl              ; third screen byte.
       ld a,(de)           ; get second byte of image again.
       rla                 ; shift fourth pixel into left position.
       and 170             ; remove right bits.
       ld b,a              ; store for now.
       inc de              ; point to next byte of data.
       ld a,(de)           ; get fifth pixel.
       rra                 ; shift into right pixel position.
       and 85              ; remove left pixel.
       or b                ; merge with previous pixel.
       ld (hl),a           ; write to screen.
       inc de              ; point to next byte of data.

       ld bc,2046          ; distance to next line.
       add hl,bc           ; look down one pixel.
       ret

; Get character address.

gchadd rlca                ; multiply char by 8.
       rlca
       rlca
       ld e,a              ; store shift in e.
       and 7               ; only want high byte bits.
       ld d,a              ; store in d.
       ld a,e              ; restore shifted value.
       and 248             ; only want low byte bits.
       ld e,a              ; that's the low byte.
       ld hl,chgfx         ; address of graphics.
       add hl,de           ; 8 * block number.
       add hl,de           ; 16 * block number.
       add hl,de           ; 24 * block number.
       ret

; Print property attributes and pixels.

colpat defb 0

pattr  ld b,a              ; store cell in b register for now.
       ld e,a              ; displacement in e.
       ld d,0              ; no high byte.
       ld hl,(proptr)      ; pointer to properties.
       add hl,de           ; property cell address.
       ld c,(hl)           ; fetch byte.
       ld a,c              ; put into accumulator.
       cp COLECT           ; is it a collectable?
       jp nz,pattr1        ; no, carry on as normal.
       ld a,b              ; restore cell.
       ld (colpat),a       ; store collectable block.
pattr1 call pradd          ; get property buffer address.
       ld (hl),c           ; write property.
       ld a,b              ; get block number.

; Print character pixels, no more.

pchr   call pchar          ; show character in accumulator.
       ld hl,dispy         ; y coordinate.
       inc (hl)            ; move along one.
       ret

; Convert ink colour 0 - 15 to bitmask, 0 - 255.

conink ld c,0              ; clear destination.
       rra                 ; test lowest bit of colour.
       rl c                ; shift carry into colour.
       rl c                ; rotate again.
       rrca                ; get bit d1 out of the way.
       rra                 ; test bit d2 of colour.
       rl c                ; shift carry into mask colour.
       rl c                ; rotate again.
       rla                 ; shift original d1 bit back to d7.
       rla                 ; rotate into carry.
       rl c                ; shift carry into colour.
       rl c                ; rotate again.
       rra                 ; put original bit d3 in d0.
       rra
       rra                 ; shift into carry.
       rl c                ; shift carry into colour.
       ld a,c              ; copy colour to accumulator.
       rla                 ; create copy for higher bit of the pair.
       or c                ; merge colours for both pixels.
       ret

; Ink and paper masks.  Don't move these around, keep together in this order.

inkmsk defb 255            ; ink mask.
papmsk defb 0              ; paper mask.

; Print text, standard 5x8 pixel size.

ptxt   rlca                ; multiply char by 8.
       rlca
       rlca
       ld e,a              ; store shift in e.
       and 7               ; only want high byte bits.
       ld d,a              ; store in d.
       ld a,e              ; restore shifted value.
       and 248             ; only want low byte bits.
       ld e,a              ; that's the low byte.
       ld hl,font-256      ; address of charset.
       add hl,de           ; point to character.
       call gprad          ; get screen address in de.
       ex de,hl            ; screen address in hl, font in de.

       ld b,8              ; height.
ptxt0  push bc             ; store row counter.

       ld bc,(inkmsk)      ; get ink and paper colours.
       ld a,(dispy)        ; get y coordinate.
       rra                 ; odd or even alignment?
       ld a,(de)           ; get byte of image.
       push de             ; store pointer to font data.
       jr c,ptxt2          ; odd alignment.
       call ptxte          ; print at even cell position.
       jp ptxt1
ptxt2  call ptxto          ; print at odd cell position.
ptxt1  dec hl              ; back to left edge of character.
       dec hl
       call nline2         ; down one line.

       pop de              ; restore text image address.
       inc de              ; next byte.
       pop bc              ; retrieve row counter.
       djnz ptxt0          ; repeat for all rows.
       ret
ptxta  call ptxt           ; show character in accumulator.
       ld hl,dispy         ; y coordinate.
       inc (hl)            ; move along one.
       res 5,(hl)          ; don't spill over line.
       ret


; Print big text, stretched to 5x16 pixels.

btxt   rlca                ; multiply char by 8.
       rlca
       rlca
       ld e,a              ; store shift in e.
       and 7               ; only want high byte bits.
       ld d,a              ; store in d.
       ld a,e              ; restore shifted value.
       and 248             ; only want low byte bits.
       ld e,a              ; that's the low byte.
       ld hl,font-256      ; address of charset.
       add hl,de           ; point to character.
       call gprad          ; get screen address in de.
       ex de,hl            ; screen address in hl, font in de.

       ld b,8              ; pixel rows.
btxt0  push bc             ; store row counter.
       ld b,2              ; vertical stretch factor.
btxt3  push bc             ; store row counter.

       ld bc,(inkmsk)      ; get ink and paper colours.
       ld a,(dispy)        ; get y coordinate.
       rra                 ; odd or even alignment?
       ld a,(de)           ; get byte of image.
       push de             ; store pointer to font data.
       jr c,btxt2          ; odd alignment.
       call ptxte          ; print at even cell position.
       jp btxt1
btxt2  call ptxto          ; print at odd cell position.
btxt1  dec hl              ; back to left edge of character.
       dec hl
       call nline2         ; down one line.

       pop de              ; restore text image address.
       pop bc              ; retrieve stretch counter.
       djnz btxt3          ; repeat for all rows.
       inc de              ; next byte.
       pop bc              ; retrieve row counter.
       djnz btxt0          ; repeat for all rows.
bchr1  call nexpos         ; display position.
       jp nz,bchr2         ; not on a new line.
bchr3  inc (hl)            ; newline.
       call nexlin         ; next line check.
bchr2  jp dscor2           ; tidy up line and column variables.


; Print text at odd cell.

ptxto  rla                 ; test bit.
       ld d,a              ; store font data.
       ld a,(hl)           ; get screen data.
       jr c,ptxto8         ; it was set.
       and 170             ; preserve left pixels.
       ld e,a              ; store in e.
       ld a,b              ; paper colour.
       jr ptxto9
ptxto8 and 170             ; preserve left pixels.
       ld e,a              ; store in e.
       ld a,c              ; ink colour.
ptxto9 and 85              ; right bits only.
       or e                ; merge with screen.
       ld (hl),a           ; write first bit.
       inc hl              ; next two pixels.
       ld a,d              ; get data again.
       rla                 ; test bit.
       jr c,ptxto0         ; it was set.
       ld (hl),b           ; set paper.
       jr ptxto1
ptxto0 ld (hl),c           ; set to ink.
ptxto1 rla                 ; test bit.
       ld d,a              ; store data.
       jr c,ptxto2         ; it was set.
       ld a,b              ; paper colour.
       jr ptxto3
ptxto2 ld a,c              ; ink colour.
ptxto3 and 85              ; right bits.
       ld e,a              ; store in e.
       ld a,(hl)           ; get screen colours.
       and 170             ; left bits only.
       or e                ; merge with right bits.
       ld (hl),a           ; write to screen.
       ld a,d              ; get bit of image data again.
       inc hl              ; next two pixels.
       ld a,d              ; get data again.
       rla                 ; test bit.
       jr c,ptxto4         ; it was set.
       ld (hl),b           ; set paper.
       jr ptxto5
ptxto4 ld (hl),c           ; set to ink.
ptxto5 rla                 ; test bit.
       ld d,a              ; store data.
       jr c,ptxto6         ; it was set.
       ld a,b              ; paper colour.
       jr ptxto7
ptxto6 ld a,c              ; ink colour.
ptxto7 and 85              ; right bits.
       ld e,a              ; store in e.
       ld a,(hl)           ; get screen colours.
       and 170             ; left bits only.
       or e                ; merge with right bits.
       ld (hl),a           ; write to screen.
       ret

; Print text at even cell.

ptxte  rla                 ; test bit.
       jr c,ptxte0         ; it was set.
       ld (hl),b           ; set paper.
       jr ptxte1
ptxte0 ld (hl),c           ; set to ink.
ptxte1 rla                 ; test bit.
       ld d,a              ; store data.
       jr c,ptxte2         ; it was set.
       ld a,b              ; paper colour.
       jr ptxte3
ptxte2 ld a,c              ; ink colour.
ptxte3 and 85              ; right bits.
       ld e,a              ; store in e.
       ld a,(hl)           ; get screen colours.
       and 170             ; left bits only.
       or e                ; merge with right bits.
       ld (hl),a           ; write to screen.
       ld a,d              ; get bit of image data again.
       inc hl              ; next screen address.
       rla                 ; test bit.
       jr c,ptxte4         ; it was set.
       ld (hl),b           ; set paper.
       jr ptxte5
ptxte4 ld (hl),c           ; set to ink.
ptxte5 rla                 ; test bit.
       ld d,a              ; store data.
       jr c,ptxte6         ; it was set.
       ld a,b              ; paper colour.
       jr ptxte7
ptxte6 ld a,c              ; ink colour.
ptxte7 and 85              ; right bits.
       ld e,a              ; store in e.
       ld a,(hl)           ; get screen colours.
       and 170             ; left bits only.
       or e                ; merge with right bits.
       ld (hl),a           ; write to screen.
       ld a,d              ; get bit of image data again.
       inc hl              ; next screen address.
       rla                 ; test bit.
       ld a,(hl)           ; get screen data.
       jr c,ptxte8         ; it was set.
       and 85              ; preserve right pixels.
       ld e,a              ; store in e.
       ld a,b              ; paper colour.
       jr ptxte9
ptxte8 and 85              ; preserve right pixels.
       ld e,a              ; store in e.
       ld a,c              ; ink colour.
ptxte9 and 170             ; left bits only.
       or e                ; merge with screen.
       ld (hl),a           ; write final bit.
       ret


; Sprite routine for objects, only appears at even pixel positions.

sprite push hl             ; store sprite graphic address.
       call scadd          ; get screen address in hl.
       pop de              ; restore graphic address in de pair.
       ld b,16             ; pixel height.
sprit1 push bc             ; store row counter.
       ld b,4              ; pixel width.
sprit0 ld a,(de)           ; fetch graphic data.
       xor (hl)            ; merge with what's on screen.
       ld (hl),a           ; bung it onto the screen.
       inc de              ; point to next piece of image.
       inc l               ; next screen address.
       djnz sprit0         ; repeat for all columns.
       call nline          ; next line down.
       pop bc              ; restore row counter.
       djnz sprit1         ; repeat for all rows.
       ret

; Get room address.

groom  ld a,(scno)         ; screen number.
groomx ld de,0             ; start at zero.
       ld hl,(scrptr)      ; pointer to screens.
       and a               ; is it the first one?
groom1 jr z,groom0         ; no more screens to skip.
       ld c,(hl)           ; low byte of screen size.
       inc hl              ; point to high byte.
       ld b,(hl)           ; high byte of screen size.
       inc hl              ; next address.
       ex de,hl            ; put total in hl, pointer in de.
       add hl,bc           ; skip a screen.
       ex de,hl            ; put total in de, pointer in hl.
       dec a               ; one less iteration.
       jr groom1           ; loop until we reach the end.
groom0 ld hl,(scrptr)      ; pointer to screens.
       add hl,de           ; add displacement.
       ld a,(numsc)        ; number of screens.
       ld d,0              ; zeroise high byte.
       ld e,a              ; displacement in de.
       add hl,de           ; add double displacement to address.
       add hl,de
       ret

; Draw present room.

droom  ld a,(wintop)       ; window top.
       ld (dispx),a        ; set x coordinate.
droom2 call groom          ; get address of current room.
       xor a               ; zero in accumulator.
       ld (comcnt),a       ; reset compression counter.
       ld a,(winhgt)       ; height of window.
droom0 push af             ; store row counter.
       ld a,(winlft)       ; window left edge.
       ld (dispy),a        ; set cursor position.
       ld a,(winwid)       ; width of window.
droom1 push af             ; store column counter.
       call flbyt          ; decompress next byte on the fly.
       push hl             ; store address of cell.
       call pattr          ; show attributes and block.
       pop hl              ; restore cell address.
       pop af              ; restore loop counter.
       dec a               ; one less column.
       jr nz,droom1        ; repeat for entire line.
       ld a,(dispx)        ; x coord.
       inc a               ; move down one line.
       ld (dispx),a        ; set new position.
       pop af              ; restore row counter.
       dec a               ; one less row.
       jr nz,droom0        ; repeat for all rows.
       ret

; Decompress bytes on-the-fly.

flbyt  ld a,(comcnt)       ; compression counter.
       and a               ; any more to decompress?
       jr nz,flbyt1        ; yes.
       ld a,(hl)           ; fetch next byte.
       inc hl              ; point to next cell.
       cp 255              ; is this byte a control code?
       ret nz              ; no, this byte is uncompressed.
       ld a,(hl)           ; fetch byte type.
       ld (combyt),a       ; set up the type.
       inc hl              ; point to quantity.
       ld a,(hl)           ; get quantity.
       inc hl              ; point to next byte.
flbyt1 dec a               ; one less.
       ld (comcnt),a       ; store new quantity.
       ld a,(combyt)       ; byte to expand.
       ret

combyt defb 0              ; byte type compressed.
comcnt defb 0              ; compression counter.

; Ladder down check.

laddd  ld a,(ix+8)         ; x coordinate.
       ld h,(ix+9)         ; y coordinate.
       add a,16            ; look down 16 pixels.
       ld l,a              ; coords in hl.
       jr laddv

; Ladder up check.

laddu  ld a,(ix+8)         ; x coordinate.
       ld h,(ix+9)         ; y coordinate.
       add a,14            ; look 2 pixels above feet.
       ld l,a              ; coords in hl.
laddv  ld (dispx),hl       ; set up test coordinates.
       call tstbl          ; get map address.
       call ldchk          ; standard ladder check.
       ret nz              ; no way through.
       inc hl              ; look right one cell.
       call ldchk          ; do the check.
       ret nz              ; impassable.
       ld a,(dispy)        ; y coordinate.
       call rem5           ; position straddling block cells.
       ret z               ; no more checks needed.
       dec a               ; one pixel to right?
       ret z               ; no more checks needed.
       inc hl              ; look to third cell.
       call ldchk          ; do the check.
       ret                 ; return with zero flag set accordingly.

; Can go up check.

cangu  ld a,(ix+8)         ; x coordinate.
       ld h,(ix+9)         ; y coordinate.
       sub 2               ; look up 2 pixels.
       ld l,a              ; coords in hl.
       ld (dispx),hl       ; set up test coordinates.
       call tstbl          ; get map address.
       call lrchk          ; standard left/right check.
       ret nz              ; no way through.
       inc hl              ; look right one cell.
       call lrchk          ; do the check.
       ret nz              ; impassable.
       ld a,(dispy)        ; y coordinate.
       call rem5           ; position straddling block cells.
       ret z               ; no more checks needed.
       dec a               ; one pixel to right?
       ret z               ; no more checks needed.
       inc hl              ; look to third cell.
       call lrchk          ; do the check.
       ret                 ; return with zero flag set accordingly.

; Can go down check.

cangd  ld a,(ix+8)         ; x coordinate.
       ld h,(ix+9)         ; y coordinate.
       add a,16            ; look down 16 pixels.
       ld l,a              ; coords in hl.
       ld (dispx),hl       ; set up test coordinates.
       call tstbl          ; get map address.
       call plchk          ; block, platform check.
       ret nz              ; no way through.
       inc hl              ; look right one cell.
       call plchk          ; block, platform check.
       ret nz              ; impassable.
       ld a,(dispy)        ; y coordinate.
       call rem5           ; position straddling block cells.
       ret z               ; no more checks needed.
       dec a               ; one pixel to right?
       ret z               ; no more checks needed.
       inc hl              ; look to third cell.
       call plchk          ; block, platform check.
       ret                 ; return with zero flag set accordingly.

; Can go left check.

cangl  ld l,(ix+8)         ; x coordinate.
       ld h,(ix+9)         ; y coordinate.
       dec h               ; look left 1 pixel.
       jr cangh            ; test if we can go there.

; Can go right check.

cangr  ld l,(ix+8)         ; x coordinate.
       ld a,(ix+9)         ; y coordinate.
       add a,9             ; look right 9 pixels.
       ld h,a              ; coords in hl.

cangh  ld (dispx),hl       ; set up test coordinates.
       call tstbl          ; get map address.
       call lrchk          ; standard left/right check.
       ret nz              ; no way through.
       ld de,32            ; distance to next cell.
       add hl,de           ; look down.
       call lrchk          ; do the check.
       ret nz              ; impassable.
       ld a,(dispx)        ; x coordinate.
       and 7               ; position straddling block cells.
       ret z               ; no more checks needed.
       add hl,de           ; look down to third cell.
       call lrchk          ; do the check.
       ret                 ; return with zero flag set accordingly.

; Check left/right movement is okay.

lrchk  ld a,(hl)           ; fetch map cell.
       cp WALL             ; is it passable?
       jr z,lrchkx         ; no.
       cp FODDER           ; fodder has to be dug.
       jr z,lrchkx         ; not passable.
always xor a               ; report it as okay.
       ret
lrchkx xor a               ; reset all bits.
       inc a
       ret

; Check platform or solid item is not in way.

plchk  ld a,(hl)           ; fetch map cell.
       cp WALL             ; is it passable?
       jr z,lrchkx         ; no.
       cp FODDER           ; fodder has to be dug.
       jr z,lrchkx         ; not passable.
       cp PLATFM           ; platform is solid.
       jr z,plchkx         ; not passable.
       cp LADDER           ; is it a ladder?
       jr z,lrchkx         ; on ladder, deny movement.
plchk0 xor a               ; report it as okay.
       ret
plchkx ld a,(dispx)        ; x coordinate.
       and 7               ; position straddling blocks.
       jr z,lrchkx         ; on platform, deny movement.
       jr plchk0

; Check ladder is available.

ldchk  ld a,(hl)           ; fetch cell.
       cp LADDER           ; is it a ladder?
       ret                 ; return with zero flag set accordingly.

; Get collectables.

getcol ld b,COLECT         ; collectable blocks.
       call tded           ; test for collectable blocks.
       cp b                ; did we find one?
       ret nz              ; none were found, job done.
       call gtblk          ; get block.
       call evnt20         ; collected block event.
       jr getcol           ; repeat until none left.

; Get collectable block.

gtblk  ld (hl),0           ; make it empty now.
       ld de,MAP           ; map address.
       and a               ; clear carry.
       sbc hl,de           ; find cell number.
       ld a,l              ; get low byte of cell number.
       and 31              ; 0 - 31 is column.
       ld d,a              ; store y in d register.
       add hl,hl           ; multiply by 8.
       add hl,hl
       add hl,hl           ; x is now in h.
       ld e,h              ; put x in e.
       ld (dispx),de       ; set display coordinates.

       ld a,(colpat)       ; get collectable block number for this screen.
       call xorbk          ; xor block from the screen.
       xor a               ; block zero.

; Now drop through and xor block onto screen.

xorbk  call gchadd         ; get character address.
       call gprad          ; get screen address in de.
       ex de,hl            ; screen in hl, block image in de.

       ld a,(dispy)        ; get horizontal position.
       rra                 ; are we displaying block at an odd position?
       jp c,xorbk0         ; yes, need to shift everything then.
       ld bc,2046          ; distance to next line.
       call xorbke         ; even pixel position.
       call xorbke
       call xorbke
       call xorbke
       call xorbke
       call xorbke
       call xorbke

xorbke ld a,(de)           ; get image byte.
       xor (hl)            ; merge with screen.
       ld (hl),a           ; write to screen.
       inc hl              ; next screen byte.
       inc de              ; next image byte.
       ld a,(de)           ; get image byte.
       xor (hl)            ; merge with screen.
       ld (hl),a           ; write to screen.
       inc hl              ; next screen byte.
       inc de              ; next image byte.
       ld a,(de)           ; get image byte.
       xor (hl)            ; merge in final pixel data.
       inc de              ; postincrement to next byte of data.
       ld (hl),a           ; write third byte.
       add hl,bc           ; look down one pixel.
       ret

xorbk0 call xorbko         ; odd pixel position.
       call xorbko
       call xorbko
       call xorbko
       call xorbko
       call xorbko
       call xorbko

xorbko ld a,(de)           ; get first byte of data to write.
       rra                 ; shift into right position.
       and 85              ; get right pixel colour.
       xor (hl)            ; merge with screen byte.
       ld (hl),a           ; write that byte.

       inc hl              ; next screen byte.
       ld a,(de)           ; get first byte of image again.
       rla                 ; shift second pixel into left position.
       and 170             ; remove right bits.
       ld b,a              ; store for now.
       inc de              ; point to next byte of data.
       ld a,(de)           ; get third pixel.
       rra                 ; shift into right pixel position.
       and 85              ; remove left pixel.
       or b                ; merge with previous pixel.
       xor (hl)            ; merge with screen.
       ld (hl),a           ; write to screen.
       inc hl              ; third screen byte.
       ld a,(de)           ; get second byte of image again.
       rla                 ; shift fourth pixel into left position.
       and 170             ; remove right bits.
       ld b,a              ; store for now.
       inc de              ; point to next byte of data.
       ld a,(de)           ; get fifth pixel.
       rra                 ; shift into right pixel position.
       and 85              ; remove left pixel.
       or b                ; merge with previous pixel.
       xor (hl)            ; merge with screen.
       ld (hl),a           ; write to screen.
       inc de              ; point to next byte of data.

       ld bc,2046          ; distance to next line.
       add hl,bc           ; look down one pixel.
       ret

; Touched deadly block check.
; Returns with DEADLY (must be non-zero) in accumulator if true.

tded   ld l,(ix+8)         ; x coordinate.
       ld h,(ix+9)         ; y coordinate.
       ld (dispx),hl       ; set up test coordinates.
       call tstbl          ; get map address.
       ld de,31            ; default distance to next line down.
       cp b                ; is this the required block?
       ret z               ; yes.
       inc hl              ; next cell.
       ld a,(hl)           ; fetch type.
       cp b                ; is this deadly/custom?
       ret z               ; yes.
       ld a,(dispy)        ; horizontal position.
       ld c,a              ; store column in c register.
       call rem5           ; is it straddling cells?
       jr z,tded0          ; no.
       dec a               ; one pixel to right?
       jr z,tded0          ; no more checks needed.
       inc hl              ; last cell.
       ld a,(hl)           ; fetch type.
       cp b                ; is this the block?
       ret z               ; yes.
       dec de              ; one less cell to next row down.
tded0  add hl,de           ; point to next row.
       ld a,(hl)           ; fetch left cell block.
       cp b                ; is this fatal?
       ret z               ; yes.
       inc hl              ; next cell.
       ld a,(hl)           ; fetch type.
       cp b                ; is this fatal?
       ret z               ; yes.
       ld a,c              ; horizontal position.
       call rem5           ; is it straddling cells?
       jr z,tded1          ; no.
       dec a               ; one pixel to right?
       jr z,tded1          ; no more checks needed.
       inc hl              ; last cell.
       ld a,(hl)           ; fetch type.
       cp b                ; is this fatal?
       ret z               ; yes.
tded1  ld a,(dispx)        ; vertical position.
       and 7               ; is it straddling cells?
       ret z               ; no, job done.
       add hl,de           ; point to next row.
       ld a,(hl)           ; fetch left cell block.
       cp b                ; is this fatal?
       ret z               ; yes.
       inc hl              ; next cell.
       ld a,(hl)           ; fetch type.
       cp b                ; is this fatal?
       ret z               ; yes.
       ld a,c              ; horizontal position.
       call rem5           ; is it straddling cells?
       ret z               ; no.
       dec a               ; one pixel to right?
       ret z               ; no more checks needed.
       inc hl              ; last cell.
       ld a,(hl)           ; fetch final type.
       ret                 ; return with final type in accumulator.


; Fetch block type at (dispx, dispy).

tstbl  ld a,(dispx)        ; fetch x coord.
       rlca                ; divide by 8,
       rlca                ; and multiply by 32.
       ld d,a              ; store in d.
       and 224             ; mask off high bits.
       ld e,a              ; low byte.
       ld a,d              ; restore shift result.
       and 3               ; high bits.
       ld d,a              ; got displacement in de.
       ld a,(dispy)        ; y coord.
       call div5           ; divide by 5.
       add a,e             ; add to displacement.
       ld e,a              ; displacement in de.
       ld hl,MAP           ; position of dummy screen.
       add hl,de           ; point to address.
       ld a,(hl)           ; fetch byte there.
       ret

; Divide y coordinate by 5 to give block position.

div5   push bc             ; store bc.
       ld b,0              ; reset result.
       cp 80               ; more than 80?
       jr c,div5a          ; no.
       sub 80              ; subtract 80.
       set 4,b             ; set bit for 16.
div5a  cp 40               ; more than 40?
       jr c,div5b          ; no.
       sub 40              ; subtract 40.
       set 3,b             ; set bit for 8.
div5b  cp 20               ; more than 20?
       jr c,div5c          ; no.
       sub 20              ; subtract 20.
       set 2,b             ; set bit for 4.
div5c  cp 10               ; more than 10?
       jr c,div5d          ; no.
       sub 10              ; subtract 10.
       set 1,b             ; set bit for 2.
div5d  cp 5                ; more than 5?
       jr c,div5e          ; no.
       inc b               ; set bit for 1.
div5e  ld a,b              ; get result.
       pop bc              ; restore bc.
       ret

; Find remainder when coordinate divided by 5 to establish block alignment.

rem5   cp 80               ; is coordinate more than 80?
       jr c,rem5a          ; no.
       sub 80              ; subtract amount.
rem5a  cp 40               ; is coordinate more than 40?
       jr c,rem5b          ; no.
       sub 40              ; subtract amount.
rem5b  cp 20               ; is coordinate more than 20?
       jr c,rem5c          ; no.
       sub 20              ; subtract amount.
rem5c  cp 10               ; is coordinate more than 10?
       jr c,rem5d          ; no.
       sub 10              ; subtract amount.
rem5d  cp 5                ; is coordinate more than 5?
       jr c,rem5e          ; no.
       sub 5               ; subtract amount.
rem5e  and a               ; remainder in accumulator, set zero flag accordingly.
       ret


; Jump - if we can.
; Requires initial speed to be set up in accumulator prior to call.

jump   neg                 ; switch sign so we jump up.
       ld c,a              ; store in c register.
;       ld a,(ix+8)         ; x coordinate.
;       ld h,(ix+9)         ; y coordinate.
;numsp4 add a,16            ; look down 16 pixels.
;       ld l,a              ; coords in hl.
;       and 7               ; are we on platform boundary?
;       ret nz              ; no, cannot jump.
;       ld (dispx),hl       ; set up test coordinates.
;       ld b,a              ; copy to b register.
;       call tstbl          ; get map address.
;       call plchk          ; block, platform check.
;       jr nz,jump0         ; it's solid, we can jump.
;       inc hl              ; look right one cell.
;       call plchk          ; block, platform check.
;       jr nz,jump0         ; it's solid, we can jump.
;       ld a,b              ; y coordinate.
;       call rem5           ; position straddling block cells.
;       ret z               ; no more checks needed.
;       inc hl              ; look to third cell.
;       call plchk          ; block, platform check.
;       ret z               ; not solid, don't jump.
jump0  ld a,(ix+13)        ; jumping flag.
       and a               ; is it set?
       ret nz              ; already in the air.
       inc (ix+13)         ; set it.
       ld (ix+14),c        ; set jump height.
       ret

hop    ld a,(ix+13)        ; jumping flag.
       and a               ; is it set?
       ret nz              ; already in the air.
       ld (ix+13),255      ; set it.
       ld (ix+14),0        ; set jump table displacement.
       ret


; Random numbers code.
; Pseudo-random number generator, 8-bit.

random ld hl,seed          ; set up seed pointer.
       ld a,(hl)           ; get last random number.
       ld b,a              ; copy to b register.
       rrca                ; multiply by 32.
       rrca
       rrca
       xor 31
       add a,b
       sbc a,255
       ld (hl),a           ; store new seed.
       ld (varrnd),a       ; return number in variable.
       ret

; Joystick and keyboard reading routines.
; General keyboard test routine used by compiler.

ktest  call 47902          ; check if keycode is pressed.
       jr nz,ktest0        ; pressed, reset carry.
       scf                 ; set carry.
       ret
ktest0 and a               ; clear carry flag.
       ret

; Joystick states:
; 1 - up.
; 2 - down.
; 4 - left.
; 8 - right.
; 16 - fire 1.
; 32 - fire 2.

joykey call 47908          ; get joystick states in hl.
       ld a,(contrl)       ; controls.
       and a               ; keyboard?
       jp z,keyb           ; read keys.
       dec a               ; joystick zero?
       jr z,joy0           ; just want joystick 0.
       ld a,l              ; joystick 1.
       ld (joyval),a       ; store joystick reading.
       ret
joy0   ld a,h              ; value of joystick 0.
       ld (joyval),a       ; remember value.
       ret
keyb   ld b,0              ; default value.
       ld a,(keys+5)       ; fire 2.
       call keybck         ; check if pressed.
       sla b               ; shift result byte.
       ld a,(keys+4)       ; fire 1.
       call keybck         ; check if pressed.
       sla b               ; shift result byte.
       ld a,(keys+3)       ; right.
       call keybck         ; check if pressed.
       sla b               ; shift result byte.
       ld a,(keys+2)       ; left.
       call keybck         ; check if pressed.
       sla b               ; shift result byte.
       ld a,(keys+1)       ; down.
       call keybck         ; check if pressed.
       sla b               ; shift result byte.
       ld a,(keys)         ; up.
       call keybck         ; check if pressed.
       ld a,b              ; put result in accumulator.
       ld (joyval),a       ; set the joystick reading.
       ret
keybck call 47902          ; check if key pressed.
       ret z               ; not pressed.
       inc b               ; set bit to say pressed.
       ret


; Display message.

dmsg   ld hl,msgdat        ; pointer to messages.
       call getwrd         ; get message number.
dmsg3  call preprt         ; pre-printing stuff.
       call checkx         ; make sure we're in a printable range.
       ld a,(prtmod)       ; print mode.
       and a               ; standard size?
       jp nz,bmsg1         ; no, double-height text.
dmsg0  push hl             ; store string pointer.
       ld a,(hl)           ; fetch byte to display.
       and 127             ; remove any end marker.
       cp 13               ; newline character?
       jr z,dmsg1
       call ptxt           ; display character.
       call nexpos         ; display position.
       jr nz,dmsg2         ; not on a new line.
       call nexlin         ; next line down.
dmsg2  pop hl
       ld a,(hl)           ; fetch last character.
       rla                 ; was it the end?
       jp c,dscor2         ; yes, job done.
       inc hl              ; next character to display.
       jr dmsg0
dmsg1  ld hl,dispx         ; x coordinate.
       inc (hl)            ; newline.
       ld a,(hl)           ; fetch position.
       cp 25               ; past screen edge?
       jr c,dmsg4          ; no, it's okay.
       ld (hl),0           ; restart at top.
dmsg4  inc hl              ; y coordinate.
       ld (hl),0           ; carriage return.
       jr dmsg2
prtmod defb 0              ; print mode, 0 = standard, 1 = double-height.

; Display message in big text.

bmsg1  ld a,(hl)           ; get character to display.
       push hl             ; store pointer to message.
       and 127             ; only want 7 bits.
       cp 13               ; newline character?
       jr z,bmsg2          ; newline instead.
       call btxt           ; display big char.
bmsg3  pop hl              ; retrieve message pointer.
       ld a,(hl)           ; look at last character.
       inc hl              ; next character in list.
       rla                 ; was terminator flag set?
       jr nc,bmsg1         ; no, keep going.
       ret
bmsg2  ld hl,charx         ; x coordinate.
       inc (hl)            ; newline.
       inc (hl)            ; newline.
       ld a,(hl)           ; fetch position.
       cp 24               ; past screen edge?
       jr c,bmsg3          ; no, it's okay.
       ld (hl),0           ; restart at top.
       inc hl              ; y coordinate.
       ld (hl),0           ; carriage return.
       jr bmsg3

; Display a character.

achar  ld b,a              ; copy to b.
       call preprt         ; get ready to print.
       ld a,(prtmod)       ; print mode.
       and a               ; standard size?
       ld a,b              ; character in accumulator.
       jp nz,btxt          ; no, double-height text.
       call ptxt           ; display character.
       call nexpos         ; display position.
       jp z,btxt3          ; next line down.
       jp btxt2            ; tidy up.

; Get next print column position.

nexpos ld hl,dispy         ; display position.
       ld a,(hl)           ; get coordinate.
       inc a               ; move along one position.
       and 31              ; reached edge of screen?
       ld (hl),a           ; set new position.
       dec hl              ; point to x now.
       ret                 ; return with status in zero flag.

; Get next print line position.

nexlin inc (hl)            ; newline.
       ld a,(hl)           ; vertical position.
       cp 24               ; past screen edge?
       ret c               ; no, still okay.
       ld (hl),0           ; restart at top.
       ret

; Pre-print preliminaries.

preprt ld de,(charx)       ; display coordinates.
       ld (dispx),de       ; set up general coordinates.
       ret

; On entry  hl points to word list
;           a contains word number.

getwrd and a               ; first word in list?
       ret z               ; yep, don't search.
       ld b,a
getwd0 ld a,(hl)
       inc hl
       cp 128              ; found end?
       jr c,getwd0         ; no, carry on.
       djnz getwd0         ; until we have right number.
       ret


; Bubble sort.

bsort  ld b,NUMSPR - 1     ; sprites to swap.
       ld ix,sprtab        ; sprite table.
bsort0 push bc             ; store loop counter for now.

       ld a,(ix+0)         ; first sprite type.
       inc a               ; is it switched off?
       jr z,swemp          ; yes, may need to switch another in here.

       ld a,(ix+TABSIZ)    ; check next slot exists.
       inc a               ; is it enabled?
       jr z,bsort2         ; no, nothing to swap.

       ld a,(ix+3+TABSIZ)  ; fetch next sprite's coordinate.
       cp (ix+3)           ; compare with this x coordinate.
       jr c,bsort1         ; next sprite is higher - may need to switch.
bsort2 ld de,TABSIZ        ; distance to next odd/even entry.
       add ix,de           ; next sprite.
       pop bc              ; retrieve loop counter.
       djnz bsort0         ; repeat for remaining sprites.
       ret

bsort1 ld a,(ix+TABSIZ)    ; sprite on/off flag.
       inc a               ; is it enabled?
       jr z,bsort2         ; no, nothing to swap.
       call swspr          ; swap positions.
       jr bsort2

swemp  ld a,(ix+TABSIZ)    ; next table entry.
       inc a               ; is that one on?
       jr z,bsort2         ; no, nothing to swap.
       call swspr          ; swap positions.
       jr bsort2

; Swap sprites.

swspr  push ix             ; table address on stack.
       pop hl              ; pop into hl pair.
       ld d,h              ; copy to de pair.
       ld e,l
       ld bc,TABSIZ        ; distance to second entry.
       add hl,bc           ; point to second sprite entry.
       ld b,TABSIZ         ; bytes to swap.
swspr0 ld c,(hl)           ; fetch second byte.
       ld a,(de)           ; fetch first byte.
       ld (hl),a           ; copy to second.
       ld a,c              ; second byte in accumulator.
       ld (de),a           ; copy to first sprite entry.
       inc de              ; next byte.
       inc hl              ; next byte.
       djnz swspr0         ; swap all bytes in table entry.
       ret


; Process sprites.

pspr   ld b,NUMSPR         ; sprites to process.
       ld ix,sprtab        ; sprite table.
pspr1  push bc             ; store loop counter for now.
       ld a,(ix+0)         ; fetch sprite type.
       cp 9                ; within range of sprite types?
       call c,pspr2        ; yes, process this one.
       ld de,TABSIZ        ; distance to next odd/even entry.
       add ix,de           ; next sprite.
       pop bc              ; retrieve loop counter.
       djnz pspr1          ; repeat for remaining sprites.
       ret
pspr2  ld (ogptr),ix       ; store original sprite pointer.
       call pspr3          ; do the routine.
rtorg  ld ix,(ogptr)       ; restore original pointer to sprite.
rtorg0 ret
pspr3  ld hl,evtyp0        ; sprite type events list.
pspr4  add a,a             ; double accumulator.
       ld e,a              ; copy to de.
       ld d,0              ; no high byte.
       add hl,de           ; point to address of routine.
       ld e,(hl)           ; address low.
       inc hl              ; next byte of address.
       ld d,(hl)           ; address high.
       ex de,hl            ; swap address into hl.
       jp (hl)             ; go there.
ogptr  defw 0              ; original sprite pointer.

; Address of each sprite type's routine.

evtyp0 defw evnt00
evtyp1 defw evnt01
evtyp2 defw evnt02
evtyp3 defw evnt03
evtyp4 defw evnt04
evtyp5 defw evnt05
evtyp6 defw evnt06
evtyp7 defw evnt07
evtyp8 defw evnt08


; Display sprites.

dspr   ld b,NUMSPR/2       ; number of sprites to display.
dspr0  push bc             ; store loop counter for now.
       ld a,(ix+0)         ; get sprite type.
       inc a               ; is it enabled?
       jr nz,dspr1         ; yes, it needs deleting.
dspr5  ld a,(ix+5)         ; new type.
       inc a               ; is it enabled?
       jr nz,dspr3         ; yes, it needs drawing.

dspr2  push ix             ; put ix on stack.
       pop hl              ; pop into hl.
       ld e,l              ; copy to de.
       ld d,h

;dspr2  ld e,ixl            ; copy ix to de.
;       ld d,ixh
;       ld l,e              ; copy to hl.
;       ld h,d
       ld bc,5             ; distance to new type.
       add hl,bc           ; point to new properties.
       ldi                 ; copy to old positions.
       ldi
       ldi
       ldi
       ldi
       ld c,TABSIZ*2       ; distance to next odd/even entry.
       add ix,bc           ; next sprite.
       pop bc              ; retrieve loop counter.
       djnz dspr0          ; repeat for remaining sprites.
       ret
;dspr1  ld a,(ix+3)         ; old x coord.
;       cp 177              ; beyond maximum?
;       jr nc,dspr5         ; yes, don't delete it.
dspr1  ld a,(ix+3)         ; old x coord.
       cp 185              ; beyond maximum?
       jr nc,dspr5         ; yes, don't delete it.
       ld a,(ix+5)         ; type of new sprite.
       inc a               ; is this enabled?
       jr nz,dspr4         ; yes, display both.
dspr6  call sspria         ; show single sprite.
       jp dspr2
dspr4  ld a,(ix+8)         ; new x coord.
       cp 185              ; beyond maximum?
       jr nc,dspr6         ; yes, don't display it.

; Displaying two sprites.  Don't bother redrawing if nothing has changed.

       ld a,(ix+4)         ; old y.
       cp (ix+9)           ; compare with new value.
       jr nz,dspr7         ; they differ, need to redraw.
       ld a,(ix+3)         ; old x.
       cp (ix+8)           ; compare against new value.
       jr nz,dspr7         ; they differ, need to redraw.
       ld a,(ix+2)         ; old frame.
       cp (ix+7)           ; compare against new value.
       jr nz,dspr7         ; they differ, need to redraw.
       ld a,(ix+1)         ; old image.
       cp (ix+6)           ; compare against new value.
       jp z,dspr2          ; everything is the same, don't redraw.
dspr7  call sspric         ; delete old sprite, draw new one simultaneously.
       jp dspr2
dspr3  ld a,(ix+8)         ; new x coord.
       cp 185              ; beyond maximum?
       jr nc,dspr2         ; yes, don't display it.
       call ssprib         ; show single sprite.
       jp dspr2


; Get sprite address calculations.
; gspran = new sprite, gsprad = old sprite.

gspran ld l,(ix+8)         ; new x coordinate.
       ld h,(ix+9)         ; new y coordinate.
       ld (dispx),hl       ; set display coordinates.
       ld a,l              ; x coordinate.
       cp 185              ; beyond maximum?
       ret nc              ; yes, don't display it.

       ld a,(ix+6)         ; new sprite image.
       call gfrm           ; fetch start frame for this sprite.
       ld a,(hl)           ; frame in accumulator.
       add a,(ix+7)        ; new add frame number.
       jp gspra0

gsprad ld l,(ix+3)         ; x coordinate.
       ld h,(ix+4)         ; y coordinate.
       ld (dispx),hl       ; set display coordinates.
       ld a,l              ; x coordinate.
       cp 185              ; beyond lowest point on screen?
       ret nc              ; yes, don't display it.

       ld a,(ix+1)         ; sprite image.
       call gfrm           ; fetch start frame for this sprite.
       ld a,(hl)           ; frame in accumulator.
       add a,(ix+2)        ; add frame number.
gspra0 rrca                ; multiply by 32.
       rrca
       rrca
       ld d,a              ; store in d.
       and 224             ; low byte bits.
       ld e,a              ; got low byte.
       ld a,d              ; restore result.
       and 31              ; high byte bits.
       ld d,a              ; displacement high byte.
       ld hl,sprgfx        ; address of play sprites.
       add hl,de           ; frame * 32.
       add hl,de           ; frame * 64.
       add hl,de           ; frame * 96.
       add hl,de           ; frame * 128.
       add hl,de           ; frame * 160.

       ld a,(dispy)        ; y coordinate.
       rra                 ; test odd pixel number.
       jr nc,gspra1        ; it's even, this address is good.

       ld de,80            ; displacement to second frame.
       add hl,de           ; add to sprite address.
gspra1 ex de,hl            ; need it in de for now.

; Drop into screen address routine.
; This routine returns a mode 0 screen address for (dispx, dispy) in hl.

scadd  ld a,(dispx)        ; x position.
       and 248             ; get character row (multiple of 8), only 25 so only need 5 bits.
       ld l,a              ; copy to hl.
       ld h,0
       add hl,hl           ; * 16.
       ld b,h              ; copy to bc.
       ld c,l              ; bc = * 16.
       add hl,hl           ; * 32.
       add hl,hl           ; * 64.
       add hl,bc           ; * 80.

; Done character row, now work out pixel position.

       ld a,(dispx)        ; restore x position.
       and 7               ; line 0-7 within character square.
       rlca                ; multiply by 2048.
       rlca
       rlca
       add a,h             ; add to high byte.
       add a,192           ; add base address of screen.
       ld h,a              ; store in high byte.
       ld a,(dispy)        ; y coordinate.
       rra                 ; divide by 2, carry already cleared.
       ld c,a              ; low byte.
       ld b,0              ; no high byte.
       add hl,bc           ; add to address.
       ret

; These are the sprite routines.
; sspria = single sprite, old (ix).
; ssprib = single sprite, new (ix+5).
; sspric = both sprites, old (ix) and new (ix+5).

sspria call gsprad         ; get old sprite address.
sspri2 ld a,16             ; vertical lines.
sspri0 push af             ; store line counter on stack.
       call dline          ; draw a line.
       pop af              ; restore line counter.
       dec a               ; one less to go.
       jp nz,sspri0
       ret

ssprib call gspran         ; get new sprite address.
       jp sspri2

sspric call gsprad         ; get old sprite address.
       ld (spad1),hl       ; store addresses.
       ld (spad2),de
       call gspran         ; get new sprite addresses.
       ld a,16             ; vertical lines.
sspri1 push af             ; store line counter.
       call dline          ; draw a line.
       ld (spad3),hl       ; store these addresses.
       ld (spad4),de
       ld hl,(spad1)       ; restore old addresses.
       ld de,(spad2)
       call dline          ; delete a line.
       ld (spad1),hl       ; store addresses.
       ld (spad2),de
       ld hl,(spad3)       ; flip to new sprite addresses.
       ld de,(spad4)
       pop af              ; restore line counter.
       dec a               ; one less to go.
       jp nz,sspri1
       ret

; Storage space for addresses.

spad1  defw 0
spad2  defw 0
spad3  defw 0
spad4  defw 0

dline  ld a,(de)           ; graphic data.
       xor (hl)            ; XOR with what's there.
       ld (hl),a           ; bung it in.
       inc de              ; next graphic.
       inc hl              ; next screen address.
       ld a,(de)           ; fetch data.
       xor (hl)            ; XOR with what's there.
       ld (hl),a           ; bung it in.
       inc de              ; next graphic.
       inc hl              ; next destination address.
       ld a,(de)           ; third bit of data.
       xor (hl)            ; XOR with what's there.
       ld (hl),a           ; bung it in.
       inc de              ; point to next piece of image.
       inc hl              ; next screen address.
       ld a,(de)           ; fetch fourth byte.
       xor (hl)            ; XOR with what's on screen.
       ld (hl),a           ; write it back.
       inc de              ; next graphic address.
       inc hl              ; next screen address.
       ld a,(de)           ; fetch last byte.
       xor (hl)            ; XOR with screen image.
       ld (hl),a           ; copy new image to screen.
       inc de              ; next graphic address.

; Line drawn, now work out next target address.

nline  ld bc,2044          ; distance to next pixel minus width.
       add hl,bc           ; point to new screen address.
       ld a,h              ; get pixel address.
       and 56              ; straddling character position?
       ret nz              ; no, address is okay.
       ld bc,49232         ; distance to next pixel.
;       ld bc,49216         ; distance to next pixel.
       add hl,bc           ; point to new screen address.
       ret

nline2 ld bc,2048          ; distance to next pixel minus width.
       add hl,bc           ; point to new screen address.
       ld a,h              ; get pixel address.
       and 56              ; straddling character position?
       ret nz              ; no, address is okay.
       ld bc,49232         ; distance to next pixel.
       add hl,bc           ; point to new screen address.
       ret


; Animates a sprite.

animsp ld hl,frmno         ; game frame.
       and (hl)            ; is it time to change the frame?
       ret nz              ; not this frame.
       ld a,(ix+6)         ; sprite image.
       call gfrm           ; get frame data.
       inc hl              ; point to frames.
       ld a,(ix+7)         ; sprite frame.
       inc a               ; next one along.
       cp (hl)             ; reached the last frame?
       jr c,anims0         ; no, not yet.
       xor a               ; start at first frame.
anims0 ld (ix+7),a         ; new frame.
       ret
animbk ld hl,frmno         ; game frame.
       and (hl)            ; is it time to change the frame?
       ret nz              ; not this frame.
       ld a,(ix+6)         ; sprite image.
       call gfrm           ; get frame data.
       inc hl              ; point to frames.
       ld a,(ix+7)         ; sprite frame.
       and a               ; first one?
       jr nz,rtanb0        ; yes, start at end.
       ld a,(hl)           ; last sprite.
rtanb0 dec a               ; next one along.
       jr anims0           ; set new frame.

; Check for collision with other sprite, strict enforcement.

sktyp  ld hl,sprtab        ; sprite table.
       ld c,NUMSPR         ; number of sprites.
sktyp0 ld (skptr),hl       ; store pointer to sprite.
       ld a,(hl)           ; get sprite type.
       cp b                ; is it the type we seek?
       jr z,coltyp         ; yes, we can use this one.
sktyp1 ld hl,(skptr)       ; retrieve sprite pointer.
       ld de,TABSIZ        ; size of each entry.
       add hl,de           ; point to next sprite in table.
       dec c               ; one iteration fewer.
       jp nz,sktyp0        ; keep going until we find a slot.
       ld hl,0             ; default to ROM address - no sprite.
       ld (skptr),hl       ; store pointer to sprite.
       or h                ; don't return with zero flag set.
       ret                 ; didn't find one.
skptr  defw 0              ; search pointer.

coltyp ld a,(ix+0)         ; current sprite type.
       cp b                ; seeking sprite of same type?
       jr z,colty1         ; yes, need to check we're not detecting ourselves.
colty0 ld de,X             ; distance to x position in table.
       add hl,de           ; point to coords.
       ld e,(hl)           ; fetch x coordinate.
       inc hl              ; now point to y.
       ld d,(hl)           ; that's y coordinate.

; Drop into collision detection.

colc16 ld a,(ix+X)         ; x coord.
       sub e               ; subtract x.
       jr nc,colc1a        ; result is positive.
       neg                 ; make negative positive.
colc1a cp 16               ; within x range?
       jr nc,sktyp1        ; no - they've missed.
       ld h,a              ; store difference.
       ld a,(ix+Y)         ; y coord.
       sub d               ; subtract y.
       jr nc,colc1b        ; result is positive.
       neg                 ; make negative positive.
colc1b cp 10               ; within y range?
       jr nc,sktyp1        ; no - they've missed.
       add a,h             ; add x difference.
       cp 21               ; only 4 corner pixels touching?
       ret c               ; carry set if there's a collision.
       jp sktyp1           ; try next sprite in table.

colty1 push ix             ; base sprite address onto stack.
       pop de              ; pop it into de.
       ex de,hl            ; flip hl into de.
       sbc hl,de           ; compare the two.
       ex de,hl            ; restore hl.
       jr z,sktyp1         ; addresses are identical.
       jp colty0

; Display number.

disply ld bc,displ0        ; display workspace.
       call num2ch         ; convert accumulator to string.
displ1 dec bc              ; back one character.
       ld a,(bc)           ; fetch digit.
       or 128              ; insert end marker.
       ld (bc),a           ; new value.
       ld hl,displ0        ; display space.
       jp dmsg3            ; display the string.
displ0 defb 0,0,0,13+128


; Initialise screen.

initsc ld a,(roomtb)       ; whereabouts in the map are we?
       call tstsc          ; find displacement.
       cp 255              ; is it valid?
       ret z               ; no, it's rubbish.
       ld (scno),a         ; store new room number.
       ret

; Test screen.

tstsc  ld hl,mapdat-MAPWID ; start of map data, subtract width for negative.
       ld b,a              ; store room in b for now.
       add a,MAPWID        ; add width in case we're negative.
       ld e,a              ; screen into e.
       ld d,0              ; zeroise d.
       add hl,de           ; add displacement to map data.
       ld a,(hl)           ; find room number there.
       ret

; Screen left.

scrl   ld a,(roomtb)       ; present room table pointer.
       dec a               ; room left.
scrl0  call tstsc          ; test screen.
       inc a               ; is there a screen this way?
       ret z               ; no, return to loop.
       ld a,b              ; restore room displacement.
       ld (roomtb),a       ; new room table position.
scrl1  call initsc         ; set new screen.
       ld hl,restfl        ; restart screen flag.
       ld (hl),2           ; set it.
       ret
scrr   ld a,(roomtb)       ; room table pointer.
       inc a               ; room right.
       jr scrl0
scru   ld a,(roomtb)       ; room table pointer.
       sub MAPWID          ; room up.
       jr scrl0
scrd   ld a,(roomtb)       ; room table pointer.
       add a,MAPWID        ; room down.
       jr scrl0

; Jump to new screen.

nwscr  ld hl,mapdat        ; start of map data.
       ld bc,256*80        ; zero room count, 80 to search.
nwscr0 cp (hl)             ; have we found a match for screen?
       jr z,nwscr1         ; yes, set new point in map.
       inc hl              ; next room.
       inc c               ; count rooms.
       djnz nwscr0         ; keep looking.
       ret
nwscr1 ld a,c              ; room displacement.
       ld (roomtb),a       ; set the map position.
       jr scrl1            ; draw new room.


; Gravity processing.

grav   ld a,(ix+13)        ; in-air flag.
       and a               ; are we in the air?
       ret z               ; no we are not.
       inc a               ; increment it.
       jp z,ogrv           ; set to 255, use old gravity.
       ld (ix+13),a        ; write new setting.
       rra                 ; every other frame.
       jr nc,grav0         ; don't apply gravity this time.
       ld a,(ix+14)        ; pixels to move.
       cp 16               ; reached maximum?
       jr z,grav0          ; yes, continue.
       inc (ix+14)         ; slow down ascent/speed up fall.
grav0  ld a,(ix+14)        ; get distance to move.
       sra a               ; divide by 2.
       and a               ; any movement required?
       ret z               ; no, not this time.
       cp 128              ; is it up or down?
       jr nc,gravu         ; it's up.
gravd  ld b,a              ; set pixels to move.
gravd0 call cangd          ; can we go down?
       jr nz,gravst        ; can't move down, so stop.
       inc (ix+8)          ; adjust new x coord.
       djnz gravd0
       ret
gravu  neg                 ; flip the sign so it's positive.
       ld b,a              ; set pixels to move.
gravu0 call cangu          ; can we go up?
       jp nz,ifalls        ; can't move up, go down next.
       dec (ix+8)          ; adjust new x coord.
       djnz gravu0
       ret
gravst ld a,(ix+14)        ; jump pointer high.
       ld (ix+13),0        ; reset falling flag.
       ld (ix+14),0        ; store new speed.
       cp 8                ; was speed the maximum?
evftf  jp z,evnt15         ; yes, fallen too far.
       ret

; Old gravity processing for compatibility with Spectrum versions 4.6 and 4.7.

ogrv   ld e,(ix+14)        ; get index to table.
       ld d,0              ; no high byte.
       ld hl,jtab          ; jump table.
       add hl,de           ; hl points to jump value.
       ld a,(hl)           ; pixels to move.
       cp 99               ; reached the end?
       jr nz,ogrv0         ; no, continue.
       dec hl              ; go back to previous value.
       ld a,(hl)           ; fetch that from table.
       jr ogrv1
ogrv0  inc (ix+14)         ; point to next table entry.
ogrv1  and a               ; any movement required?
       ret z               ; no, not this time.
       cp 128              ; is it up or down?
       jr nc,ogrvu         ; it's up.
ogrvd  ld b,a              ; set pixels to move.
ogrvd0 call cangd          ; can we go down?
       jr nz,ogrvst        ; can't move down, so stop.
       inc (ix+8)          ; adjust new x coord.
       djnz ogrvd0
       ret
ogrvu  neg                 ; flip the sign so it's positive.
       ld b,a              ; set pixels to move.
ogrvu0 call cangu          ; can we go up?
       jr nz,ogrv2         ; can't move up, go down next.
       dec (ix+8)          ; adjust new x coord.
       djnz ogrvu0
       ret
ogrvst ld e,(ix+14)        ; get index to table.
       ld d,0              ; no high byte.
       ld hl,jtab          ; jump table.
       add hl,de           ; hl points to jump value.
       ld a,(hl)           ; fetch byte from table.
       cp 99               ; is it the end marker?
       ld (ix+13),0        ; reset jump flag.
       ld (ix+14),0        ; reset pointer.
       jp evftf
ogrv2  ld hl,jtab          ; jump table.
       ld b,0              ; offset into table.
ogrv4  ld a,(hl)           ; fetch table byte.
       cp 100              ; hit end or downward move?
       jr c,ogrv3          ; yes.
       inc hl              ; next byte of table.
       inc b               ; next offset.
       jr ogrv4            ; keep going until we find crest/end of table.
ogrv3  ld (ix+14),b        ; set next table offset.
       ret

; Initiate fall check.

ifall  ld a,(ix+13)        ; jump pointer flag.
       and a               ; are we in the air?
       ret nz              ; if set, we're already in the air.
       ld h,(ix+9)         ; y coordinate.
       ld a,16             ; look down 16 pixels.
       add a,(ix+8)        ; add x coordinate.
       ld l,a              ; coords in hl.
       ld (dispx),hl       ; set up test coordinates.
       call tstbl          ; get map address.
       call plchk          ; block, platform check.
       ret nz              ; it's solid, don't fall.
       inc hl              ; look right one cell.
       call plchk          ; block, platform check.
       ret nz              ; it's solid, don't fall.
       ld a,(dispy)        ; y coordinate.
       call rem5           ; position straddling block cells.
       jr z,ifalls         ; no more checks needed.
       inc hl              ; look to third cell.
       call plchk          ; block, platform check.
       ret nz              ; it's solid, don't fall.
ifalls inc (ix+13)         ; set in air flag.
       ld (ix+14),0        ; initial speed = 0.
       ret
tfall  ld a,(ix+13)        ; jump pointer flag.
       and a               ; are we in the air?
       ret nz              ; if set, we're already in the air.
       call ifall          ; do fall test.
       ld a,(ix+13)        ; get falling flag.
       and a               ; is it set?
       ret z               ; no.
       ld (ix+13),255      ; we're using the table.
       jr ogrv2            ; find position in table.


; Get frame data for a particular sprite.

gfrm   rlca                ; multiple of 2.
       ld e,a              ; copy to de.
       ld d,0              ; no high byte as max sprite is 128.
       ld hl,(frmptr)      ; frames used by game.
       add hl,de           ; point to frame start.
       ret

; Find sprite list for current room.

sprlst ld a,(scno)         ; screen number.
sprls2 ld hl,(nmeptr)      ; pointer to enemies.
       ld b,a              ; loop counter in b register.
       and a               ; is it the first screen?
       ret z               ; yes, don't need to search data.
       ld de,NMESIZ        ; bytes to skip.
sprls1 ld a,(hl)           ; fetch type of sprite.
       inc a               ; is it an end marker?
       jr z,sprls0         ; yes, end of this room.
       add hl,de           ; point to next sprite in list.
       jr sprls1           ; continue until end of room.
sprls0 inc hl              ; point to start of next screen.
       djnz sprls1         ; continue until room found.
       ret


; Clear all but a single player sprite.

nspr   ld b,NUMSPR         ; sprite slots in table.
       ld ix,sprtab        ; sprite table.
       ld de,TABSIZ        ; distance to next odd/even entry.
nspr0  ld a,(ix+0)         ; fetch sprite type.
       and a               ; is it a player?
       jr z,nspr1          ; yes, keep this one.
       ld (ix+0),255       ; delete sprite.
       ld (ix+5),255       ; remove next type.
       add ix,de           ; next sprite.
       djnz nspr0          ; one less space in the table.
       ret
nspr1  ld (ix+0),255       ; delete sprite.
       add ix,de           ; point to next sprite.
       djnz nspr2          ; one less to do.
       ret
nspr2  ld (ix+0),255       ; delete sprite.
       ld (ix+5),255       ; remove next type.
       add ix,de           ; next sprite.
       djnz nspr2          ; one less space in the table.
       ret


; Two initialisation routines.
; Initialise sprites - copy everything from list to table.

ispr   ld b,NUMSPR         ; sprite slots in table.
       ld ix,sprtab        ; sprite table.
ispr2  ld a,(hl)           ; fetch byte.
       cp 255              ; is it an end marker?
       ret z               ; yes, no more to do.
ispr1  ld a,(ix+0)         ; fetch sprite type.
       cp 255              ; is it enabled yet?
       jr nz,ispr4         ; yes, try another slot.
       ld a,(ix+5)         ; next type.
       cp 255              ; is it enabled yet?
       jr z,ispr3          ; no, process this one.
ispr4  ld de,TABSIZ        ; distance to next odd/even entry.
       add ix,de           ; next sprite.
       djnz ispr1          ; repeat for remaining sprites.
       ret                 ; no more room in table.
ispr3  call cpsp           ; initialise a sprite.
       djnz ispr2          ; one less space in the table.
       ret

; Initialise sprites - but not player, we're keeping the old one.

kspr   ld b,NUMSPR         ; sprite slots in table.
       ld ix,sprtab        ; sprite table.
kspr2  ld a,(hl)           ; fetch byte.
       cp 255              ; is it an end marker?
       ret z               ; yes, no more to do.
       and a               ; is it a player sprite?
       jr nz,kspr1         ; no, add to table as normal.
       ld de,NMESIZ        ; distance to next item in list.
       add hl,de           ; point to next one.
       jr kspr2
kspr1  ld a,(ix+0)         ; fetch sprite type.
       cp 255              ; is it enabled yet?
       jr nz,kspr4         ; yes, try another slot.
       ld a,(ix+5)         ; next type.
       cp 255              ; is it enabled yet?
       jr z,kspr3          ; no, process this one.
kspr4  ld de,TABSIZ        ; distance to next odd/even entry.
       add ix,de           ; next sprite.
       djnz kspr1          ; repeat for remaining sprites.
       ret                 ; no more room in table.
kspr3  call cpsp           ; copy sprite to table.
       djnz kspr2          ; one less space in the table.
       ret

; Copy sprite from list to table.

cpsp   ld a,(hl)           ; fetch byte from table.
       ld (ix+0),a         ; set up type.
       ld (ix+PAM1ST),a    ; set up type.
       inc hl              ; move to next byte.
       ld a,(hl)           ; fetch byte from table.
       ld (ix+6),a         ; set up image.
       inc hl              ; move to next byte.
       ld a,(hl)           ; fetch byte from table.
       ld (ix+3),200       ; set initial coordinate off screen.
       ld (ix+8),a         ; set up coordinate.
       inc hl              ; move to next byte.
       ld a,(hl)           ; fetch byte from table.
       ld (ix+9),a         ; set up coordinate.
       inc hl              ; move to next byte.
       xor a               ; zeroes in accumulator.
       ld (ix+7),a         ; reset frame number.
       ld (ix+10),a        ; reset direction.
;       ld (ix+12),a        ; reset parameter B.
       ld (ix+13),a        ; reset jump pointer low.
       ld (ix+14),a        ; reset jump pointer high.
       ld (ix+16),255      ; reset data pointer to auto-restore.
       push ix             ; store ix pair.
       push hl             ; store hl pair.
       push bc
evis0  call evnt09         ; perform event.
       pop bc
       pop hl              ; restore hl.
       pop ix              ; restore ix.
       ld de,TABSIZ        ; distance to next odd/even entry.
       add ix,de           ; next sprite.
       ret

; Clear the play area window.

clw    ld hl,(wintop)      ; get coordinates of window.
       ld (dispx),hl       ; put into dispx for calculation.
       ld a,(papmsk)       ; get paper colour.
       ld c,a              ; set colour in c register.
       call gprad          ; get print address.
       ex de,hl            ; flip screen address into hl.
       ld a,(winhgt)       ; height of window.
       rlca                ; multiply by 8 for block height.
       rlca
       rlca
       ld b,a              ; copy to b register.
clw1   push bc             ; store lines on stack.
       ld a,(winwid)       ; width of window.
       ld e,a              ; copy to e.
       rlca                ; multiply by 4.
       rlca
       add a,e             ; multiple of 5 = pixels to write.
       srl a               ; divide by 2 = bytes to write.
       push hl             ; store screen address.
clw0   ld (hl),c           ; write to screen.
       inc hl              ; next 2 pixels.
       dec a               ; one less byte.
       jr nz,clw0          ; repeat to end of window.
       pop hl              ; restore screen address.
       call nline2         ; down one line.
       pop bc              ; restore line counter.
       djnz clw1           ; repeat down the screen.
       ld hl,(wintop)      ; get coordinates of window.
       ld (charx),hl       ; put into display position.
       ret

; Redefine key.
; Go through table of 80 keys testing each one.
; Return code for first detected keypress.

redky  call debkey         ; debounce previous key.
       ld b,80             ; codes to check.
redky0 ld a,b              ; put code in accumulator.
       dec a               ; test 0 - 79, not 1 - 80.
       call 47902          ; check if key pressed.
       jr nz,redky1        ; pressed.
       djnz redky0         ; repeat until we've scanned them all.
       jr redky            ; repeat until something is pressed.
redky1 dec b               ; always one less.
       ret
       

; Effects code.
; Ticker routine is called 25 times per second.

scrly  ret
       defw txtscr         ; get screen address.
       ld b,8              ; 8 pixel rows.
       push hl             ; store screen address.
scrly1 push bc             ; store rows on stack.
       push hl

       ld a,(txtwid)       ; characters wide.
       ld b,a              ; copy to b.
       rlca                ; double it.
       rlca                ; now quadruple it.
       add a,b             ; quintuple width.
       srl a               ; divide by 2 for number of bytes to shift.
       ld b,a              ; put into the loop counter.
       ld d,0              ; clear previous pixels byte.
scrly0 ld a,(hl)           ; get byte from screen.
       ld e,a              ; copy to e.
       and 85              ; take right pixel.
       rlca                ; shift into left position.
       ld (hl),a           ; write to screen.
       ld a,d              ; value of 2 pixels to our right.
       and 170             ; we want the left pixel of that.
       rrca                ; shift into right position.
       or (hl)             ; merge with screen image.
       ld (hl),a           ; write to screen.
       ld d,e              ; get old value for next time round.
       dec hl              ; 2 pixels left.
       djnz scrly0         ; repeat for width of ticker message.
       pop hl
       call nline2         ; next screen row down.

       pop bc              ; retrieve row counter from stack.
       djnz scrly1         ; repeat for all rows.
       ld hl,(txtpos)      ; get text pointer.
       ld a,(hl)           ; find character we're displaying.
       and 127             ; remove end marker bit if applicable.
       cp 13               ; is it newline?
       jr nz,scrly5        ; no, it's okay.
       ld a,32             ; convert to a space instead.
scrly5 rlca
       rlca
       rlca                ; multiply by 8 to find char.
       ld b,a              ; store shift in b.
       and 3               ; keep within 768-byte range of font.
       ld d,a              ; that's our high byte.
       ld a,b              ; restore the shift.
       and 248
       ld e,a
       ld hl,font-256      ; font address.
       add hl,de           ; point to image of character.
       ex de,hl            ; need the address in de.
       pop hl              ; restore screen address.

       ld a,(txtbit)
       ld c,a
       ld b,8              ; height of ticker in pixels.
scrly3 ld a,(hl)           ; get screen byte.
       and 170             ; mask off right bit colours.
       ld (hl),a           ; write back to screen.
       ld a,(de)           ; get image of char line.
       and c               ; test relevant bit of char.
       jr z,scrly7         ; not set, write paper pixel.
       ld a,(inkmsk)       ; get ink colour.
       jr scrly2
scrly7 ld a,(papmsk)       ; get paper colour.
scrly2 and 85              ; remove left bit.
       or (hl)             ; merge with screen.
       ld (hl),a           ; write new bit.
       push bc             ; preserve row number and column bit.
       call nline2         ; next screen row down.
       pop bc              ; restore row number and column bit.
       inc de              ; next line of char.
       djnz scrly3
       ld hl,txtbit        ; bit of text to display.
       rr (hl)             ; next bit of char to use.
       ld a,(hl)           ; get bit value.
       cp 8                ; last valid value.
       ret nc              ; not reached end of character yet.
       ld (hl),128         ; start at leftmost bit again.
       ld hl,(txtpos)      ; text pointer.
       ld a,(hl)           ; what was the character?
       inc hl              ; next character in message.
       rla                 ; end of message?
;       ret nc              ; not yet, exit here.
;       ld a,201            ; code for ret.
;       ld (scrly),a        ; disable scrolling routine.
       jr nc,scrly6        ; not yet - continue.
scrly4 ld hl,(txtini)      ; start of scrolling message.
scrly6 ld (txtpos),hl      ; new text pointer position.
       ret

iscrly call preprt         ; set up display position.
       ld hl,msgdat        ; text messages.
       ld a,b              ; width.
       dec a               ; subtract one.
       cp 32               ; is it between 1 and 32?
       jr nc,iscrl0        ; no, disable messages.
       ld a,c              ; message number.
       ld d,b              ; copy width to d.
       call getwrd         ; find message start.
       ld b,d              ; restore width to b register.
       ld (txtini),hl      ; set initial text position.
       ld a,42             ; code for ld hl,(nn).
iscrl0 ld (scrly),a        ; enable/disable scrolling routine.
       call preprt         ; set up display position.
       call gprad          ; get print address.
       ld a,b              ; width of ticker.
       rlca                ; multiple of 2.
       rlca                ; multiple of 4.
       add a,b             ; now a multiple of 5.
       srl a               ; divide by 2 for number of bytes.
       ld l,a              ; copy to hl.
       ld h,0              ; no high byte.
       dec hl              ; left to end of ticker display.
       add hl,de           ; add width.
       ld (txtscr),hl      ; set text screen address.
       ld a,b              ; width.
       ld (txtwid),a       ; set width in working storage.
       ld hl,txtbit        ; bit of text to display.
       ld (hl),128         ; start with leftmost bit.
       jr scrly4

lstrnd equ $               ; end of "random" area.

; Sprite table.
; ix+0  = type.
; ix+1  = sprite image number.
; ix+2  = frame.
; ix+3  = x coord.
; ix+4  = y coord.

; ix+5  = new type.
; ix+6  = new image number.
; ix+7  = new frame.
; ix+8  = new x coord.
; ix+9  = new y coord.

; ix+10 = direction.
; ix+11 = parameter 1.
; ix+12 = parameter 2.
; ix+13 = jump pointer low.
; ix+14 = jump pointer high.
; ix+15 = data pointer low.
; ix+16 = data pointer high.


sprtab equ $

;       block NUMSPR * TABSIZ,255

       defb 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
       defb 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
       defb 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
       defb 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
       defb 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
       defb 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
       defb 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
       defb 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
       defb 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
       defb 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
       defb 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
       defb 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
ssprit defb 255,255,255,255,255,255,255,0,192,120,0,0,0,255,255,255,255

roomtb defb 34                     ; room number.
nosnd  defb 255

; Everything below here will be generated by the editors.

; Sounds.

fx1    defb 128+15         ; volume and mixer.
       defb 31             ; white noise.
       defw 1000           ; tone register.
       defb 128+15         ; volume and mixer.
       defb 25             ; white noise.
       defw 1000           ; tone register.
       defb 128+14         ; volume and mixer.
       defb 19             ; white noise.
       defw 1000           ; tone register.
       defb 128+13         ; volume and mixer.
       defb 13             ; white noise.
       defw 1000           ; tone register.
       defb 128+12         ; volume and mixer.
       defb 7              ; white noise.
       defw 1000           ; tone register.
       defb 128+11         ; volume and mixer.
       defb 0              ; white noise.
       defw 1000           ; tone register.
       defb 128+10         ; volume and mixer.
       defb 6              ; white noise.
       defw 1000           ; tone register.
       defb 128+8          ; volume and mixer.
       defb 12             ; white noise.
       defw 1000           ; tone register.
       defb 128+6          ; volume and mixer.
       defb 18             ; white noise.
       defw 1000           ; tone register.
       defb 128+3          ; volume and mixer.
       defb 24             ; white noise.
       defw 1000           ; tone register.
       defb 255

fx2    defb 064+15         ; volume and mixer.
       defb 27             ; white noise.
       defw 1000           ; tone register.
       defb 064+14         ; volume and mixer.
       defb 31             ; white noise.
       defw 2000           ; tone register.
       defb 064+13         ; volume and mixer.
       defb 28             ; white noise.
       defw 3000           ; tone register.
       defb 064+12         ; volume and mixer.
       defb 31             ; white noise.
       defw 2000           ; tone register.
       defb 064+11         ; volume and mixer.
       defb 29             ; white noise.
       defw 1000           ; tone register.
       defb 064+10         ; volume and mixer.
       defb 31             ; white noise.
       defw 2000           ; tone register.
       defb 064+9          ; volume and mixer.
       defb 30             ; white noise.
       defw 3000           ; tone register.
       defb 064+8          ; volume and mixer.
       defb 31             ; white noise.
       defw 2000           ; tone register.
       defb 064+7          ; volume and mixer.
       defb 31             ; white noise.
       defw 1000           ; tone register.
       defb 064+6          ; volume and mixer.
       defb 31             ; white noise.
       defw 2000           ; tone register.
       defb 255

fx3    defb 064+15         ; volume and mixer.
       defb 0              ; white noise.
       defw 4000           ; tone register.
       defb 064+15         ; volume and mixer.
       defb 0              ; white noise.
       defw 4100           ; tone register.
       defb 064+14         ; volume and mixer.
       defb 0              ; white noise.
       defw 4200           ; tone register.
       defb 064+14         ; volume and mixer.
       defb 0              ; white noise.
       defw 4300           ; tone register.
       defb 064+13         ; volume and mixer.
       defb 0              ; white noise.
       defw 4400           ; tone register.
       defb 064+13         ; volume and mixer.
       defb 0              ; white noise.
       defw 4500           ; tone register.
       defb 064+12         ; volume and mixer.
       defb 0              ; white noise.
       defw 4600           ; tone register.
       defb 064+12         ; volume and mixer.
       defb 0              ; white noise.
       defw 4700           ; tone register.
       defb 064+11         ; volume and mixer.
       defb 0              ; white noise.
       defw 4800           ; tone register.
       defb 064+10         ; volume and mixer.
       defb 0              ; white noise.
       defw 4900           ; tone register.
       defb 255

       defb 99             ; temporary marker.

       include "user.asm"


WINDOWTOP equ 3
WINDOWLFT equ 1
WINDOWHGT equ 20
WINDOWWID equ 30 ;
MAPWID equ 18
       defb 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
mapdat equ $
       defb 255,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,255
       defb 255,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,255,255
       defb 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
stmap  defb 1
evnt00 equ $
       ld a,(ix+9)
       ld (varv),a
       ld a,(ix+8)
       ld (varw),a
       ld a,(ix+10)
       ld (varu),a
       xor a
       ld hl,vars
       cp (hl)
       jp nz,a01115
       xor a
       ld hl,scno
       cp (hl)
       jp nz,a00082
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00082 ld a,1
       ld hl,scno
       cp (hl)
       jp nz,a00116
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00116 ld a,2
       ld hl,scno
       cp (hl)
       jp nz,a00150
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00150 ld a,3
       ld hl,scno
       cp (hl)
       jp nz,a00184
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00184 ld a,4
       ld hl,scno
       cp (hl)
       jp nz,a00218
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00218 ld a,5
       ld hl,scno
       cp (hl)
       jp nz,a00252
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00252 ld a,6
       ld hl,scno
       cp (hl)
       jp nz,a00286
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00286 ld a,7
       ld hl,scno
       cp (hl)
       jp nz,a00320
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00320 ld a,8
       ld hl,scno
       cp (hl)
       jp nz,a00354
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00354 ld a,9
       ld hl,scno
       cp (hl)
       jp nz,a00388
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00388 ld a,10
       ld hl,scno
       cp (hl)
       jp nz,a00422
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00422 ld a,10
       ld hl,scno
       cp (hl)
       jp nz,a00456
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00456 ld a,11
       ld hl,scno
       cp (hl)
       jp nz,a00491
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00491 ld a,12
       ld hl,scno
       cp (hl)
       jp nz,a00525
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00525 ld a,13
       ld hl,scno
       cp (hl)
       jp nz,a00559
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00559 ld a,14
       ld hl,scno
       cp (hl)
       jp nz,a00593
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00593 ld a,15
       ld hl,scno
       cp (hl)
       jp nz,a00628
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00628 ld a,16
       ld hl,scno
       cp (hl)
       jp nz,a00662
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00662 ld a,17
       ld hl,scno
       cp (hl)
       jp nz,a00696
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00696 ld a,18
       ld hl,scno
       cp (hl)
       jp nz,a00730
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00730 ld a,19
       ld hl,scno
       cp (hl)
       jp nz,a00765
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00765 ld a,20
       ld hl,scno
       cp (hl)
       jp nz,a00799
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00799 ld a,21
       ld hl,scno
       cp (hl)
       jp nz,a00833
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00833 ld a,22
       ld hl,scno
       cp (hl)
       jp nz,a00867
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00867 ld a,23
       ld hl,scno
       cp (hl)
       jp nz,a00902
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00902 ld a,24
       ld hl,scno
       cp (hl)
       jp nz,a00936
       ld a,64
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00936 ld a,25
       ld hl,scno
       cp (hl)
       jp nz,a00970
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a00970 ld a,26
       ld hl,scno
       cp (hl)
       jp nz,a01004
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a01004 ld a,27
       ld hl,scno
       cp (hl)
       jp nz,a01039
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a01039 ld a,28
       ld hl,scno
       cp (hl)
       jp nz,a01073
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a01073 ld a,29
       ld hl,scno
       cp (hl)
       jp nz,a01107
       ld a,16
       ld (ix+9),a
       ld a,152
       ld (ix+8),a
a01107 ld a,1
       ld (vars),a
a01115 call skobj
       ld (varobj),a
       ld a,3
       ld hl,varobj
       cp (hl)
       jp nz,a01215
       ld c,10
       ld a,(varb)
       add a,c
       ld (varb),a
       ld a,(varobj)
       call getob
       ld hl,fx1
       ld de,205
       add hl,de
       call isnd
       ld hl,5888
       ld (charx),hl
       ld a,(varb)
       ld bc,displ0
       call num2td
       call displ1
a01215 call getcol
       call ifall
       ld b,DEADLY
       call tded
       cp b
       jp nz,a01267
       xor a
       ld hl,varc
       cp (hl)
       jp nz,a01267
       ld hl,deadf
       ld (hl),h
a01267 ld a,(joyval)
       and 4
       jp z,a01350
       ld a,2
       ld (ix+10),a
       ld a,1
       ld (ix+6),a
       ld a,(wnlftx)
       cp (ix+9)
       jp c,a01328
       ld a,(wnlftx)
       ld (ix+9),a
       ret
       jp a01350
a01328 call cangl
       jp nz,a01350
       dec (ix+9)
       xor a
       call animsp
a01350 ld a,(joyval)
       and 8
       jp z,a01437
       ld a,3
       ld (ix+10),a
       xor a
       ld (ix+6),a
       ld a,(wnrgtx)
       cp (ix+9)
       jr z,$+5
       jp nc,a01415
       ld a,(wnrgtx)
       ld (ix+9),a
       ret
       jp a01437
a01415 call cangr
       jp nz,a01437
       inc (ix+9)
       xor a
       call animsp
a01437 ld a,(joyval)
       and 1
       jp z,a01497
       call laddu
       jp nz,a01490
       ld a,4
       ld (ix+6),a
       xor a
       call animsp
       dec (ix+8)
       dec (ix+8)
       jp a01497
a01490 ld a,9
       call jump
a01497 ld a,(joyval)
       and 2
       jp z,a01545
       call laddd
       jp nz,a01545
       ld a,4
       ld (ix+6),a
       xor a
       call animsp
       inc (ix+8)
       inc (ix+8)
a01545 ld a,(joyval)
       and 16
       jp z,a01825
       ld a,1
       ld hl,vard
       cp (hl)
       jp nz,a01817
       xor a
       ld hl,varb
       cp (hl)
       jp nc,a01791
       ld hl,fx1
       ld de,82
       add hl,de
       call isnd
       ld bc,2305
       call spawn
       ld a,(ix+10)
       ld (varu),a
       ld a,(ix+9)
       ld (varv),a
       ld a,(ix+8)
       ld (varw),a
       ld a,3
       cp (ix+10)
       jp nz,a01681
       ld c,16
       ld a,(varv)
       add a,c
       ld (varv),a
       jp a01697
a01681 ld c,16
       ld a,(varv)
       sub c
       ld (varv),a
a01697 ld ix,(spptr)
       ld a,(varu)
       ld (ix+10),a
       xor a
       ld (ix+11),a
       ld a,(varv)
       ld (ix+9),a
       ld a,(varw)
       ld (ix+8),a
       ld ix,(ogptr)
       ld a,(varb)
       dec a
       ld (varb),a
       ld hl,5888
       ld (charx),hl
       ld a,(varb)
       ld bc,displ0
       call num2td
       call displ1
       jp a01817
a01791 ld hl,deadf
       ld (hl),h
       ld c,100
       ld a,(varb)
       add a,c
       ld (varb),a
a01817 xor a
       ld (vard),a
a01825 ld a,(joyval)
       and 32
       jp z,a01885
       call laddu
       jp nz,a01878
       ld a,4
       ld (ix+6),a
       xor a
       call animsp
       dec (ix+8)
       dec (ix+8)
       jp a01885
a01878 ld a,9
       call jump
a01885 ld a,(keys+6)
       call ktest
       jp c,a01947
       call laddu
       jp nz,a01939
       ld a,4
       ld (ix+6),a
       xor a
       call animsp
       dec (ix+8)
       dec (ix+8)
       jp a01947
a01939 ld a,9
       call jump
a01947 ld b,1
       call sktyp
       jp nc,a02076
       ld a,6
       ld (loopa),a
a01968 ld a,6
       ld c,a
       ld b,a
       call 48184
       ld a,7
       ld c,a
       ld b,a
       call 48184
       ld hl,loopa
       dec (hl)
       jp nz,a01968
       ld a,1
       ld c,a
       ld b,a
       call 48184
       ld hl,fx1
       ld de,205
       add hl,de
       call isnd
       ld ix,(skptr)
       ld (ix+5),255
       ld ix,(ogptr)
       ld a,1
       ld (vard),a
       ld a,1
       ld (varo),a
a02076 jp grav
evnt01 equ $
       xor a
       cp (ix+11)
       jp nz,b00028
       ld a,9
       ld (ix+6),a
       jp b00050
b00028 ld a,1
       cp (ix+11)
       jp nz,b00050
       ld a,39
       ld (ix+6),a
b00050 xor a
       cp (ix+10)
       jp nz,b00137
       ld a,6
       ld (loopa),a
b00071 call cangu
       jp nz,b00118
       dec (ix+8)
       dec (ix+8)
       ld a,(wntopx)
       cp (ix+8)
       jp c,b00114
       jp z,b00114
       ld (ix+5),255
b00114 jp b00123
b00118 ld (ix+5),255
b00123 ld hl,loopa
       dec (hl)
       jp nz,b00071
b00137 ld a,1
       cp (ix+10)
       jp nz,b00220
       ld a,6
       ld (loopa),a
b00158 call cangd
       jp nz,b00201
       inc (ix+8)
       inc (ix+8)
       ld a,(wnbotx)
       cp (ix+8)
       jp nc,b00197
       ld (ix+5),255
b00197 jp b00206
b00201 ld (ix+5),255
b00206 ld hl,loopa
       dec (hl)
       jp nz,b00158
b00220 ld a,2
       cp (ix+10)
       jp nz,b00336
       ld a,6
       ld (loopa),a
b00241 call cangl
       jp nz,b00300
       dec (ix+9)
       ld a,(wnlftx)
       cp (ix+9)
       jp c,b00296
       jp z,b00296
       ld a,1
       ld (vard),a
       ld a,1
       ld (varo),a
       ld (ix+5),255
b00296 jp b00322
b00300 ld a,1
       ld (vard),a
       ld a,1
       ld (varo),a
       ld (ix+5),255
b00322 ld hl,loopa
       dec (hl)
       jp nz,b00241
b00336 ld a,3
       cp (ix+10)
       jp nz,b00447
       ld a,6
       ld (loopa),a
b00357 call cangr
       jp nz,b00412
       inc (ix+9)
       ld a,(wnrgtx)
       cp (ix+8)
       jp nc,b00408
       ld a,1
       ld (vard),a
       ld a,1
       ld (varo),a
       ld (ix+5),255
b00408 jp b00434
b00412 ld a,1
       ld (vard),a
       ld a,1
       ld (varo),a
       ld (ix+5),255
b00434 ld hl,loopa
       dec (hl)
       jp nz,b00357
b00447 xor a
       call animsp
       ret
evnt02 equ $
       xor a
       ld hl,vara
       cp (hl)
       jp z,c00568
       ld a,2
       cp (ix+10)
       jp nz,c00298
       xor a
       cp (ix+11)
       jp nz,c00053
       ld a,11
       ld (ix+6),a
c00053 ld a,1
       cp (ix+11)
       jp nz,c00075
       ld a,13
       ld (ix+6),a
c00075 ld a,2
       cp (ix+11)
       jp nz,c00096
       ld a,15
       ld (ix+6),a
c00096 ld a,3
       cp (ix+11)
       jp nz,c00118
       ld a,17
       ld (ix+6),a
c00118 ld a,4
       cp (ix+11)
       jp nz,c00139
       ld a,19
       ld (ix+6),a
c00139 ld a,5
       cp (ix+11)
       jp nz,c00161
       ld a,21
       ld (ix+6),a
c00161 ld a,6
       cp (ix+11)
       jp nz,c00182
       ld a,23
       ld (ix+6),a
c00182 ld c,9
       ld a,(ix+9)
       sub c
       ld (ix+9),a
       call cangd
       jp nz,c00217
       ld a,3
       ld (ix+10),a
c00217 ld c,9
       ld a,(ix+9)
       add a,c
       ld (ix+9),a
       call cangl
       jp nz,c00285
       ld a,(ix+9)
       dec a
       ld (ix+9),a
       ld b,DEADLY
       call tded
       cp b
       jp nz,c00281
       ld a,3
       ld (ix+10),a
c00281 jp c00294
c00285 ld a,3
       ld (ix+10),a
c00294 jp c00560
c00298 xor a
       cp (ix+11)
       jp nz,c00319
       ld a,10
       ld (ix+6),a
c00319 ld a,1
       cp (ix+11)
       jp nz,c00341
       ld a,12
       ld (ix+6),a
c00341 ld a,2
       cp (ix+11)
       jp nz,c00362
       ld a,14
       ld (ix+6),a
c00362 ld a,3
       cp (ix+11)
       jp nz,c00384
       ld a,16
       ld (ix+6),a
c00384 ld a,4
       cp (ix+11)
       jp nz,c00405
       ld a,18
       ld (ix+6),a
c00405 ld a,5
       cp (ix+11)
       jp nz,c00427
       ld a,20
       ld (ix+6),a
c00427 ld a,6
       cp (ix+11)
       jp nz,c00448
       ld a,24
       ld (ix+6),a
c00448 ld c,9
       ld a,(ix+9)
       add a,c
       ld (ix+9),a
       call cangd
       jp nz,c00483
       ld a,2
       ld (ix+10),a
c00483 ld c,9
       ld a,(ix+9)
       sub c
       ld (ix+9),a
       call cangr
       jp nz,c00551
       ld a,(ix+9)
       inc a
       ld (ix+9),a
       ld b,DEADLY
       call tded
       cp b
       jp nz,c00547
       ld a,2
       ld (ix+10),a
c00547 jp c00560
c00551 ld a,2
       ld (ix+10),a
c00560 ld a,3
       call animsp
c00568 ld b,0
       call sktyp
       jp nc,c00854
       ld a,7
       cp (ix+11)
       jp nz,c00828
       ld a,(ix+8)
       ld (varq),a
       ld ix,(skptr)
       ld a,(ix+8)
       ld hl,varq
       cp (hl)
       jp nc,c00742
       ld a,(ix+8)
       ld c,a
       ld a,(varq)
       sub c
       ld (varq),a
       ld a,2
       ld hl,varq
       cp (hl)
       jp nc,c00717
       ld a,(ix+8)
       ld c,a
       ld a,(varq)
       add a,c
       ld (varq),a
       ld a,(varq)
       ld (ix+8),a
       ld c,16
       ld a,(ix+8)
       sub c
       ld (ix+8),a
       call gravst
c00717 ld a,(joyval)
       and 1
       jp z,c00738
       ld a,9
       call jump
c00738 jp c00824
c00742 ld a,(ix+8)
       ld c,a
       ld a,(varq)
       sub c
       ld (varq),a
       call cangd
       jp nz,c00815
       ld a,2
       ld hl,varq
       cp (hl)
       jp nc,c00811
       ld c,2
       ld a,(ix+8)
       add a,c
       ld (ix+8),a
       call ifall
c00811 jp c00824
c00815 ld hl,deadf
       ld (hl),h
c00824 jp c00854
c00828 xor a
       ld hl,varc
       cp (hl)
       jp nz,c00854
       ld hl,deadf
       ld (hl),h
c00854 ld b,1
       call sktyp
       jp nc,c01248
       ld a,6
       cp (ix+11)
       jp c,c01248
       jp z,c01248
       ld a,30
       call explod
       ld ix,(skptr)
       xor a
       cp (ix+11)
       jp nz,c00923
       xor a
       ld (varq),a
       jp c00931
c00923 ld a,1
       ld (varq),a
c00931 ld a,1
       ld (vard),a
       ld a,1
       ld (varo),a
       ld ix,(ogptr)
       xor a
       ld hl,varq
       cp (hl)
       jp nz,c01248
       ld a,12
       cp (ix+6)
       jp nz,c01012
       ld a,(ix+9)
       ld h,a
       ld a,(ix+8)
       ld l,a
       ld (dispx),hl
       ld a,3
       call drpob
c01012 ld a,13
       cp (ix+6)
       jp nz,c01055
       ld a,(ix+9)
       ld h,a
       ld a,(ix+8)
       ld l,a
       ld (dispx),hl
       ld a,3
       call drpob
c01055 ld a,16
       cp (ix+6)
       jp nz,c01098
       ld a,(ix+9)
       ld h,a
       ld a,(ix+8)
       ld l,a
       ld (dispx),hl
       ld a,3
       call drpob
c01098 ld a,17
       cp (ix+6)
       jp nz,c01140
       ld a,(ix+9)
       ld h,a
       ld a,(ix+8)
       ld l,a
       ld (dispx),hl
       ld a,3
       call drpob
c01140 ld a,20
       cp (ix+6)
       jp nz,c01183
       ld a,(ix+9)
       ld h,a
       ld a,(ix+8)
       ld l,a
       ld (dispx),hl
       ld a,3
       call drpob
c01183 ld a,21
       cp (ix+6)
       jp nz,c01226
       ld a,(ix+9)
       ld h,a
       ld a,(ix+8)
       ld l,a
       ld (dispx),hl
       ld a,3
       call drpob
c01226 ld (ix+5),255
       ld hl,fx1
       ld de,205
       add hl,de
       call isnd
c01248 jp grav
evnt03 equ $
       ld a,(ix+9)
       ld (varf),a
       ld a,(ix+8)
       ld (varg),a
       ld a,1
       ld hl,varr
       cp (hl)
       jp nz,d00097
       ld a,(varz)
       inc a
       ld (varz),a
       ld a,4
       ld hl,varz
       cp (hl)
       jp nz,d00088
       xor a
       ld (varz),a
       xor a
       ld (ix+7),a
       jp d00097
d00088 ld a,1
       ld (ix+7),a
d00097 ld a,1
       ld hl,varz
       cp (hl)
       jr z,$+5
       jp nc,d00515
       xor a
       cp (ix+11)
       jp nz,d00201
       ld a,2
       cp (ix+10)
       jp nz,d00174
       call cangl
       jp nz,d00161
       dec (ix+9)
       jp d00170
d00161 ld a,3
       ld (ix+10),a
d00170 jp d00201
d00174 call cangr
       jp nz,d00192
       inc (ix+9)
       jp d00201
d00192 ld a,2
       ld (ix+10),a
d00201 ld a,1
       cp (ix+11)
       jp nz,d00293
       xor a
       cp (ix+10)
       jp nz,d00262
       call cangu
       jp nz,d00249
       dec (ix+8)
       dec (ix+8)
       jp d00258
d00249 ld a,1
       ld (ix+10),a
d00258 jp d00293
d00262 call cangd
       jp nz,d00285
       inc (ix+8)
       inc (ix+8)
       jp d00293
d00285 xor a
       ld (ix+10),a
d00293 ld a,2
       cp (ix+11)
       jp nz,d00511
       ld a,2
       ld hl,varz
       cp (hl)
       jp nz,d00511
       ld a,(varw)
       cp (ix+8)
       jp nc,d00367
       call cangu
       jp nz,d00367
       ld a,(ix+8)
       dec a
       ld (ix+8),a
       xor a
       ld (ix+10),a
d00367 ld a,(varw)
       cp (ix+8)
       jp c,d00417
       jp z,d00417
       call cangd
       jp nz,d00417
       ld a,(ix+8)
       inc a
       ld (ix+8),a
       ld a,1
       ld (ix+10),a
d00417 ld a,(varv)
       cp (ix+9)
       jp nc,d00461
       call cangl
       jp nz,d00461
       ld a,(ix+9)
       dec a
       ld (ix+9),a
       ld a,2
       ld (ix+10),a
d00461 ld a,(varv)
       cp (ix+9)
       jp c,d00511
       jp z,d00511
       call cangr
       jp nz,d00511
       ld a,(ix+9)
       inc a
       ld (ix+9),a
       ld a,3
       ld (ix+10),a
d00511 jp d00954
d00515 ld a,(varw)
       cp (ix+8)
       jp nc,d00547
       call cangu
       jp nz,d00547
       dec (ix+8)
       dec (ix+8)
d00547 ld a,(varw)
       cp (ix+8)
       jp c,d00584
       jp z,d00584
       call cangd
       jp nz,d00584
       inc (ix+8)
       inc (ix+8)
d00584 ld a,(varv)
       cp (ix+9)
       jp nc,d00612
       call cangl
       jp nz,d00612
       dec (ix+9)
d00612 ld a,(varv)
       cp (ix+9)
       jp c,d00645
       jp z,d00645
       call cangr
       jp nz,d00645
       inc (ix+9)
d00645 ld a,1
       ld hl,varo
       cp (hl)
       jp nz,d00954
       ld hl,fx1
       ld de,82
       add hl,de
       call isnd
       ld bc,2305
       call spawn
       ld a,(ix+9)
       ld (varv),a
       ld a,(ix+8)
       ld (varw),a
       ld a,1
       cp (ix+10)
       jp nz,d00749
       ld c,20
       ld a,(varv)
       add a,c
       ld (varv),a
       ld a,1
       ld (varu),a
       jp d00890
d00749 ld a,2
       cp (ix+10)
       jp nz,d00791
       ld c,16
       ld a,(varv)
       sub c
       ld (varv),a
       xor a
       ld (varu),a
       jp d00890
d00791 ld a,2
       ld d,a
       call random
       ld h,a
       call imul
       ld a,h
       ld (varrnd),a
       xor a
       ld hl,varrnd
       cp (hl)
       jp nz,d00865
       ld c,20
       ld a,(varv)
       sub c
       ld (varv),a
       ld a,2
       ld (varu),a
       jp d00890
d00865 ld c,20
       ld a,(varv)
       add a,c
       ld (varv),a
       ld a,3
       ld (varu),a
d00890 ld ix,(spptr)
       ld a,1
       ld (ix+11),a
       ld a,29
       ld (ix+6),a
       ld a,(varu)
       ld (ix+10),a
       ld a,(varv)
       ld (ix+9),a
       ld a,(varw)
       ld (ix+8),a
       ld ix,(ogptr)
       xor a
       ld (varo),a
d00954 ld b,0
       call sktyp
       jp nc,d00993
       xor a
       ld hl,varc
       cp (hl)
       jp nz,d00993
       ld hl,deadf
       ld (hl),h
d00993 ld b,1
       call sktyp
       jp nc,d01298
       ld hl,fx1
       ld de,164
       add hl,de
       call isnd
       ld a,(vark)
       dec a
       ld (vark),a
       xor a
       ld hl,vark
       cp (hl)
       jp c,d01074
       ld a,30
       call explod
       ld (ix+5),255
       xor a
       ld (vara),a
d01074 ld a,1
       ld (vard),a
       ld hl,515
       ld (charx),hl
       ld a,2
       call dmsg
       ld hl,1795
       ld (charx),hl
       ld a,(vark)
       ld bc,displ0
       call num2dd
       call displ1
       ld hl,2563
       ld (charx),hl
       ld a,3
       call dmsg
       ld a,80
       ld (varn),a
       ld a,(vark)
       ld (varw),a
       ld a,(varn)
       ld c,a
       ld a,(varw)
       add a,c
       ld (varw),a
d01185 ld a,(varw)
       ld hl,varn
       cp (hl)
       jp c,d01298
       jp z,d01298
       ld a,24
       ld (varv),a
d01216 ld a,32
       ld hl,varv
       cp (hl)
       jp c,d01281
       jp z,d01281
       ld a,(varn)
       ld h,a
       ld a,(varv)
       ld l,a
       ld (dispx),hl
       call plot0
       ld a,(varv)
       inc a
       ld (varv),a
       jp d01216
d01281 ld a,(varn)
       inc a
       ld (varn),a
       jp d01185
d01298 ret
evnt04 equ $
       xor a
       ld hl,vara
       cp (hl)
       jp z,e00157
       xor a
       cp (ix+10)
       jp nz,e00093
       call cangu
       jp nz,e00080
       dec (ix+8)
       dec (ix+8)
       ld b,DEADLY
       call tded
       cp b
       jp nz,e00076
       ld a,1
       ld (ix+10),a
e00076 jp e00089
e00080 ld a,1
       ld (ix+10),a
e00089 jp e00149
e00093 call cangd
       jp nz,e00141
       inc (ix+8)
       inc (ix+8)
       ld b,DEADLY
       call tded
       cp b
       jp nz,e00137
       xor a
       ld (ix+10),a
e00137 jp e00149
e00141 xor a
       ld (ix+10),a
e00149 ld a,3
       call animsp
e00157 ld b,0
       call sktyp
       jp nc,e00448
       xor a
       cp (ix+11)
       jp nz,e00213
       xor a
       ld hl,varc
       cp (hl)
       jp nz,e00209
       ld hl,deadf
       ld (hl),h
e00209 jp e00448
e00213 ld a,(ix+8)
       ld (varq),a
       ld ix,(skptr)
       ld a,(ix+8)
       ld hl,varq
       cp (hl)
       jp nc,e00361
       ld a,(ix+8)
       ld c,a
       ld a,(varq)
       sub c
       ld (varq),a
       ld a,2
       ld hl,varq
       cp (hl)
       jp nc,e00336
       ld a,(ix+8)
       ld c,a
       ld a,(varq)
       add a,c
       ld (varq),a
       ld a,(varq)
       ld (ix+8),a
       ld c,16
       ld a,(ix+8)
       sub c
       ld (ix+8),a
       call gravst
e00336 ld a,(joyval)
       and 1
       jp z,e00357
       ld a,9
       call jump
e00357 jp e00443
e00361 ld a,(ix+8)
       ld c,a
       ld a,(varq)
       sub c
       ld (varq),a
       call cangd
       jp nz,e00434
       ld a,2
       ld hl,varq
       cp (hl)
       jp nc,e00429
       ld c,2
       ld a,(ix+8)
       add a,c
       ld (ix+8),a
       call ifall
e00429 jp e00443
e00434 ld hl,deadf
       ld (hl),h
e00443 ld ix,(ogptr)
e00448 ld b,1
       call sktyp
       jp nc,e00506
       xor a
       cp (ix+11)
       jp nz,e00506
       ld a,30
       call explod
       ld ix,(skptr)
       ld ix,(ogptr)
       ld (ix+5),255
       ld a,1
       ld (vard),a
e00506 jp grav
evnt05 equ $
       xor a
       ld hl,vara
       cp (hl)
       jp z,f00209
       ld a,2
       cp (ix+11)
       jp nz,f00071
       call cangl
       jp nz,f00050
       dec (ix+9)
       jp f00067
f00050 ld a,3
       ld (ix+11),a
       xor a
       ld (ix+7),a
f00067 jp f00106
f00071 call cangr
       jp nz,f00089
       inc (ix+9)
       jp f00106
f00089 ld a,2
       ld (ix+11),a
       xor a
       ld (ix+7),a
f00106 xor a
       cp (ix+10)
       jp nz,f00162
       call cangu
       jp nz,f00141
       dec (ix+8)
       dec (ix+8)
       jp f00158
f00141 ld a,1
       ld (ix+10),a
       xor a
       ld (ix+7),a
f00158 jp f00201
f00162 call cangd
       jp nz,f00185
       inc (ix+8)
       inc (ix+8)
       jp f00201
f00185 xor a
       ld (ix+10),a
       xor a
       ld (ix+7),a
f00201 ld a,3
       call animsp
f00209 ld b,0
       call sktyp
       jp nc,f00248
       xor a
       ld hl,varc
       cp (hl)
       jp nz,f00248
       ld hl,deadf
       ld (hl),h
f00248 ld b,1
       call sktyp
       jp nc,f00329
       ld a,(ix+12)
       inc a
       ld (ix+12),a
       ld a,10
       cp (ix+12)
       jr z,$+5
       jp nc,f00329
       ld a,30
       call explod
       ld ix,(skptr)
       ld (ix+5),255
       ld ix,(ogptr)
       ld (ix+5),255
       ld a,1
       ld (vard),a
f00329 ret
evnt06 equ $
       xor a
       ld hl,varz
       cp (hl)
       jp nz,g00024
       jp g00024
g00024 xor a
       cp (ix+11)
       jp nz,g00110
       xor a
       ld hl,varp
       cp (hl)
       jp nz,g00076
       ld a,(varv)
       ld (ix+9),a
       ld a,(varw)
       ld (ix+8),a
       jp g00110
g00076 ld a,1
       ld hl,varp
       cp (hl)
       jp nz,g00110
       xor a
       ld (ix+9),a
       ld a,212
       ld (ix+8),a
g00110 ld a,1
       cp (ix+11)
       jp nz,g00320
       ld a,(varf)
       ld (ix+9),a
       ld a,(varg)
       ld (ix+8),a
       ld c,16
       ld a,(ix+8)
       add a,c
       ld (ix+8),a
       ld b,0
       call sktyp
       jp nc,g00197
       xor a
       ld hl,varc
       cp (hl)
       jp nz,g00197
       ld hl,deadf
       ld (hl),h
g00197 ld b,1
       call sktyp
       jp nc,g00299
       ld hl,fx1
       ld de,164
       add hl,de
       call isnd
       ld a,(vark)
       dec a
       ld (vark),a
       ld a,1
       ld hl,vark
       cp (hl)
       jp c,g00275
       jp z,g00275
       ld a,30
       call explod
       ld (ix+5),255
g00275 ld ix,(skptr)
       ld (ix+5),255
       ld ix,(ogptr)
       ld a,1
       ld (vard),a
g00299 xor a
       ld hl,vark
       cp (hl)
       jp c,g00320
       ld (ix+5),255
g00320 ret
evnt07 equ $
       xor a
       ld hl,vara
       cp (hl)
       jp z,h00535
       xor a
       cp (ix+10)
       jp nz,h00121
       call laddu
       jp nz,h00055
       dec (ix+8)
       dec (ix+8)
       jp h00121
h00055 ld a,2
       ld d,a
       call random
       ld h,a
       call imul
       ld a,h
       ld (varrnd),a
       xor a
       ld hl,varrnd
       cp (hl)
       jp nz,h00113
       ld a,2
       ld (ix+10),a
       jp h00121
h00113 ld a,3
       ld (ix+10),a
h00121 ld a,1
       cp (ix+10)
       jp nz,h00223
       call laddd
       jp nz,h00157
       inc (ix+8)
       inc (ix+8)
       jp h00223
h00157 ld a,2
       ld d,a
       call random
       ld h,a
       call imul
       ld a,h
       ld (varrnd),a
       xor a
       ld hl,varrnd
       cp (hl)
       jp nz,h00215
       ld a,2
       ld (ix+10),a
       jp h00223
h00215 ld a,3
       ld (ix+10),a
h00223 ld a,2
       cp (ix+10)
       jp nz,h00375
       call cangd
       jp nz,h00263
       inc (ix+9)
       ld a,3
       ld (ix+10),a
       jp h00375
h00263 call cangl
       jp nz,h00281
       dec (ix+9)
       jp h00290
h00281 ld a,3
       ld (ix+10),a
h00290 ld a,4
       ld d,a
       call random
       ld h,a
       call imul
       ld a,h
       ld (varrnd),a
       xor a
       ld hl,varrnd
       cp (hl)
       jp nz,h00375
       call laddu
       jp nz,h00357
       xor a
       ld (ix+10),a
       jp h00375
h00357 call laddd
       jp nz,h00375
       ld a,1
       ld (ix+10),a
h00375 ld a,3
       cp (ix+10)
       jp nz,h00527
       call cangd
       jp nz,h00415
       dec (ix+9)
       ld a,2
       ld (ix+10),a
       jp h00527
h00415 call cangr
       jp nz,h00433
       inc (ix+9)
       jp h00442
h00433 ld a,2
       ld (ix+10),a
h00442 ld a,4
       ld d,a
       call random
       ld h,a
       call imul
       ld a,h
       ld (varrnd),a
       xor a
       ld hl,varrnd
       cp (hl)
       jp nz,h00527
       call laddu
       jp nz,h00509
       xor a
       ld (ix+10),a
       jp h00527
h00509 call laddd
       jp nz,h00527
       ld a,1
       ld (ix+10),a
h00527 ld a,3
       call animsp
h00535 ld b,0
       call sktyp
       jp nc,h00548
h00548 ret
evnt08 equ $
       xor a
       cp (ix+11)
       jp nz,i00225
       ld a,2
       xor a
       ld (ix+7),a
       ld a,(varh)
       ld l,a
       ld a,(vari)
       ld h,a
       ld (charx),hl
       ld hl,(charx)
       ld (dispx),hl
       ld a,9
       call pattr
       ld hl,(dispx)
       ld (charx),hl
       ld a,(varh)
       inc a
       ld (varh),a
       ld a,(varh)
       ld l,a
       ld a,(vari)
       ld h,a
       ld (charx),hl
       ld hl,(charx)
       ld (dispx),hl
       ld a,9
       call pattr
       ld hl,(dispx)
       ld (charx),hl
       ld a,(varh)
       inc a
       ld (varh),a
       ld a,(varh)
       ld l,a
       ld a,(vari)
       ld h,a
       ld (charx),hl
       ld hl,(charx)
       ld (dispx),hl
       ld a,9
       call pattr
       ld hl,(dispx)
       ld (charx),hl
       ld c,2
       ld a,(varh)
       sub c
       ld (varh),a
       jp i00440
i00225 ld a,4
       ld a,1
       ld (ix+7),a
       ld hl,7177
       ld (charx),hl
       ld a,(varh)
       ld l,a
       ld a,(vari)
       ld h,a
       ld (charx),hl
       ld hl,(charx)
       ld (dispx),hl
       xor a
       call pattr
       ld hl,(dispx)
       ld (charx),hl
       ld a,(varh)
       inc a
       ld (varh),a
       ld a,(varh)
       ld l,a
       ld a,(vari)
       ld h,a
       ld (charx),hl
       ld hl,(charx)
       ld (dispx),hl
       xor a
       call pattr
       ld hl,(dispx)
       ld (charx),hl
       ld a,(varh)
       inc a
       ld (varh),a
       ld a,(varh)
       ld l,a
       ld a,(vari)
       ld h,a
       ld (charx),hl
       ld hl,(charx)
       ld (dispx),hl
       xor a
       call pattr
       ld hl,(dispx)
       ld (charx),hl
       ld c,2
       ld a,(varh)
       sub c
       ld (varh),a
i00440 ld b,0
       call sktyp
       jp nc,i00491
       xor a
       cp (ix+11)
       jp nz,i00491
       ld a,1
       ld (ix+11),a
       ld hl,fx1
       ld de,0
       add hl,de
       call isnd
i00491 ret
evnt09 equ $
       xor a
       cp (ix+5)
       jp nz,j00036
       ld a,6
       ld a,3
       ld (ix+10),a
       xor a
       ld (ix+11),a
j00036 ld a,1
       cp (ix+5)
       jp nz,j00052
       ld a,2
j00052 ld a,2
       cp (ix+5)
       jp nz,j00269
       ld a,5
       ld a,10
       cp (ix+6)
       jp nz,j00093
       xor a
       ld (ix+11),a
       ld a,5
j00093 ld a,12
       cp (ix+6)
       jp nz,j00118
       ld a,7
       ld a,1
       ld (ix+11),a
j00118 ld a,14
       cp (ix+6)
       jp nz,j00143
       ld a,8
       ld a,2
       ld (ix+11),a
j00143 ld a,16
       cp (ix+6)
       jp nz,j00168
       ld a,9
       ld a,3
       ld (ix+11),a
j00168 ld a,18
       cp (ix+6)
       jp nz,j00193
       ld a,10
       ld a,4
       ld (ix+11),a
j00193 ld a,20
       cp (ix+6)
       jp nz,j00218
       ld a,11
       ld a,5
       ld (ix+11),a
j00218 ld a,23
       cp (ix+6)
       jp nz,j00244
       ld a,11
       ld a,6
       ld (ix+11),a
j00244 ld a,31
       cp (ix+6)
       jp nz,j00269
       ld a,11
       ld a,7
       ld (ix+11),a
j00269 ld a,3
       cp (ix+5)
       jp nz,j00357
       ld a,33
       cp (ix+6)
       jp nz,j00307
       ld a,10
       xor a
       ld (ix+11),a
j00307 ld a,35
       cp (ix+6)
       jp nz,j00332
       ld a,13
       ld a,1
       ld (ix+11),a
j00332 ld a,37
       cp (ix+6)
       jp nz,j00357
       ld a,11
       ld a,2
       ld (ix+11),a
j00357 ld a,4
       cp (ix+5)
       jp nz,j00416
       ld a,3
       ld a,26
       cp (ix+6)
       jp nz,j00395
       xor a
       ld (ix+11),a
j00395 ld a,32
       cp (ix+6)
       jp nz,j00416
       ld a,1
       ld (ix+11),a
j00416 ld a,6
       cp (ix+5)
       jp nz,j00529
       ld a,2
       cp (ix+6)
       jp nz,j00453
       xor a
       ld (ix+11),a
       ld a,4
j00453 ld a,34
       cp (ix+6)
       jp nz,j00479
       ld a,1
       ld (ix+11),a
       ld a,11
j00479 ld a,36
       cp (ix+6)
       jp nz,j00504
       ld a,1
       ld (ix+11),a
       ld a,13
j00504 ld a,38
       cp (ix+6)
       jp nz,j00529
       ld a,1
       ld (ix+11),a
       ld a,11
j00529 ld a,8
       cp (ix+5)
       jp nz,j00558
       xor a
       ld (vare),a
       xor a
       ld (ix+11),a
j00558 ret
evnt10 equ $
       xor a
       ld hl,vara
       cp (hl)
       jp nc,k00292
       xor a
       ld hl,clock
       cp (hl)
       jp nc,k00287
       ld a,(varr)
       inc a
       ld (varr),a
       ld a,59
       ld hl,varr
       cp (hl)
       jp nc,k00159
       ld a,(vart)
       dec a
       ld (vart),a
       xor a
       ld (varr),a
       xor a
       ld hl,vart
       cp (hl)
       jp nz,k00159
       ld a,(varm)
       dec a
       ld (varm),a
       ld a,59
       ld (vart),a
       ld a,255
       ld hl,varm
       cp (hl)
       jp nz,k00159
       ld hl,deadf
       ld (hl),h
       ld a,2
       ld (varm),a
k00159 ld hl,6914
       ld (charx),hl
       ld a,(varm)
       ld bc,displ0
       call num2dd
       call displ1
       ld hl,7426
       ld (charx),hl
       ld a,4
       call dmsg
       ld hl,7682
       ld (charx),hl
       ld a,(vart)
       ld bc,displ0
       call num2dd
       call displ1
       xor a
       ld hl,varm
       cp (hl)
       jp nz,k00287
       ld a,10
       ld hl,vart
       cp (hl)
       jp c,k00287
       jp z,k00287
       ld a,6
       ld c,a
       ld b,a
       call 48184
k00287 jp k00292
k00292 ld a,200
       ld hl,vara
       cp (hl)
       jp nc,k00317
       xor a
       ld (vara),a
k00317 xor a
       ld hl,varj
       cp (hl)
       jp nz,k00430
       xor a
       ld hl,vara
       cp (hl)
       jp c,k00426
       ld hl,2313
       ld (charx),hl
       xor a
       call dmsg
       ld a,(varc)
       inc a
       ld (varc),a
       ld a,100
       ld hl,varc
       cp (hl)
       jp nz,k00426
       xor a
       ld (varc),a
       call redraw
       call cls
       call scrr
       xor a
       ld (vars),a
k00426 jp k00591
k00430 ld a,200
       ld hl,vark
       cp (hl)
       jp nc,k00455
       xor a
       ld (vark),a
k00455 xor a
       ld hl,vark
       cp (hl)
       jp c,k00591
       ld a,29
       ld hl,scno
       cp (hl)
       jp nz,k00502
       ld hl,gamwon
       ld (hl),h
       jp k00587
k00502 ld hl,2313
       ld (charx),hl
       ld a,1
       call dmsg
       ld a,(varc)
       inc a
       ld (varc),a
       ld a,100
       ld hl,varc
       cp (hl)
       jp nz,k00587
       call redraw
       call cls
       call scrr
       xor a
       ld (varc),a
       xor a
       ld (varj),a
       xor a
       ld (vars),a
k00587 jp k00591
k00591 xor a
       ld hl,vars
       cp (hl)
       jp nz,k02176
       xor a
       ld hl,scno
       cp (hl)
       jp nz,k00649
       ld a,8
       ld (vara),a
       ld a,8
       ld (varh),a
       ld a,28
       ld (vari),a
k00649 ld a,1
       ld hl,scno
       cp (hl)
       jp nz,k00691
       ld a,8
       ld (vara),a
       ld a,18
       ld (varh),a
       ld a,26
       ld (vari),a
k00691 ld a,2
       ld hl,scno
       cp (hl)
       jp nz,k00733
       ld a,8
       ld (vara),a
       ld a,18
       ld (varh),a
       ld a,25
       ld (vari),a
k00733 ld a,3
       ld hl,scno
       cp (hl)
       jp nz,k00775
       ld a,8
       ld (vara),a
       ld a,4
       ld (varh),a
       ld a,26
       ld (vari),a
k00775 ld a,4
       ld hl,scno
       cp (hl)
       jp nz,k00817
       ld a,9
       ld (vara),a
       ld a,18
       ld (varh),a
       ld a,26
       ld (vari),a
k00817 ld a,5
       ld hl,scno
       cp (hl)
       jp nz,k00858
       ld a,9
       ld (vara),a
       ld a,3
       ld (varh),a
       ld a,3
       ld (vari),a
k00858 ld a,6
       ld hl,scno
       cp (hl)
       jp nz,k00900
       ld a,9
       ld (vara),a
       ld a,14
       ld (varh),a
       ld a,3
       ld (vari),a
k00900 ld a,7
       ld hl,scno
       cp (hl)
       jp nz,k00942
       ld a,7
       ld (vara),a
       ld a,14
       ld (varh),a
       ld a,23
       ld (vari),a
k00942 ld a,8
       ld hl,scno
       cp (hl)
       jp nz,k00984
       ld a,7
       ld (vara),a
       ld a,11
       ld (varh),a
       ld a,29
       ld (vari),a
k00984 ld a,9
       ld hl,scno
       cp (hl)
       jp nz,k01242
       ld a,1
       ld (vara),a
       ld a,50
       ld (vark),a
       ld a,1
       ld (varj),a
       ld hl,515
       ld (charx),hl
       ld a,5
       call dmsg
       ld hl,1795
       ld (charx),hl
       ld a,(vark)
       ld bc,displ0
       call num2dd
       call displ1
       ld hl,2563
       ld (charx),hl
       ld a,6
       call dmsg
       ld a,80
       ld (varn),a
       ld a,(vark)
       ld (varw),a
       ld a,(varn)
       ld c,a
       ld a,(varw)
       add a,c
       ld (varw),a
k01129 ld a,(varw)
       ld hl,varn
       cp (hl)
       jp c,k01242
       jp z,k01242
       ld a,24
       ld (varv),a
k01160 ld a,32
       ld hl,varv
       cp (hl)
       jp c,k01225
       jp z,k01225
       ld a,(varn)
       ld h,a
       ld a,(varv)
       ld l,a
       ld (dispx),hl
       call plot0
       ld a,(varv)
       inc a
       ld (varv),a
       jp k01160
k01225 ld a,(varn)
       inc a
       ld (varn),a
       jp k01129
k01242 ld a,10
       ld hl,scno
       cp (hl)
       jp nz,k01284
       ld a,7
       ld (vara),a
       ld a,14
       ld (varh),a
       ld a,27
       ld (vari),a
k01284 ld a,11
       ld hl,scno
       cp (hl)
       jp nz,k01326
       ld a,8
       ld (vara),a
       ld a,9
       ld (varh),a
       ld a,2
       ld (vari),a
k01326 ld a,12
       ld hl,scno
       cp (hl)
       jp nz,k01367
       ld a,8
       ld (vara),a
       ld a,8
       ld (varh),a
       ld a,9
       ld (vari),a
k01367 ld a,13
       ld hl,scno
       cp (hl)
       jp nz,k01409
       ld a,8
       ld (vara),a
       ld a,7
       ld (varh),a
       ld a,28
       ld (vari),a
k01409 ld a,14
       ld hl,scno
       cp (hl)
       jp nz,k01451
       ld a,8
       ld (vara),a
       ld a,7
       ld (varh),a
       ld a,3
       ld (vari),a
k01451 ld a,15
       ld hl,scno
       cp (hl)
       jp nz,k01493
       ld a,8
       ld (vara),a
       ld a,16
       ld (varh),a
       ld a,20
       ld (vari),a
k01493 ld a,16
       ld hl,scno
       cp (hl)
       jp nz,k01536
       ld a,9
       ld (vara),a
       ld a,18
       ld (varh),a
       ld a,28
       ld (vari),a
k01536 ld a,17
       ld hl,scno
       cp (hl)
       jp nz,k01578
       ld a,6
       ld (vara),a
       ld a,18
       ld (varh),a
       ld a,28
       ld (vari),a
k01578 ld a,18
       ld hl,scno
       cp (hl)
       jp nz,k01620
       ld a,8
       ld (vara),a
       ld a,7
       ld (varh),a
       ld a,3
       ld (vari),a
k01620 ld a,19
       ld hl,scno
       cp (hl)
       jp nz,k01662
       ld a,1
       ld (vara),a
       ld a,50
       ld (vark),a
       ld a,1
       ld (varj),a
k01662 ld a,20
       ld hl,scno
       cp (hl)
       jp nz,k01703
       ld a,8
       ld (vara),a
       ld a,5
       ld (varh),a
       ld a,3
       ld (vari),a
k01703 ld a,21
       ld hl,scno
       cp (hl)
       jp nz,k01746
       ld a,6
       ld (vara),a
       ld a,12
       ld (varh),a
       ld a,10
       ld (vari),a
k01746 ld a,22
       ld hl,scno
       cp (hl)
       jp nz,k01788
       ld a,7
       ld (vara),a
       ld a,3
       ld (varh),a
       ld a,18
       ld (vari),a
k01788 ld a,23
       ld hl,scno
       cp (hl)
       jp nz,k01830
       ld a,7
       ld (vara),a
       ld a,3
       ld (varh),a
       ld a,24
       ld (vari),a
k01830 ld a,24
       ld hl,scno
       cp (hl)
       jp nz,k01872
       ld a,7
       ld (vara),a
       ld a,3
       ld (varh),a
       ld a,12
       ld (vari),a
k01872 ld a,25
       ld hl,scno
       cp (hl)
       jp nz,k01914
       ld a,7
       ld (vara),a
       ld a,3
       ld (varh),a
       ld a,28
       ld (vari),a
k01914 ld a,26
       ld hl,scno
       cp (hl)
       jp nz,k01956
       ld a,7
       ld (vara),a
       ld a,10
       ld (varh),a
       ld a,8
       ld (vari),a
k01956 ld a,27
       ld hl,scno
       cp (hl)
       jp nz,k01998
       ld a,7
       ld (vara),a
       ld a,3
       ld (varh),a
       ld a,25
       ld (vari),a
k01998 ld a,28
       ld hl,scno
       cp (hl)
       jp nz,k02040
       ld a,8
       ld (vara),a
       ld a,3
       ld (varh),a
       ld a,20
       ld (vari),a
k02040 ld a,29
       ld hl,scno
       cp (hl)
       jp nz,k02082
       ld a,1
       ld (vara),a
       ld a,30
       ld (vark),a
       ld a,1
       ld (varj),a
k02082 ld hl,4864
       ld (charx),hl
       ld a,7
       call dmsg
       ld hl,4608
       ld (charx),hl
       ld a,(vara)
       call disply
       ld hl,5888
       ld (charx),hl
       ld a,(varb)
       ld bc,displ0
       call num2td
       call displ1
       ld hl,7680
       ld (charx),hl
       ld a,(scno)
       ld bc,displ0
       call num2dd
       call displ1
k02176 ret
evnt11 equ $
       ret
evnt12 equ $
       call cls
       xor a
       ld c,a
       ld b,a
       call 48184
       ld a,99
       ld (contrl),a
m00031 ld a,99
       ld hl,contrl
       cp (hl)
       jr z,$+5
       jp nc,m00487
       ld hl,1794
       ld (charx),hl
       ld a,8
       call dmsg
       ld hl,1796
       ld (charx),hl
       ld a,9
       call dmsg
       ld hl,2568
       ld (charx),hl
       ld a,10
       call dmsg
       ld hl,2570
       ld (charx),hl
       ld a,11
       call dmsg
       ld hl,2572
       ld (charx),hl
       ld a,12
       call dmsg
       ld a,99
       ld (contrl),a
m00149 ld a,99
       ld hl,contrl
       cp (hl)
       jp nz,m00483
       ld a,(keys+7)
       call ktest
       jp c,m00190
       xor a
       ld (contrl),a
m00190 ld a,(keys+8)
       call ktest
       jp c,m00213
       ld a,1
       ld (contrl),a
m00213 ld a,(keys+9)
       call ktest
       jp c,m00479
       call cls
       ld hl,2307
       ld (charx),hl
       ld a,13
       call dmsg
       ld hl,3078
       ld (charx),hl
       ld hl,3078
       ld (charx),hl
       ld a,14
       call dmsg
       call redky
       ld hl,keys+0
       ld (hl),b
       ld hl,2570
       ld (charx),hl
       ld hl,3079
       ld (charx),hl
       ld a,15
       call dmsg
       call redky
       ld hl,keys+1
       ld (hl),b
       ld hl,2570
       ld (charx),hl
       ld hl,3080
       ld (charx),hl
       ld a,16
       call dmsg
       call redky
       ld hl,keys+2
       ld (hl),b
       ld hl,2570
       ld (charx),hl
       ld hl,3081
       ld (charx),hl
       ld a,17
       call dmsg
       call redky
       ld hl,keys+3
       ld (hl),b
       ld hl,2570
       ld (charx),hl
       ld hl,3082
       ld (charx),hl
       ld a,18
       call dmsg
       call redky
       ld hl,keys+4
       ld (hl),b
       ld hl,2570
       ld (charx),hl
       ld a,100
       ld (contrl),a
       call cls
m00479 jp m00149
m00483 jp m00031
m00487 call cls
       ret
evnt13 equ $
       ld a,1
       ld (scno),a
       call nwscr
       ld a,10
       ld (numlif),a
       ld a,1
       ld (vara),a
       ld a,100
       ld (varb),a
       xor a
       ld (varc),a
       ld a,1
       ld (vard),a
       xor a
       ld (varf),a
       xor a
       ld (varg),a
       xor a
       ld (varh),a
       xor a
       ld (vari),a
       xor a
       ld (varj),a
       xor a
       ld (vark),a
       ld a,2
       ld (varm),a
       xor a
       ld (varn),a
       ld a,1
       ld (varo),a
       ld a,1
       ld (varp),a
       xor a
       ld (varq),a
       xor a
       ld (varr),a
       xor a
       ld (vars),a
       ld a,60
       ld (vart),a
       xor a
       ld (varu),a
       xor a
       ld (varv),a
       xor a
       ld (varw),a
       xor a
       ld (ix+9),a
       xor a
       ld (ix+8),a
       xor a
       ld (varz),a
       ret
evnt14 equ $
       ld a,2
       ld (varm),a
       ld a,60
       ld (vart),a
       xor a
       ld c,a
       ld b,a
       call 48184
       ld hl,0
       ld (charx),hl
       ld a,19
       call dmsg
       ld hl,512
       ld (charx),hl
       ld hl,score
       ld b,6
       call dscor
       ld a,3
       call conink
       ld (inkmsk),a
       ld hl,2560
       ld (charx),hl
       ld a,20
       call dmsg
       ld a,6
       call conink
       ld (inkmsk),a
       ld hl,3072
       ld (charx),hl
       ld a,(numlif)
       ld bc,displ0
       call num2dd
       call displ1
       ld a,4
       call conink
       ld (inkmsk),a
       ld hl,4096
       ld (charx),hl
       ld a,21
       call dmsg
       ld a,6
       call conink
       ld (inkmsk),a
       ld a,5
       call conink
       ld (inkmsk),a
       ld hl,5376
       ld (charx),hl
       ld a,22
       call dmsg
       ld a,6
       call conink
       ld (inkmsk),a
       ld a,1
       call conink
       ld (inkmsk),a
       ld hl,7168
       ld (charx),hl
       ld a,23
       call dmsg
       ld a,6
       call conink
       ld (inkmsk),a
       ret
evnt15 equ $
       ret
evnt16 equ $
       ld hl,fx1
       ld de,205
       add hl,de
       call isnd
       ld a,1
       ld (vard),a
       ld a,(numlif)
       dec a
       ld (numlif),a
       ld hl,3072
       ld (charx),hl
       ld a,(numlif)
       ld bc,displ0
       call num2dd
       call displ1
       push ix
       ld b,30
       call delay
       pop ix
       ret
evnt17 equ $
       ret
evnt18 equ $
       call cls
       ld hl,1282
       ld (charx),hl
       ld a,24
       call dmsg
       ld hl,5
       ld (charx),hl
       ld a,25
       call dmsg
       ld hl,6
       ld (charx),hl
       ld a,26
       call dmsg
       ld hl,9
       ld (charx),hl
       ld a,27
       call dmsg
       ld hl,11
       ld (charx),hl
       ld a,28
       call dmsg
       ld hl,13
       ld (charx),hl
       ld a,29
       call dmsg
       ld hl,15
       ld (charx),hl
       ld a,30
       call dmsg
       ld hl,19
       ld (charx),hl
       ld a,31
       call dmsg
       call prskey
       ret
evnt19 equ $
       ret
evnt20 equ $
       ld hl,10
       call addsc
       ld a,(vara)
       dec a
       ld (vara),a
       ld hl,fx1
       ld de,123
       add hl,de
       call isnd
       ld hl,512
       ld (charx),hl
       ld hl,score
       ld b,6
       call dscor
       ld hl,4608
       ld (charx),hl
       ld a,32
       call dmsg
       ld hl,4608
       ld (charx),hl
       ld a,(vara)
       call disply
       ret
ptcusr ret
WINH   equ WINDOWHGT * 8
WINW   equ WINDOWWID * 5
msgdat equ $
       defb 'CLEAR SCREEN!',161
       defb 'LEVEL CLEAR!!!!',161
       defb 'Boss',186
       defb '        ',160
       defb 186
       defb 'Boss',186
       defb '        ',160
       defb 160
       defb 'MSX Murcia presents',186
       defb 'El cazador/The hunte',242
       defb '1. KEYBOAR',196
       defb '2. KEMPSTO',206
       defb '3. REDEFIN',197
       defb 'PRESS KEY FOR',186
       defb 'u',240
       defb 'DOW',206
       defb 'LEF',212
       defb 'RIGH',212
       defb 'FIRE',177
       defb 164
       defb 165
       defb 166
       defb 167
       defb 168
       defb 'Has ganado!!',161
       defb 'Credito',243
       defb '-------',173
       defb 'Logica: Kike Madriga',236
       defb 'Programacion: Kike Madriga',236
       defb 'Graficos: Kike Madriga',236
       defb 'Musica y efectos: Kike Madriga',236
       defb 'Presione una tecla para continua',242
       defb 160
nummsg defb 33
chgfx  equ $
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 240,240,160,240,240,160,240,240,160,240,240,160,240,240,160,240,240,160,240,240,160,240,240,160
       defb 252,252,168,252,252,168,252,252,168,252,252,168,252,252,168,252,252,168,252,252,168,252,252,168
       defb 9,9,8,9,9,8,9,9,8,9,9,8,25,9,34,51,25,34,51,51,34,51,51,34
       defb 8,8,8,8,8,8,8,40,8,40,60,40,60,60,40,60,60,40,60,60,40,60,60,40
       defb 12,0,8,12,0,8,12,4,8,12,12,8,12,12,8,12,12,8,60,60,40,60,60,40
       defb 51,51,34,51,51,34,51,51,0,51,34,0,51,34,0,51,0,0,51,0,0,34,0,0
       defb 56,56,40,52,52,32,56,56,40,52,52,32,56,56,40,52,52,32,56,56,40,52,52,32
       defb 15,15,10,135,75,10,75,15,130,135,195,10,195,75,10,135,195,130,195,135,130,195,195,130
       defb 212,212,128,192,192,128,192,192,128,192,232,168,192,192,128,192,192,128,232,232,128,192,192,128
       defb 49,50,34,49,50,34,50,48,32,50,51,32,48,51,32,48,48,32,51,48,32,51,48,32
       defb 195,225,160,195,225,160,195,225,160,195,225,160,210,225,160,210,225,160,240,240,160,240,240,160
       defb 240,240,160,240,240,160,240,240,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160
       defb 243,243,162,243,81,162,162,0,162,162,0,162,243,81,162,243,81,162,0,0,0,0,0,0
       defb 240,195,160,240,195,130,240,195,160,240,195,160,195,195,160,240,195,160,240,195,160,240,195,160
       defb 195,195,130,195,195,130,195,195,130,195,195,130,195,195,130,195,195,130,195,195,130,195,195,130
       defb 240,120,160,240,120,160,240,120,160,180,60,160,180,60,160,180,60,160,60,60,40,60,60,40
       defb 216,204,160,216,204,160,216,204,160,216,204,160,216,204,160,216,204,160,216,204,160,216,204,160
       defb 228,216,136,228,216,136,228,216,136,228,216,136,228,216,136,228,216,136,228,216,136,228,216,136
       defb 0,32,0,0,32,0,0,32,0,3,33,2,48,48,32,48,48,32,48,48,32,48,48,32
       defb 195,195,130,130,0,130,130,130,130,130,130,130,130,130,130,130,130,130,130,0,130,195,195,130
       defb 3,18,32,3,18,32,3,18,32,3,18,32,3,18,32,3,18,32,3,16,32,3,18,32
       defb 48,0,0,48,0,0,48,0,0,48,0,0,48,0,0,48,0,0,48,0,0,48,0,0
       defb 195,233,130,195,233,130,195,233,130,195,233,130,195,233,130,195,233,130,195,233,130,195,233,130
       defb 240,240,160,240,240,160,240,240,160,242,241,160,242,243,162,241,242,162,243,243,162,243,243,162
       defb 243,243,162,243,243,162,243,243,162,243,243,162,243,243,162,243,243,162,243,243,162,243,243,162
       defb 0,8,0,0,8,0,0,8,0,0,8,0,12,12,8,12,12,8,12,12,8,12,12,8
       defb 195,65,130,195,65,130,195,65,130,130,65,130,130,0,130,0,0,0,0,0,0,0,0,0
       defb 130,130,130,65,65,0,65,65,0,130,130,130,130,0,0,0,0,0,0,0,0,0,0,0
       defb 130,130,130,130,130,130,130,0,130,130,0,130,0,0,130,0,0,0,0,0,0,0,0,0
       defb 255,255,170,255,255,170,85,255,0,85,255,0,85,255,0,0,170,0,0,170,0,0,170,0
bprop  equ $
       defb 0
       defb 2
       defb 0
       defb 5
       defb 5
       defb 5
       defb 0
       defb 2
       defb 2
       defb 2
       defb 2
       defb 0
       defb 0
       defb 0
       defb 0
       defb 2
       defb 0
       defb 0
       defb 0
       defb 8
       defb 0
       defb 3
       defb 3
       defb 0
       defb 2
       defb 2
       defb 8
       defb 0
       defb 0
       defb 0
       defb 5
sprgfx equ $
       defb 0,68,136,0,0
       defb 0,204,204,0,0
       defb 0,85,170,0,0
       defb 0,85,170,0,0
       defb 0,85,0,0,0
       defb 0,231,161,0,0
       defb 81,231,161,12,8
       defb 81,231,161,8,0
       defb 243,231,240,40,0
       defb 243,231,139,2,0
       defb 243,231,139,0,0
       defb 81,69,139,8,0
       defb 0,5,15,8,0
       defb 0,5,0,0,0
       defb 0,5,0,0,0
       defb 0,4,8,0,0
       defb 0,0,204,0,0
       defb 0,68,204,136,0
       defb 0,0,255,0,0
       defb 0,0,255,0,0
       defb 0,0,170,0,0
       defb 0,81,218,2,0
       defb 0,243,218,6,12
       defb 0,243,218,6,0
       defb 81,243,218,180,0
       defb 81,243,207,3,0
       defb 81,243,207,2,0
       defb 0,162,207,6,0
       defb 0,0,15,14,0
       defb 0,0,10,0,0
       defb 0,0,10,0,0
       defb 0,0,12,0,0
       defb 0,68,136,0,0
       defb 0,204,204,0,0
       defb 0,17,34,0,0
       defb 0,17,34,0,0
       defb 0,17,0,0,0
       defb 0,231,160,0,0
       defb 81,231,160,12,8
       defb 81,231,160,8,0
       defb 243,231,240,40,0
       defb 243,231,138,0,0
       defb 243,231,138,0,0
       defb 81,69,138,2,0
       defb 0,5,11,2,0
       defb 0,15,10,0,0
       defb 0,11,10,0,0
       defb 0,11,15,0,0
       defb 0,0,204,0,0
       defb 0,68,204,136,0
       defb 0,0,51,0,0
       defb 0,0,51,0,0
       defb 0,0,34,0,0
       defb 0,81,218,0,0
       defb 0,243,218,4,12
       defb 0,243,218,4,0
       defb 81,243,218,180,0
       defb 81,243,207,0,0
       defb 81,243,207,0,0
       defb 0,162,207,1,0
       defb 0,0,15,3,0
       defb 0,5,15,0,0
       defb 0,5,7,0,0
       defb 0,5,7,10,0
       defb 0,0,204,0,0
       defb 0,68,204,136,0
       defb 0,0,51,0,0
       defb 0,0,51,0,0
       defb 0,0,17,0,0
       defb 0,0,229,162,0
       defb 12,8,229,243,0
       defb 0,8,229,243,0
       defb 0,120,229,243,162
       defb 0,0,207,243,162
       defb 0,0,207,243,162
       defb 0,8,207,81,0
       defb 0,13,15,0,0
       defb 0,0,5,0,0
       defb 0,0,5,0,0
       defb 0,0,12,0,0
       defb 0,0,68,136,0
       defb 0,0,204,204,0
       defb 0,0,17,34,0
       defb 0,0,17,34,0
       defb 0,0,0,34,0
       defb 0,0,80,219,0
       defb 4,12,80,219,162
       defb 0,4,80,219,162
       defb 0,20,240,219,243
       defb 0,0,69,219,243
       defb 0,0,69,219,243
       defb 0,4,69,138,162
       defb 0,4,15,10,0
       defb 0,0,0,10,0
       defb 0,0,0,10,0
       defb 0,0,4,8,0
       defb 0,0,204,0,0
       defb 0,68,204,136,0
       defb 0,0,51,0,0
       defb 0,0,51,0,0
       defb 0,0,17,0,0
       defb 0,0,229,162,0
       defb 12,8,229,243,0
       defb 0,8,229,243,0
       defb 0,120,229,243,162
       defb 0,0,207,243,162
       defb 0,0,207,243,162
       defb 0,2,207,81,0
       defb 0,3,15,0,0
       defb 0,0,15,10,0
       defb 0,0,11,10,0
       defb 0,5,11,10,0
       defb 0,0,68,136,0
       defb 0,0,204,204,0
       defb 0,0,17,34,0
       defb 0,0,17,34,0
       defb 0,0,0,34,0
       defb 0,0,80,219,0
       defb 4,12,80,219,162
       defb 0,4,80,219,162
       defb 0,20,240,219,243
       defb 0,0,69,219,243
       defb 0,0,69,219,243
       defb 0,1,69,138,162
       defb 0,1,7,10,0
       defb 0,0,5,15,0
       defb 0,0,5,7,0
       defb 0,0,15,7,0
       defb 1,3,2,0,0
       defb 1,3,3,0,0
       defb 0,17,51,0,0
       defb 0,1,35,0,0
       defb 0,5,15,0,0
       defb 0,5,15,0,0
       defb 0,5,5,3,2
       defb 0,5,5,3,0
       defb 0,5,5,2,0
       defb 0,5,5,2,0
       defb 0,5,15,2,0
       defb 0,5,15,2,0
       defb 0,1,3,2,0
       defb 0,1,0,0,0
       defb 0,1,0,0,0
       defb 0,1,2,0,0
       defb 0,3,3,0,0
       defb 0,3,3,2,0
       defb 0,0,51,34,0
       defb 0,0,19,2,0
       defb 0,0,15,10,0
       defb 0,0,15,10,0
       defb 0,0,10,11,3
       defb 0,0,10,11,2
       defb 0,0,10,11,0
       defb 0,0,10,11,0
       defb 0,0,15,11,0
       defb 0,0,15,11,0
       defb 0,0,3,3,0
       defb 0,0,2,0,0
       defb 0,0,2,0,0
       defb 0,0,3,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,84,252,0,0
       defb 0,0,84,0,0
       defb 0,0,84,0,0
       defb 0,0,252,0,0
       defb 0,0,84,0,0
       defb 0,0,84,0,0
       defb 0,84,252,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,252,168,0
       defb 0,0,0,168,0
       defb 0,0,0,168,0
       defb 0,0,84,168,0
       defb 0,0,0,168,0
       defb 0,0,0,168,0
       defb 0,0,252,168,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 1,3,136,0,0
       defb 1,70,204,0,0
       defb 0,17,51,0,0
       defb 0,1,35,0,0
       defb 0,5,15,5,0
       defb 0,15,15,10,0
       defb 5,69,207,3,2
       defb 0,69,207,3,0
       defb 0,69,207,2,0
       defb 0,69,207,2,0
       defb 0,5,143,2,0
       defb 0,5,15,2,0
       defb 0,5,7,2,0
       defb 0,5,1,10,0
       defb 0,5,0,0,0
       defb 0,5,2,0,0
       defb 0,3,70,0,0
       defb 0,3,204,136,0
       defb 0,0,51,34,0
       defb 0,0,19,2,0
       defb 0,0,15,10,10
       defb 0,5,15,15,0
       defb 0,10,207,139,3
       defb 0,0,207,139,2
       defb 0,0,207,139,0
       defb 0,0,207,139,0
       defb 0,0,79,11,0
       defb 0,0,15,11,0
       defb 0,0,11,11,0
       defb 0,0,10,7,0
       defb 0,0,10,0,0
       defb 0,0,11,0,0
       defb 1,3,136,0,0
       defb 1,70,204,0,0
       defb 0,17,51,0,0
       defb 0,1,35,0,0
       defb 5,5,15,1,0
       defb 0,15,15,10,0
       defb 1,69,207,7,2
       defb 0,69,207,3,0
       defb 0,69,207,2,0
       defb 0,69,207,2,0
       defb 0,5,143,2,0
       defb 0,5,15,2,0
       defb 0,5,7,2,0
       defb 0,11,5,2,0
       defb 5,1,5,0,0
       defb 0,1,7,0,0
       defb 0,3,70,0,0
       defb 0,3,204,136,0
       defb 0,0,51,34,0
       defb 0,0,19,2,0
       defb 0,10,15,10,2
       defb 0,5,15,15,0
       defb 0,2,207,139,11
       defb 0,0,207,139,2
       defb 0,0,207,139,0
       defb 0,0,207,139,0
       defb 0,0,79,11,0
       defb 0,0,15,11,0
       defb 0,0,11,11,0
       defb 0,5,2,11,0
       defb 0,10,2,10,0
       defb 0,0,3,10,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,84,252,168,0
       defb 0,84,0,0,0
       defb 0,84,0,0,0
       defb 0,84,252,168,0
       defb 0,0,0,168,0
       defb 0,0,1,168,0
       defb 0,84,252,168,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,252,252,0
       defb 0,0,168,0,0
       defb 0,0,168,0,0
       defb 0,0,252,252,0
       defb 0,0,0,84,0
       defb 0,0,0,86,0
       defb 0,0,252,252,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,84,252,2,0
       defb 0,84,3,2,0
       defb 0,84,0,0,0
       defb 0,84,0,0,0
       defb 0,84,252,0,0
       defb 0,84,84,0,0
       defb 0,84,84,0,0
       defb 0,84,252,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,252,169,0
       defb 0,0,169,3,0
       defb 0,0,168,0,0
       defb 0,0,168,0,0
       defb 0,0,252,168,0
       defb 0,0,168,168,0
       defb 0,0,168,168,0
       defb 0,0,252,168,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,84,252,0,0
       defb 0,0,84,0,0
       defb 0,0,84,0,0
       defb 0,0,252,0,0
       defb 0,0,168,0,0
       defb 0,1,168,0,0
       defb 0,1,168,0,0
       defb 0,1,168,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,252,168,0
       defb 0,0,0,168,0
       defb 0,0,0,168,0
       defb 0,0,84,168,0
       defb 0,0,84,0,0
       defb 0,0,86,0,0
       defb 0,0,86,0,0
       defb 0,0,86,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,252,252,0,0
       defb 0,168,84,0,0
       defb 0,168,84,0,0
       defb 0,168,84,0,0
       defb 0,252,252,0,0
       defb 0,168,84,0,0
       defb 0,168,84,2,0
       defb 0,168,84,2,0
       defb 0,252,252,2,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,84,252,168,0
       defb 0,84,0,168,0
       defb 0,84,0,168,0
       defb 0,84,0,168,0
       defb 0,84,252,168,0
       defb 0,84,0,168,0
       defb 0,84,0,169,0
       defb 0,84,0,169,0
       defb 0,84,252,169,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 3,0,0,0,0
       defb 51,51,51,51,34
       defb 51,51,51,51,34
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 1,2,0,0,0
       defb 17,51,51,51,51
       defb 17,51,51,51,51
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,5,0,0
       defb 12,0,5,10,0
       defb 12,0,5,15,0
       defb 12,2,0,11,0
       defb 4,2,0,11,0
       defb 4,6,9,11,0
       defb 0,12,12,8,0
       defb 0,12,12,8,0
       defb 0,12,0,9,0
       defb 0,8,130,73,0
       defb 0,8,0,8,0
       defb 0,195,0,195,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,10,0
       defb 4,8,0,15,0
       defb 4,8,0,15,10
       defb 4,9,0,5,2
       defb 0,9,0,5,2
       defb 0,9,12,7,2
       defb 0,4,12,12,0
       defb 0,4,12,12,0
       defb 0,4,8,4,2
       defb 0,4,65,4,130
       defb 0,4,0,4,0
       defb 0,65,130,65,130
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,5,0,0
       defb 3,0,5,10,0
       defb 3,0,5,15,0
       defb 12,2,0,11,0
       defb 12,2,0,11,0
       defb 4,6,9,11,0
       defb 0,12,12,8,0
       defb 0,12,12,8,0
       defb 0,12,0,9,0
       defb 0,12,67,9,0
       defb 65,6,130,131,0
       defb 0,67,0,3,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,10,0
       defb 1,2,0,15,0
       defb 1,2,0,15,10
       defb 4,9,0,5,2
       defb 4,9,0,5,2
       defb 0,9,12,7,2
       defb 0,4,12,12,0
       defb 0,4,12,12,0
       defb 0,4,8,4,2
       defb 0,4,9,134,2
       defb 0,131,73,65,2
       defb 0,1,130,1,2
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,5,0,0,0
       defb 0,15,0,4,8
       defb 5,15,0,4,8
       defb 1,10,0,6,8
       defb 1,10,0,6,0
       defb 1,11,12,6,0
       defb 0,12,12,8,0
       defb 0,12,12,8,0
       defb 1,8,4,8,0
       defb 65,8,130,8,0
       defb 0,8,0,8,0
       defb 65,130,65,130,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,10,0,0
       defb 0,5,10,0,12
       defb 0,15,10,0,12
       defb 0,7,0,1,12
       defb 0,7,0,1,8
       defb 0,7,6,9,8
       defb 0,4,12,12,0
       defb 0,4,12,12,0
       defb 0,6,0,12,0
       defb 0,134,65,4,0
       defb 0,4,0,4,0
       defb 0,195,0,195,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,5,0,0,0
       defb 0,15,0,1,2
       defb 5,15,0,1,2
       defb 1,10,0,6,8
       defb 1,10,0,6,8
       defb 1,11,12,6,0
       defb 0,12,12,8,0
       defb 0,12,12,8,0
       defb 1,8,4,8,0
       defb 1,73,6,8,0
       defb 1,130,134,67,0
       defb 1,2,65,2,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,10,0,0
       defb 0,5,10,0,3
       defb 0,15,10,0,3
       defb 0,7,0,1,12
       defb 0,7,0,1,12
       defb 0,7,6,9,8
       defb 0,4,12,12,0
       defb 0,4,12,12,0
       defb 0,6,0,12,0
       defb 0,6,131,12,0
       defb 0,67,65,9,130
       defb 0,3,0,131,0
       defb 3,7,7,0,0
       defb 3,7,7,2,0
       defb 0,5,175,2,0
       defb 0,85,255,170,0
       defb 0,85,255,191,2
       defb 0,0,63,43,42
       defb 0,0,21,63,42
       defb 1,3,1,63,0
       defb 1,15,3,63,2
       defb 7,11,15,15,2
       defb 7,15,7,15,2
       defb 15,15,7,15,2
       defb 7,15,7,10,2
       defb 7,15,7,0,2
       defb 3,42,1,42,0
       defb 1,127,1,127,0
       defb 1,3,11,10,0
       defb 1,3,11,11,0
       defb 0,0,95,11,0
       defb 0,0,255,255,0
       defb 0,0,255,255,43
       defb 0,0,21,63,23
       defb 0,0,0,63,63
       defb 0,3,2,23,42
       defb 0,7,11,23,43
       defb 1,15,7,15,11
       defb 1,15,11,15,11
       defb 5,15,11,15,11
       defb 1,15,11,15,1
       defb 1,15,11,10,1
       defb 1,23,0,23,0
       defb 0,23,170,23,170
       defb 0,15,15,2,0
       defb 0,85,87,170,0
       defb 0,85,255,191,2
       defb 0,0,63,43,42
       defb 0,0,21,63,42
       defb 1,3,1,63,0
       defb 1,15,3,63,2
       defb 7,11,15,15,2
       defb 7,15,7,15,2
       defb 7,15,7,15,2
       defb 15,15,7,10,2
       defb 7,15,7,0,2
       defb 3,42,1,42,0
       defb 1,43,1,43,0
       defb 87,3,87,0,0
       defb 3,3,3,2,0
       defb 0,5,15,11,0
       defb 0,0,171,255,0
       defb 0,0,255,255,43
       defb 0,0,21,63,23
       defb 0,0,0,63,63
       defb 0,3,2,23,42
       defb 0,7,11,23,43
       defb 1,15,7,15,11
       defb 1,15,11,15,11
       defb 1,15,11,15,11
       defb 5,15,11,15,1
       defb 1,15,11,10,1
       defb 1,23,0,23,0
       defb 0,23,2,23,2
       defb 1,171,3,170,0
       defb 1,3,3,3,0
       defb 0,5,7,3,2
       defb 0,7,7,3,2
       defb 0,7,175,0,0
       defb 0,255,255,0,0
       defb 23,255,255,0,0
       defb 43,63,42,0,0
       defb 63,63,0,0,0
       defb 21,43,1,3,0
       defb 23,43,7,11,0
       defb 7,15,11,15,2
       defb 7,15,7,15,2
       defb 7,15,7,15,10
       defb 2,15,7,15,2
       defb 2,5,7,15,2
       defb 0,43,0,43,2
       defb 85,43,85,43,0
       defb 0,0,11,11,3
       defb 0,1,11,11,3
       defb 0,1,95,10,0
       defb 0,85,255,170,0
       defb 1,127,255,170,0
       defb 21,23,63,0,0
       defb 21,63,42,0,0
       defb 0,63,2,3,2
       defb 1,63,3,15,2
       defb 1,15,15,7,11
       defb 1,15,11,15,11
       defb 1,15,11,15,15
       defb 1,5,11,15,11
       defb 1,0,11,15,11
       defb 0,21,2,21,3
       defb 0,191,2,191,2
       defb 0,7,15,10,0
       defb 0,255,87,0,0
       defb 23,255,255,0,0
       defb 43,63,42,0,0
       defb 63,63,0,0,0
       defb 21,43,1,3,0
       defb 23,43,7,11,0
       defb 7,15,11,15,2
       defb 7,15,7,15,2
       defb 7,15,7,15,2
       defb 2,15,7,15,10
       defb 2,5,7,15,2
       defb 0,43,0,43,2
       defb 1,43,1,43,0
       defb 0,85,3,87,2
       defb 0,3,3,3,2
       defb 0,1,15,15,0
       defb 0,85,171,170,0
       defb 1,127,255,170,0
       defb 21,23,63,0,0
       defb 21,63,42,0,0
       defb 0,63,2,3,2
       defb 1,63,3,15,2
       defb 1,15,15,7,11
       defb 1,15,11,15,11
       defb 1,15,11,15,11
       defb 1,5,11,15,15
       defb 1,0,11,15,11
       defb 0,21,2,21,3
       defb 0,23,2,23,2
       defb 0,0,171,3,171
       defb 0,1,3,3,3
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,255,0
       defb 0,64,128,171,170
       defb 0,192,192,255,170
       defb 64,192,192,85,2
       defb 192,192,192,255,0
       defb 66,192,128,171,0
       defb 170,192,85,0,0
       defb 85,0,255,0,0
       defb 4,174,4,0,0
       defb 4,4,4,0,0
       defb 9,8,8,8,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,85,170
       defb 0,0,192,85,87
       defb 0,64,192,213,255
       defb 0,192,192,128,171
       defb 64,192,192,213,170
       defb 1,192,192,85,2
       defb 85,64,128,170,0
       defb 0,170,85,170,0
       defb 0,93,8,8,0
       defb 0,8,8,8,0
       defb 4,6,4,4,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,255,0
       defb 0,64,128,171,170
       defb 0,192,192,255,170
       defb 64,192,192,85,2
       defb 192,192,192,255,0
       defb 66,192,128,171,0
       defb 170,192,85,0,0
       defb 85,0,255,0,0
       defb 4,174,4,8,0
       defb 4,4,4,6,0
       defb 3,9,8,8,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,85,170
       defb 0,0,192,85,87
       defb 0,64,192,213,255
       defb 0,192,192,128,171
       defb 64,192,192,213,170
       defb 1,192,192,85,2
       defb 85,64,128,170,0
       defb 0,170,85,170,0
       defb 0,93,8,12,0
       defb 0,8,8,9,8
       defb 1,6,6,4,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 85,170,0,0,0
       defb 171,170,192,0,0
       defb 255,234,192,128,0
       defb 87,64,192,192,0
       defb 85,234,192,192,128
       defb 1,170,192,192,2
       defb 0,85,64,128,170
       defb 0,85,170,85,0
       defb 0,4,4,174,0
       defb 0,4,4,4,0
       defb 0,8,8,9,8
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,255,0,0,0
       defb 85,87,64,128,0
       defb 85,255,192,192,0
       defb 1,170,192,192,128
       defb 0,255,192,192,192
       defb 0,87,64,192,129
       defb 0,0,170,192,85
       defb 0,0,255,0,170
       defb 0,0,8,93,8
       defb 0,0,8,8,8
       defb 0,4,4,4,6
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 85,170,0,0,0
       defb 171,170,192,0,0
       defb 255,234,192,128,0
       defb 87,64,192,192,0
       defb 85,234,192,192,128
       defb 1,170,192,192,2
       defb 0,85,64,128,170
       defb 0,85,170,85,0
       defb 0,12,4,174,0
       defb 4,6,4,4,0
       defb 0,8,9,9,2
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,255,0,0,0
       defb 85,87,64,128,0
       defb 85,255,192,192,0
       defb 1,170,192,192,128
       defb 0,255,192,192,192
       defb 0,87,64,192,129
       defb 0,0,170,192,85
       defb 0,0,255,0,170
       defb 0,4,8,93,8
       defb 0,9,8,8,8
       defb 0,4,4,6,3
       defb 84,3,0,0,0
       defb 84,3,0,0,0
       defb 84,3,8,0,8
       defb 169,2,4,4,0
       defb 169,0,84,252,2
       defb 169,0,169,169,168
       defb 169,0,84,252,2
       defb 169,0,84,86,2
       defb 15,15,0,169,2
       defb 15,15,10,9,0
       defb 15,15,15,2,0
       defb 15,10,15,0,0
       defb 6,0,14,0,0
       defb 4,0,4,0,0
       defb 168,168,168,168,0
       defb 168,168,168,168,0
       defb 0,169,2,0,0
       defb 0,169,2,0,0
       defb 0,169,6,0,4
       defb 84,3,0,8,8
       defb 84,2,0,252,169
       defb 84,2,84,86,86
       defb 84,2,0,252,169
       defb 84,2,0,169,169
       defb 5,15,10,84,3
       defb 5,15,15,4,2
       defb 5,15,15,11,0
       defb 5,15,5,10,0
       defb 1,8,5,8,0
       defb 0,8,0,8,0
       defb 84,84,84,84,0
       defb 84,84,84,84,0
       defb 1,3,0,0,0
       defb 1,3,0,0,0
       defb 1,3,6,4,2
       defb 3,2,4,4,0
       defb 3,0,84,252,2
       defb 169,0,169,169,168
       defb 169,0,252,252,168
       defb 169,0,84,86,2
       defb 15,15,0,169,2
       defb 15,15,10,9,0
       defb 15,15,15,2,0
       defb 15,10,15,0,0
       defb 6,0,14,0,0
       defb 4,168,4,168,0
       defb 86,86,86,86,0
       defb 2,168,2,168,0
       defb 0,3,2,0,0
       defb 0,3,2,0,0
       defb 0,3,3,8,9
       defb 1,3,0,8,8
       defb 1,2,0,252,169
       defb 84,2,84,86,86
       defb 84,2,84,252,252
       defb 84,2,0,169,169
       defb 5,15,10,84,3
       defb 5,15,15,4,2
       defb 5,15,15,11,0
       defb 5,15,5,10,0
       defb 1,8,5,8,0
       defb 0,92,0,92,0
       defb 1,169,169,169,168
       defb 1,84,1,84,0
       defb 0,0,1,86,0
       defb 0,0,1,86,0
       defb 8,0,9,86,0
       defb 4,4,0,3,168
       defb 86,252,0,1,168
       defb 169,169,168,1,168
       defb 86,252,0,1,168
       defb 86,86,0,1,168
       defb 3,168,5,15,10
       defb 1,8,15,15,10
       defb 0,7,15,15,10
       defb 0,5,10,15,10
       defb 0,4,10,4,2
       defb 0,4,0,4,0
       defb 0,168,168,168,168
       defb 0,168,168,168,168
       defb 0,0,0,3,168
       defb 0,0,0,3,168
       defb 4,0,4,3,168
       defb 0,8,8,1,86
       defb 1,252,168,0,86
       defb 84,86,86,0,86
       defb 1,252,168,0,86
       defb 1,169,168,0,86
       defb 1,86,0,15,15
       defb 0,6,5,15,15
       defb 0,1,15,15,15
       defb 0,0,15,5,15
       defb 0,0,13,0,9
       defb 0,0,8,0,8
       defb 0,84,84,84,84
       defb 0,84,84,84,84
       defb 0,0,1,3,0
       defb 0,0,1,3,0
       defb 6,4,3,3,0
       defb 4,4,0,3,2
       defb 86,252,0,1,2
       defb 169,169,168,1,168
       defb 252,252,168,1,168
       defb 86,86,0,1,168
       defb 3,168,5,15,10
       defb 1,8,15,15,10
       defb 0,7,15,15,10
       defb 0,5,10,15,10
       defb 0,4,10,4,2
       defb 0,172,0,172,0
       defb 84,86,86,86,2
       defb 0,168,2,168,2
       defb 0,0,0,3,2
       defb 0,0,0,3,2
       defb 1,8,9,3,2
       defb 0,8,8,1,3
       defb 1,252,168,0,3
       defb 84,86,86,0,86
       defb 84,252,252,0,86
       defb 1,169,168,0,86
       defb 1,86,0,15,15
       defb 0,6,5,15,15
       defb 0,1,15,15,15
       defb 0,0,15,5,15
       defb 0,0,13,0,9
       defb 0,84,8,84,8
       defb 0,169,169,169,169
       defb 0,84,1,84,1
       defb 0,0,21,63,2
       defb 0,0,21,183,10
       defb 0,0,21,63,0
       defb 0,0,0,43,0
       defb 0,0,0,21,2
       defb 0,21,63,42,42
       defb 2,43,19,63,42
       defb 2,63,3,252,168
       defb 86,63,3,252,168
       defb 252,252,169,252,168
       defb 0,252,252,252,0
       defb 0,84,252,84,0
       defb 0,5,0,7,0
       defb 0,11,0,11,0
       defb 0,11,0,11,2
       defb 2,15,0,15,2
       defb 0,0,0,63,43
       defb 0,0,0,123,47
       defb 0,0,0,63,42
       defb 0,0,0,21,2
       defb 0,0,0,0,43
       defb 0,0,63,63,21
       defb 1,21,3,55,63
       defb 1,21,43,86,252
       defb 1,189,43,86,252
       defb 84,252,252,86,252
       defb 0,84,252,252,168
       defb 0,0,252,168,168
       defb 0,0,10,1,10
       defb 0,5,2,5,2
       defb 0,5,2,5,3
       defb 1,5,10,5,11
       defb 0,0,87,127,0
       defb 0,0,87,247,10
       defb 0,0,23,127,0
       defb 0,0,1,42,0
       defb 0,0,0,87,0
       defb 0,63,63,1,170
       defb 21,3,23,63,2
       defb 84,3,252,252,168
       defb 252,3,252,252,168
       defb 169,86,252,252,0
       defb 84,252,252,168,0
       defb 0,252,168,168,0
       defb 0,10,1,10,0
       defb 7,2,5,2,0
       defb 11,2,11,3,0
       defb 1,2,1,3,2
       defb 0,0,1,191,170
       defb 0,0,1,251,175
       defb 0,0,1,63,170
       defb 0,0,0,23,0
       defb 0,0,0,1,170
       defb 0,21,63,42,87
       defb 0,43,3,63,43
       defb 0,169,86,252,252
       defb 84,169,86,252,252
       defb 84,3,252,252,168
       defb 0,252,252,252,0
       defb 0,84,252,84,0
       defb 0,5,0,7,0
       defb 1,11,0,11,0
       defb 5,3,5,3,2
       defb 0,3,0,3,3
       defb 23,63,0,0,0
       defb 31,183,0,0,0
       defb 21,63,0,0,0
       defb 1,42,0,0,0
       defb 23,0,0,0,0
       defb 42,63,63,0,0
       defb 63,59,3,42,2
       defb 252,169,23,42,2
       defb 252,169,23,126,2
       defb 252,169,252,252,168
       defb 84,252,252,168,0
       defb 84,84,252,0,0
       defb 5,2,5,0,0
       defb 1,10,1,10,0
       defb 3,10,1,10,0
       defb 7,10,5,10,2
       defb 1,63,42,0,0
       defb 5,123,42,0,0
       defb 0,63,42,0,0
       defb 0,23,0,0,0
       defb 1,42,0,0,0
       defb 21,21,63,42,0
       defb 21,63,35,23,1
       defb 84,252,3,63,1
       defb 84,252,3,63,169
       defb 84,252,86,252,252
       defb 0,252,252,252,0
       defb 0,168,252,168,0
       defb 0,11,0,10,0
       defb 0,7,0,7,0
       defb 1,7,0,7,0
       defb 1,15,0,15,1
       defb 85,127,2,0,0
       defb 95,247,2,0,0
       defb 85,63,2,0,0
       defb 0,43,0,0,0
       defb 85,2,0,0,0
       defb 171,21,63,42,0
       defb 23,63,3,23,0
       defb 252,252,169,86,0
       defb 252,252,169,86,168
       defb 84,252,252,3,168
       defb 0,252,252,252,0
       defb 0,168,252,168,0
       defb 0,11,0,10,0
       defb 0,7,0,7,2
       defb 1,3,10,3,10
       defb 3,3,0,3,0
       defb 0,191,171,0,0
       defb 5,251,171,0,0
       defb 0,191,43,0,0
       defb 0,21,2,0,0
       defb 0,171,0,0,0
       defb 85,2,63,63,0
       defb 1,63,43,3,42
       defb 84,252,252,3,168
       defb 84,252,252,3,252
       defb 0,252,252,169,86
       defb 0,84,252,252,168
       defb 0,84,84,252,0
       defb 0,5,2,5,0
       defb 0,1,10,1,11
       defb 0,3,7,1,7
       defb 1,3,2,1,2
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,69,207,0,0
       defb 168,207,207,138,168
       defb 237,139,139,207,168
       defb 237,139,139,207,168
       defb 237,207,207,207,168
       defb 71,139,3,207,2
       defb 71,207,207,207,2
       defb 84,71,207,86,2
       defb 84,169,139,252,2
       defb 0,252,86,169,2
       defb 0,84,252,3,2
       defb 0,7,7,3,2
       defb 0,11,11,3,2
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,207,138,0
       defb 84,69,207,207,84
       defb 84,207,71,71,222
       defb 84,207,71,71,222
       defb 84,207,207,207,222
       defb 1,207,3,71,139
       defb 1,207,207,207,139
       defb 0,169,207,139,169
       defb 0,252,71,86,169
       defb 0,84,169,252,3
       defb 0,0,252,169,3
       defb 0,1,11,11,3
       defb 0,5,7,3,3
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,69,207,0,0
       defb 0,207,207,138,0
       defb 71,139,139,207,0
       defb 71,139,139,207,2
       defb 69,207,207,207,2
       defb 237,207,3,207,168
       defb 237,207,71,207,168
       defb 84,71,207,86,2
       defb 84,169,139,252,2
       defb 0,252,86,169,2
       defb 0,84,252,3,2
       defb 0,7,7,3,2
       defb 0,7,15,11,2
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,207,138,0
       defb 0,69,207,207,0
       defb 1,207,71,71,138
       defb 1,207,71,71,139
       defb 0,207,207,207,139
       defb 84,207,139,71,222
       defb 84,207,139,207,222
       defb 0,169,207,139,169
       defb 0,252,71,86,169
       defb 0,84,169,252,3
       defb 0,0,252,169,3
       defb 0,1,11,11,3
       defb 0,1,15,15,3
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,69,207,0,0
       defb 0,207,207,138,0
       defb 69,139,139,207,2
       defb 71,139,139,207,2
       defb 71,207,207,207,0
       defb 237,139,71,207,168
       defb 237,207,71,207,168
       defb 86,71,207,86,0
       defb 86,169,139,252,0
       defb 3,252,86,168,0
       defb 3,86,252,0,0
       defb 3,7,7,2,0
       defb 3,15,15,2,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,207,138,0
       defb 0,69,207,207,0
       defb 0,207,71,71,139
       defb 1,207,71,71,139
       defb 1,207,207,207,138
       defb 84,207,3,207,222
       defb 84,207,139,207,222
       defb 1,169,207,139,168
       defb 1,252,71,86,168
       defb 1,86,169,252,0
       defb 1,3,252,168,0
       defb 1,3,11,11,0
       defb 1,7,15,11,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,69,207,0,0
       defb 168,207,207,138,168
       defb 237,139,139,207,168
       defb 237,139,139,207,168
       defb 237,207,207,207,168
       defb 71,139,3,207,2
       defb 71,207,207,207,2
       defb 86,71,207,86,0
       defb 86,169,139,252,0
       defb 3,252,86,168,0
       defb 3,86,252,0,0
       defb 3,7,7,2,0
       defb 3,3,11,10,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,207,138,0
       defb 84,69,207,207,84
       defb 84,207,71,71,222
       defb 84,207,71,71,222
       defb 84,207,207,207,222
       defb 1,207,3,71,139
       defb 1,207,207,207,139
       defb 1,169,207,139,168
       defb 1,252,71,86,168
       defb 1,86,169,252,0
       defb 1,3,252,168,0
       defb 1,3,11,11,0
       defb 1,3,7,7,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,84,252,0,0
       defb 0,84,84,0,0
       defb 0,84,84,0,0
       defb 0,84,84,0,0
       defb 0,84,252,0,0
       defb 0,84,84,2,0
       defb 0,84,84,2,0
       defb 0,84,84,2,0
       defb 0,84,252,2,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,252,168,0
       defb 0,0,168,168,0
       defb 0,0,168,168,0
       defb 0,0,168,168,0
       defb 0,0,252,168,0
       defb 0,0,168,169,0
       defb 0,0,168,169,0
       defb 0,0,168,169,0
       defb 0,0,252,169,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,3,131,0,0
       defb 0,67,195,2,0
       defb 1,195,195,131,0
       defb 67,195,75,195,2
       defb 195,135,195,195,130
       defb 195,75,195,195,130
       defb 135,195,195,195,130
       defb 135,195,195,195,130
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 67,195,195,195,2
       defb 3,195,195,131,2
       defb 1,67,195,3,0
       defb 0,3,131,2,0
       defb 0,1,67,2,0
       defb 0,1,195,131,0
       defb 0,67,195,195,2
       defb 1,195,135,195,131
       defb 65,195,75,195,195
       defb 65,135,195,195,195
       defb 65,75,195,195,195
       defb 65,75,195,195,195
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 1,195,195,195,131
       defb 1,67,195,195,3
       defb 0,3,195,131,2
       defb 0,1,67,3,0
       defb 0,3,131,0,0
       defb 0,67,195,2,0
       defb 1,195,195,131,0
       defb 67,195,195,195,2
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 135,195,195,195,130
       defb 135,195,195,195,130
       defb 195,75,195,195,130
       defb 195,135,195,195,130
       defb 67,195,75,195,2
       defb 3,195,195,131,2
       defb 1,67,195,3,0
       defb 0,3,131,2,0
       defb 0,1,67,2,0
       defb 0,1,195,131,0
       defb 0,67,195,195,2
       defb 1,195,195,195,131
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 65,75,195,195,195
       defb 65,75,195,195,195
       defb 65,135,195,195,195
       defb 65,195,75,195,195
       defb 1,195,135,195,131
       defb 1,67,195,195,3
       defb 0,3,195,131,2
       defb 0,1,67,3,0
       defb 0,3,131,0,0
       defb 0,67,195,2,0
       defb 1,195,195,131,0
       defb 67,195,195,195,2
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 195,195,195,135,130
       defb 195,195,195,135,130
       defb 195,195,195,75,130
       defb 195,195,135,195,130
       defb 67,195,75,195,2
       defb 3,195,195,131,2
       defb 1,67,195,3,0
       defb 0,3,131,2,0
       defb 0,1,67,2,0
       defb 0,1,195,131,0
       defb 0,67,195,195,2
       defb 1,195,195,195,131
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 65,195,195,195,75
       defb 65,195,195,195,75
       defb 65,195,195,135,195
       defb 65,195,195,75,195
       defb 1,195,135,195,131
       defb 1,67,195,195,3
       defb 0,3,195,131,2
       defb 0,1,67,3,0
       defb 0,3,131,0,0
       defb 0,67,195,2,0
       defb 1,195,195,131,0
       defb 67,195,75,195,2
       defb 195,195,135,195,130
       defb 195,195,195,75,130
       defb 195,195,195,135,130
       defb 195,195,195,135,130
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 67,195,195,195,2
       defb 3,195,195,131,2
       defb 1,67,195,3,0
       defb 0,3,131,2,0
       defb 0,1,67,2,0
       defb 0,1,195,131,0
       defb 0,67,195,195,2
       defb 1,195,135,195,131
       defb 65,195,195,75,195
       defb 65,195,195,135,195
       defb 65,195,195,195,75
       defb 65,195,195,195,75
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 1,195,195,195,131
       defb 1,67,195,195,3
       defb 0,3,195,131,2
       defb 0,1,67,3,0
       defb 0,3,131,0,0
       defb 0,67,195,2,0
       defb 1,195,195,131,0
       defb 67,195,75,195,2
       defb 195,135,195,195,130
       defb 195,75,195,195,130
       defb 135,195,195,195,130
       defb 135,195,195,195,130
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 67,195,195,195,2
       defb 3,195,195,131,2
       defb 1,67,195,3,0
       defb 0,3,131,2,0
       defb 0,1,67,2,0
       defb 0,1,195,131,0
       defb 0,67,195,195,2
       defb 1,195,135,195,131
       defb 65,195,75,195,195
       defb 65,135,195,195,195
       defb 65,75,195,195,195
       defb 65,75,195,195,195
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 1,195,195,195,131
       defb 1,67,195,195,3
       defb 0,3,195,131,2
       defb 0,1,67,3,0
       defb 0,3,131,0,0
       defb 0,67,195,2,0
       defb 1,195,195,131,0
       defb 67,195,75,195,2
       defb 195,195,135,195,130
       defb 195,195,195,75,130
       defb 195,195,195,135,130
       defb 195,195,195,135,130
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 67,195,195,195,2
       defb 3,195,195,131,2
       defb 1,67,195,3,0
       defb 0,3,131,2,0
       defb 0,1,67,2,0
       defb 0,1,195,131,0
       defb 0,67,195,195,2
       defb 1,195,135,195,131
       defb 65,195,195,75,195
       defb 65,195,195,135,195
       defb 65,195,195,195,75
       defb 65,195,195,195,75
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 1,195,195,195,131
       defb 1,67,195,195,3
       defb 0,3,195,131,2
       defb 0,1,67,3,0
       defb 0,3,131,0,0
       defb 0,67,195,2,0
       defb 1,195,195,131,0
       defb 67,195,195,195,2
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 195,195,195,135,130
       defb 195,195,195,135,130
       defb 195,195,195,75,130
       defb 195,195,135,195,130
       defb 67,195,75,195,2
       defb 3,195,195,131,2
       defb 1,67,195,3,0
       defb 0,3,131,2,0
       defb 0,1,67,2,0
       defb 0,1,195,131,0
       defb 0,67,195,195,2
       defb 1,195,195,195,131
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 65,195,195,195,75
       defb 65,195,195,195,75
       defb 65,195,195,135,195
       defb 65,195,195,75,195
       defb 1,195,135,195,131
       defb 1,67,195,195,3
       defb 0,3,195,131,2
       defb 0,1,67,3,0
       defb 0,3,131,0,0
       defb 0,67,195,2,0
       defb 1,195,195,131,0
       defb 67,195,195,195,2
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 135,195,195,195,130
       defb 135,195,195,195,130
       defb 195,75,195,195,130
       defb 195,135,195,195,130
       defb 67,195,75,195,2
       defb 3,195,195,131,2
       defb 1,67,195,3,0
       defb 0,3,131,2,0
       defb 0,1,67,2,0
       defb 0,1,195,131,0
       defb 0,67,195,195,2
       defb 1,195,195,195,131
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 65,75,195,195,195
       defb 65,75,195,195,195
       defb 65,135,195,195,195
       defb 65,195,75,195,195
       defb 1,195,135,195,131
       defb 1,67,195,195,3
       defb 0,3,195,131,2
       defb 0,1,67,3,0
       defb 0,0,10,0,0
       defb 0,5,15,0,0
       defb 3,11,11,10,0
       defb 7,15,15,15,0
       defb 7,135,15,135,0
       defb 7,135,7,135,0
       defb 3,15,15,11,0
       defb 0,1,11,0,0
       defb 0,195,75,130,0
       defb 8,131,11,130,0
       defb 9,135,15,131,0
       defb 131,7,135,3,0
       defb 195,7,135,3,0
       defb 0,135,135,3,0
       defb 0,7,15,2,0
       defb 0,65,65,0,0
       defb 0,0,5,0,0
       defb 0,0,15,10,0
       defb 1,7,7,7,0
       defb 1,15,15,15,10
       defb 1,75,15,75,10
       defb 1,75,11,75,10
       defb 1,7,15,15,2
       defb 0,0,7,2,0
       defb 0,65,135,195,0
       defb 4,65,7,67,0
       defb 4,67,15,75,2
       defb 65,3,75,11,2
       defb 65,131,75,11,2
       defb 0,65,75,11,2
       defb 0,1,15,11,0
       defb 0,0,130,130,0
       defb 0,0,10,0,0
       defb 0,5,15,0,0
       defb 3,11,11,10,0
       defb 7,11,11,15,0
       defb 7,135,15,135,0
       defb 7,135,7,135,0
       defb 3,15,15,11,0
       defb 65,1,11,65,0
       defb 65,195,75,195,0
       defb 3,3,11,2,0
       defb 3,7,15,3,0
       defb 3,7,135,3,0
       defb 3,7,135,3,0
       defb 73,135,135,3,0
       defb 0,7,15,2,0
       defb 0,67,67,0,0
       defb 0,0,5,0,0
       defb 0,0,15,10,0
       defb 1,7,7,7,0
       defb 1,15,7,7,10
       defb 1,75,15,75,10
       defb 1,75,11,75,10
       defb 1,7,15,15,2
       defb 0,130,7,2,130
       defb 0,195,135,195,130
       defb 1,3,7,3,0
       defb 1,3,15,11,2
       defb 1,3,75,11,2
       defb 1,3,75,11,2
       defb 4,195,75,11,2
       defb 0,1,15,11,0
       defb 0,1,131,130,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 69,0,0,69,0
       defb 69,0,0,69,2
       defb 207,0,0,69,138
       defb 207,0,0,69,138
       defb 207,0,0,69,138
       defb 207,3,0,69,138
       defb 207,3,2,69,138
       defb 207,1,2,69,138
       defb 207,139,2,207,138
       defb 69,138,35,207,0
       defb 0,207,103,138,0
       defb 0,1,35,0,0
       defb 0,69,103,0,0
       defb 0,69,207,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,138,0,0,138
       defb 0,138,0,0,139
       defb 69,138,0,0,207
       defb 69,138,0,0,207
       defb 69,138,0,0,207
       defb 69,139,2,0,207
       defb 69,139,3,0,207
       defb 69,138,3,0,207
       defb 69,207,3,69,207
       defb 0,207,17,71,138
       defb 0,69,155,207,0
       defb 0,0,19,2,0
       defb 0,0,155,138,0
       defb 0,0,207,138,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 1,2,0,1,0
       defb 3,0,0,1,2
       defb 3,0,0,1,2
       defb 3,0,0,1,2
       defb 3,0,0,1,2
       defb 3,3,0,1,2
       defb 207,3,2,69,138
       defb 207,1,2,69,138
       defb 207,139,2,207,138
       defb 207,138,35,207,138
       defb 71,207,103,207,2
       defb 3,1,35,1,2
       defb 0,69,103,0,0
       defb 0,69,207,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,3,0,0,2
       defb 1,2,0,0,3
       defb 1,2,0,0,3
       defb 1,2,0,0,3
       defb 1,2,0,0,3
       defb 1,3,2,0,3
       defb 69,139,3,0,207
       defb 69,138,3,0,207
       defb 69,207,3,69,207
       defb 69,207,17,71,207
       defb 1,207,155,207,139
       defb 1,2,19,2,3
       defb 0,0,155,138,0
       defb 0,0,207,138,0
       defb 1,2,0,2,0
       defb 0,195,65,130,0
       defb 0,131,33,130,0
       defb 0,16,48,2,0
       defb 32,48,48,32,32
       defb 48,48,48,48,32
       defb 48,3,33,18,32
       defb 48,3,33,18,32
       defb 18,3,33,18,2
       defb 3,48,48,33,2
       defb 3,117,255,33,2
       defb 3,48,186,33,0
       defb 1,48,48,32,0
       defb 0,16,48,0,0
       defb 0,5,5,2,0
       defb 0,7,5,2,0
       defb 0,3,0,1,0
       defb 0,65,130,195,0
       defb 0,65,18,67,0
       defb 0,0,48,33,0
       defb 16,16,48,48,16
       defb 16,48,48,48,48
       defb 16,33,18,3,48
       defb 16,33,18,3,48
       defb 1,33,18,3,33
       defb 1,18,48,48,3
       defb 1,18,255,186,3
       defb 1,18,117,48,2
       defb 0,18,48,48,0
       defb 0,0,48,32,0
       defb 0,0,10,11,0
       defb 0,1,10,11,0
       defb 1,67,65,2,0
       defb 0,67,65,2,0
       defb 0,3,33,2,0
       defb 0,16,48,2,0
       defb 0,48,48,32,0
       defb 16,48,48,48,0
       defb 48,3,33,18,32
       defb 48,3,33,18,32
       defb 48,3,33,18,32
       defb 33,48,48,33,32
       defb 3,117,255,33,2
       defb 3,48,186,33,0
       defb 1,48,48,32,0
       defb 0,16,48,0,0
       defb 0,5,5,0,0
       defb 0,15,5,10,0
       defb 0,3,130,131,0
       defb 0,1,130,131,0
       defb 0,1,18,3,0
       defb 0,0,48,33,0
       defb 0,16,48,48,0
       defb 0,48,48,48,32
       defb 16,33,18,3,48
       defb 16,33,18,3,48
       defb 16,33,18,3,48
       defb 16,18,48,48,18
       defb 1,18,255,186,3
       defb 1,18,117,48,2
       defb 0,18,48,48,0
       defb 0,0,48,32,0
       defb 0,0,10,10,0
       defb 0,5,10,15,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,12,0,0
       defb 0,0,12,8,2
       defb 0,2,12,9,2
       defb 15,15,8,12,0
       defb 10,15,8,12,0
       defb 15,10,8,8,8
       defb 10,10,12,8,0
       defb 10,10,12,0,0
       defb 10,10,12,0,0
       defb 10,10,0,0,0
       defb 10,10,0,0,0
       defb 10,10,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,4,8,0
       defb 0,0,4,12,1
       defb 0,1,4,12,3
       defb 5,15,14,4,8
       defb 5,5,14,4,8
       defb 5,15,4,4,4
       defb 5,5,4,12,0
       defb 5,5,4,8,0
       defb 5,5,4,8,0
       defb 5,5,0,0,0
       defb 5,5,0,0,0
       defb 5,5,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,48,0,0
       defb 0,0,48,32,2
       defb 0,2,48,33,32
       defb 15,15,32,48,0
       defb 10,15,32,48,0
       defb 15,10,32,32,2
       defb 10,10,48,32,0
       defb 10,10,48,0,0
       defb 10,10,48,0,0
       defb 10,10,0,0,0
       defb 10,10,0,0,0
       defb 10,10,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,16,32,0
       defb 0,0,16,48,1
       defb 0,1,16,48,18
       defb 5,15,26,16,32
       defb 5,5,26,16,32
       defb 5,15,16,16,1
       defb 5,5,16,48,0
       defb 5,5,16,32,0
       defb 5,5,16,32,0
       defb 5,5,0,0,0
       defb 5,5,0,0,0
       defb 5,5,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,4,8,0,0
       defb 2,12,8,0,0
       defb 3,12,8,2,0
       defb 4,8,13,15,10
       defb 4,8,13,10,10
       defb 8,8,8,15,10
       defb 0,12,8,10,10
       defb 0,4,8,10,10
       defb 0,4,8,10,10
       defb 0,0,0,10,10
       defb 0,0,0,10,10
       defb 0,0,0,10,10
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,12,0,0
       defb 1,4,12,0,0
       defb 1,6,12,1,0
       defb 0,12,4,15,15
       defb 0,12,4,15,5
       defb 4,4,4,5,15
       defb 0,4,12,5,5
       defb 0,0,12,5,5
       defb 0,0,12,5,5
       defb 0,0,0,5,5
       defb 0,0,0,5,5
       defb 0,0,0,5,5
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,16,32,0,0
       defb 2,48,32,0,0
       defb 33,48,32,2,0
       defb 16,32,37,15,10
       defb 16,32,37,10,10
       defb 2,32,32,15,10
       defb 0,48,32,10,10
       defb 0,16,32,10,10
       defb 0,16,32,10,10
       defb 0,0,0,10,10
       defb 0,0,0,10,10
       defb 0,0,0,10,10
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,48,0,0
       defb 1,16,48,0,0
       defb 16,18,48,1,0
       defb 0,48,16,15,15
       defb 0,48,16,15,5
       defb 1,16,16,5,15
       defb 0,16,48,5,5
       defb 0,0,48,5,5
       defb 0,0,48,5,5
       defb 0,0,0,5,5
       defb 0,0,0,5,5
       defb 0,0,0,5,5
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,195,195,130,0
       defb 65,15,15,75,0
       defb 135,15,15,15,130
       defb 195,195,195,195,130
       defb 195,195,195,195,130
       defb 135,15,135,15,130
       defb 135,15,135,15,130
       defb 135,15,135,15,130
       defb 135,15,135,15,130
       defb 135,15,135,15,130
       defb 195,195,195,195,130
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,0,0,0,0
       defb 0,65,195,195,0
       defb 0,135,15,15,130
       defb 65,15,15,15,75
       defb 65,195,195,195,195
       defb 65,195,195,195,195
       defb 65,15,75,15,75
       defb 65,15,75,15,75
       defb 65,15,75,15,75
       defb 65,15,75,15,75
       defb 65,15,75,15,75
       defb 65,195,195,195,195
       defb 0,195,195,130,0
       defb 65,4,4,65,0
       defb 131,8,8,9,130
       defb 131,4,4,0,130
       defb 130,8,8,8,130
       defb 134,6,4,5,130
       defb 131,9,0,15,130
       defb 131,6,5,15,130
       defb 131,3,15,15,130
       defb 131,3,15,15,130
       defb 131,2,15,15,130
       defb 130,0,15,15,130
       defb 67,0,15,75,0
       defb 1,130,15,130,0
       defb 1,67,75,0,0
       defb 0,3,131,0,0
       defb 0,65,195,195,0
       defb 0,130,8,8,130
       defb 65,6,4,4,67
       defb 65,2,8,8,65
       defb 65,4,4,4,65
       defb 65,9,8,8,75
       defb 65,6,2,5,75
       defb 65,3,8,15,75
       defb 65,3,7,15,75
       defb 65,3,7,15,75
       defb 65,3,5,15,75
       defb 65,0,5,15,75
       defb 1,130,5,15,130
       defb 0,67,5,75,0
       defb 0,3,135,130,0
       defb 0,1,67,2,0
       defb 6,12,12,12,2
       defb 13,15,15,15,8
       defb 13,15,15,15,8
       defb 12,15,15,14,8
       defb 73,12,12,73,8
       defb 73,195,195,195,8
       defb 6,195,135,134,2
       defb 6,135,195,134,2
       defb 6,195,195,134,2
       defb 6,75,135,134,2
       defb 4,73,195,12,2
       defb 0,73,75,9,0
       defb 0,73,195,9,0
       defb 0,12,134,8,0
       defb 0,4,12,0,0
       defb 0,0,8,0,0
       defb 1,12,12,12,9
       defb 4,15,15,15,14
       defb 4,15,15,15,14
       defb 4,13,15,15,12
       defb 4,134,12,12,134
       defb 4,195,195,195,134
       defb 1,73,195,75,9
       defb 1,73,75,195,9
       defb 1,73,195,195,9
       defb 1,13,195,75,9
       defb 0,12,195,134,9
       defb 0,4,135,134,2
       defb 0,4,195,134,2
       defb 0,4,73,12,0
       defb 0,0,12,8,0
       defb 0,0,4,0,0
       defb 0,0,10,0,0
       defb 0,5,143,0,0
       defb 0,79,207,10,0
       defb 5,15,15,15,0
       defb 1,255,255,171,0
       defb 1,170,170,171,0
       defb 85,255,255,255,0
       defb 85,255,255,255,0
       defb 85,171,3,255,0
       defb 0,255,255,170,0
       defb 0,85,255,0,0
       defb 0,0,170,0,0
       defb 0,0,170,0,0
       defb 1,12,12,9,0
       defb 4,12,12,12,0
       defb 12,4,12,4,8
       defb 0,0,5,0,0
       defb 0,0,79,10,0
       defb 0,5,207,143,0
       defb 0,15,15,15,10
       defb 0,87,255,255,2
       defb 0,87,85,85,2
       defb 0,255,255,255,170
       defb 0,255,255,255,170
       defb 0,255,3,87,170
       defb 0,85,255,255,0
       defb 0,0,255,170,0
       defb 0,0,85,0,0
       defb 0,0,85,0,0
       defb 0,6,12,12,2
       defb 0,12,12,12,8
       defb 4,8,12,8,12
       defb 0,0,10,0,0
       defb 0,5,143,0,0
       defb 0,79,207,10,0
       defb 5,15,15,15,0
       defb 1,255,255,171,0
       defb 1,170,170,171,0
       defb 127,171,171,255,42
       defb 127,255,255,255,42
       defb 3,171,3,171,2
       defb 0,171,3,170,0
       defb 0,85,255,0,0
       defb 0,0,170,0,0
       defb 0,0,170,0,0
       defb 1,12,12,9,0
       defb 4,12,12,12,0
       defb 12,4,12,4,8
       defb 0,0,5,0,0
       defb 0,0,79,10,0
       defb 0,5,207,143,0
       defb 0,15,15,15,10
       defb 0,87,255,255,2
       defb 0,87,85,85,2
       defb 21,255,87,87,191
       defb 21,255,255,255,191
       defb 1,87,3,87,3
       defb 0,85,3,87,0
       defb 0,0,255,170,0
       defb 0,0,85,0,0
       defb 0,0,85,0,0
       defb 0,6,12,12,2
       defb 0,12,12,12,8
       defb 4,8,12,8,12
       defb 9,4,6,2,8
       defb 9,4,6,2,8
       defb 9,4,12,2,8
       defb 12,2,8,4,8
       defb 4,9,8,12,2
       defb 0,12,12,12,0
       defb 0,12,12,8,0
       defb 0,204,204,136,0
       defb 0,204,68,136,0
       defb 0,137,1,204,0
       defb 68,2,0,70,136
       defb 68,2,2,3,136
       defb 137,70,70,1,136
       defb 137,70,68,1,136
       defb 68,137,0,204,0
       defb 68,137,0,204,0
       defb 4,2,9,9,4
       defb 4,2,9,9,4
       defb 4,2,12,9,4
       defb 4,9,4,0,12
       defb 0,12,6,4,9
       defb 0,4,12,12,8
       defb 0,4,12,12,0
       defb 0,68,204,204,0
       defb 0,68,136,204,0
       defb 0,68,2,70,136
       defb 0,137,0,1,204
       defb 0,137,1,1,70
       defb 68,3,137,136,70
       defb 68,3,136,136,70
       defb 0,204,2,68,136
       defb 0,204,2,68,136
       defb 9,9,3,8,8
       defb 9,9,3,8,8
       defb 9,9,3,8,8
       defb 12,9,2,12,8
       defb 9,9,2,9,8
       defb 12,12,12,12,8
       defb 0,12,12,8,0
       defb 0,204,204,136,0
       defb 0,204,68,136,0
       defb 0,137,1,204,0
       defb 68,2,0,70,136
       defb 68,2,2,3,136
       defb 137,70,70,1,136
       defb 137,70,68,1,136
       defb 68,137,0,204,0
       defb 68,137,0,204,0
       defb 4,6,3,6,4
       defb 4,6,3,6,4
       defb 4,6,3,6,4
       defb 4,12,3,4,12
       defb 4,6,3,4,6
       defb 4,12,12,12,12
       defb 0,4,12,12,0
       defb 0,68,204,204,0
       defb 0,68,136,204,0
       defb 0,68,2,70,136
       defb 0,137,0,1,204
       defb 0,137,1,1,70
       defb 68,3,137,136,70
       defb 68,3,136,136,70
       defb 0,204,2,68,136
       defb 0,204,2,68,136
       defb 160,0,0,0,160
       defb 160,0,40,0,160
       defb 160,20,60,0,160
       defb 160,20,60,0,160
       defb 160,60,60,40,160
       defb 80,40,40,120,0
       defb 0,40,40,40,0
       defb 2,60,60,40,0
       defb 3,60,22,40,0
       defb 1,60,22,40,0
       defb 0,60,60,40,0
       defb 0,20,60,0,0
       defb 0,1,131,0,0
       defb 0,1,131,0,0
       defb 0,1,131,0,0
       defb 0,1,131,0,0
       defb 80,0,0,0,80
       defb 80,0,20,0,80
       defb 80,0,60,40,80
       defb 80,0,60,40,80
       defb 80,20,60,60,80
       defb 0,180,20,20,160
       defb 0,20,20,20,0
       defb 1,20,60,60,0
       defb 1,22,41,60,0
       defb 0,22,41,60,0
       defb 0,20,60,60,0
       defb 0,0,60,40,0
       defb 0,0,67,2,0
       defb 0,0,67,2,0
       defb 0,0,67,2,0
       defb 0,0,67,2,0
       defb 2,0,0,0,2
       defb 2,0,40,0,2
       defb 2,20,60,0,2
       defb 2,20,60,0,2
       defb 2,60,60,40,2
       defb 82,60,60,120,0
       defb 160,40,40,40,160
       defb 160,60,60,40,160
       defb 161,60,60,40,160
       defb 161,60,22,40,160
       defb 160,41,41,40,160
       defb 160,20,60,0,160
       defb 0,1,131,0,0
       defb 0,1,131,0,0
       defb 0,1,131,0,0
       defb 0,1,131,0,0
       defb 1,0,0,0,1
       defb 1,0,20,0,1
       defb 1,0,60,40,1
       defb 1,0,60,40,1
       defb 1,20,60,60,1
       defb 1,180,60,60,160
       defb 80,20,20,20,80
       defb 80,20,60,60,80
       defb 80,22,60,60,80
       defb 80,22,41,60,80
       defb 80,20,22,22,80
       defb 80,0,60,40,80
       defb 0,0,67,2,0
       defb 0,0,67,2,0
       defb 0,0,67,2,0
       defb 0,0,67,2,0
       defb 0,3,131,2,0
       defb 0,3,131,2,0
       defb 0,3,131,2,0
       defb 65,195,195,195,0
       defb 131,195,195,131,130
       defb 195,67,195,67,130
       defb 195,131,131,195,130
       defb 195,195,195,130,130
       defb 195,195,195,130,130
       defb 65,1,0,65,0
       defb 0,11,11,10,0
       defb 0,11,10,10,0
       defb 0,11,10,10,0
       defb 0,11,10,10,0
       defb 0,11,10,10,0
       defb 1,11,10,10,0
       defb 0,1,67,3,0
       defb 0,1,67,3,0
       defb 0,1,67,3,0
       defb 0,195,195,195,130
       defb 65,67,195,195,67
       defb 65,131,195,131,195
       defb 65,195,67,67,195
       defb 65,195,195,195,65
       defb 65,195,195,195,65
       defb 0,130,2,0,130
       defb 0,5,7,7,0
       defb 0,5,7,5,0
       defb 0,5,7,5,0
       defb 0,5,7,5,0
       defb 0,5,7,5,0
       defb 0,7,7,5,0
       defb 0,3,131,2,0
       defb 0,3,131,2,0
       defb 0,3,131,2,0
       defb 65,195,195,195,0
       defb 131,195,195,131,130
       defb 131,195,195,131,130
       defb 131,195,195,131,130
       defb 131,195,195,130,130
       defb 131,195,195,130,130
       defb 65,1,0,65,0
       defb 0,11,11,10,0
       defb 5,3,10,7,0
       defb 10,3,11,2,10
       defb 2,3,10,2,0
       defb 0,3,2,2,0
       defb 1,3,2,2,0
       defb 0,1,67,3,0
       defb 0,1,67,3,0
       defb 0,1,67,3,0
       defb 0,195,195,195,130
       defb 65,67,195,195,67
       defb 65,67,195,195,67
       defb 65,67,195,195,67
       defb 65,67,195,195,65
       defb 65,67,195,195,65
       defb 0,130,2,0,130
       defb 0,5,7,7,0
       defb 0,11,7,1,10
       defb 5,1,7,3,5
       defb 1,1,7,1,0
       defb 0,1,3,1,0
       defb 0,3,3,1,0
       defb 0,0,0,0,0
       defb 0,0,8,0,0
       defb 0,4,12,2,0
       defb 0,12,12,8,0
       defb 4,12,12,12,0
       defb 12,3,9,6,8
       defb 12,3,9,6,8
       defb 12,12,12,12,8
       defb 12,9,9,12,8
       defb 12,12,12,12,8
       defb 4,6,12,6,0
       defb 0,9,3,8,0
       defb 0,4,12,2,0
       defb 0,0,128,0,0
       defb 64,192,192,192,0
       defb 128,192,192,129,128
       defb 0,0,0,0,0
       defb 0,0,4,0,0
       defb 0,0,12,9,0
       defb 0,4,12,12,0
       defb 0,12,12,12,8
       defb 4,9,6,3,12
       defb 4,9,6,3,12
       defb 4,12,12,12,12
       defb 4,12,6,6,12
       defb 4,12,12,12,12
       defb 0,9,12,9,8
       defb 0,4,3,6,0
       defb 0,0,12,9,0
       defb 0,0,64,0,0
       defb 0,192,192,192,128
       defb 64,64,192,192,66
       defb 0,0,0,0,0
       defb 0,0,8,0,0
       defb 0,4,12,2,0
       defb 0,12,12,8,0
       defb 4,12,12,12,0
       defb 12,3,9,6,8
       defb 12,12,12,12,8
       defb 12,12,12,12,8
       defb 12,9,9,12,8
       defb 12,12,12,12,8
       defb 4,9,3,12,0
       defb 0,9,3,8,0
       defb 0,4,12,2,0
       defb 0,0,128,0,0
       defb 64,192,192,192,0
       defb 128,192,192,129,128
       defb 0,0,0,0,0
       defb 0,0,4,0,0
       defb 0,0,12,9,0
       defb 0,4,12,12,0
       defb 0,12,12,12,8
       defb 4,9,6,3,12
       defb 4,12,12,12,12
       defb 4,12,12,12,12
       defb 4,12,6,6,12
       defb 4,12,12,12,12
       defb 0,12,3,6,8
       defb 0,4,3,6,0
       defb 0,0,12,9,0
       defb 0,0,64,0,0
       defb 0,192,192,192,128
       defb 64,64,192,192,66
       defb 129,192,192,129,128
       defb 129,192,192,129,128
       defb 129,192,192,129,128
       defb 129,192,192,129,128
       defb 128,192,192,129,128
       defb 128,192,192,128,128
       defb 128,192,192,129,128
       defb 128,64,192,1,128
       defb 192,0,10,64,128
       defb 5,10,10,15,0
       defb 0,15,15,10,0
       defb 0,7,15,2,0
       defb 0,67,65,2,0
       defb 0,67,65,2,0
       defb 1,195,65,131,0
       defb 1,195,65,131,0
       defb 64,66,192,192,66
       defb 64,66,192,192,66
       defb 64,66,192,192,66
       defb 64,66,192,192,66
       defb 64,64,192,192,66
       defb 64,64,192,192,64
       defb 64,64,192,192,66
       defb 64,0,192,128,66
       defb 64,128,5,0,192
       defb 0,15,5,5,10
       defb 0,5,15,15,0
       defb 0,1,15,11,0
       defb 0,1,130,131,0
       defb 0,1,130,131,0
       defb 0,67,130,195,2
       defb 0,67,130,195,2
       defb 129,66,192,3,128
       defb 129,66,192,3,128
       defb 129,66,192,3,128
       defb 129,66,192,3,128
       defb 128,66,192,3,128
       defb 128,66,192,2,128
       defb 192,192,192,192,128
       defb 192,192,192,192,128
       defb 15,15,15,15,10
       defb 5,15,15,15,0
       defb 0,15,15,10,0
       defb 0,7,15,2,0
       defb 0,67,65,2,0
       defb 0,67,65,2,0
       defb 1,195,65,131,0
       defb 1,195,65,131,0
       defb 64,3,192,129,66
       defb 64,3,192,129,66
       defb 64,3,192,129,66
       defb 64,3,192,129,66
       defb 64,1,192,129,66
       defb 64,1,192,129,64
       defb 64,192,192,192,192
       defb 64,192,192,192,192
       defb 5,15,15,15,15
       defb 0,15,15,15,10
       defb 0,5,15,15,0
       defb 0,1,15,11,0
       defb 0,1,130,131,0
       defb 0,1,130,131,0
       defb 0,67,130,195,2
       defb 0,67,130,195,2
       defb 0,0,34,0,0
       defb 0,0,35,0,0
       defb 0,17,51,0,0
       defb 0,51,51,34,0
       defb 17,38,38,59,0
       defb 51,25,51,25,34
       defb 38,51,51,38,34
       defb 38,51,51,38,34
       defb 38,51,51,38,34
       defb 38,51,51,38,34
       defb 51,25,51,25,34
       defb 17,38,38,51,0
       defb 0,51,51,34,0
       defb 0,17,51,0,0
       defb 0,0,34,0,0
       defb 0,0,34,0,0
       defb 0,0,17,0,0
       defb 0,0,17,2,0
       defb 0,0,51,34,0
       defb 0,17,51,51,0
       defb 0,51,25,29,34
       defb 17,38,51,38,51
       defb 17,25,51,51,25
       defb 17,25,51,51,25
       defb 17,25,51,51,25
       defb 17,25,51,51,25
       defb 17,38,51,38,51
       defb 0,51,25,25,34
       defb 0,17,51,51,0
       defb 0,0,51,34,0
       defb 0,0,17,0,0
       defb 0,0,17,0,0
frmlst equ $
       defb 0,2
       defb 2,2
       defb 4,1
       defb 5,1
       defb 6,2
       defb 8,1
       defb 9,1
       defb 10,1
       defb 11,1
       defb 12,1
       defb 13,2
       defb 15,2
       defb 17,2
       defb 19,2
       defb 21,2
       defb 23,2
       defb 25,2
       defb 27,2
       defb 29,2
       defb 31,2
       defb 33,2
       defb 35,2
       defb 37,1
       defb 38,4
       defb 42,4
       defb 46,2
       defb 48,2
       defb 50,2
       defb 52,2
       defb 54,2
       defb 56,1
       defb 57,1
       defb 58,1
       defb 59,2
       defb 61,2
       defb 63,2
       defb 65,2
       defb 67,2
       defb 69,2
       defb 71,1,72,0
scdat  equ $
       defw 223,224,151,159,153,111,119,149,140,109,97,236,180,182,147,193,191,175,189,113,168,166,159,164,262,142
       defw 314,266,209,97
       defb 255,0,6,30,30,255,0,15,30,30,255,0,65,19,255,0,28,19,255,8,5,255,0,9,19,255,0,10,255,8,5,255,0,8
       defb 15,0,0,0,255,8,6,0,0,0,15,255,0,16,15,255,0,12,15,255,0,8,19,255,0,7,15,255,0,12,15,255,0,7
       defb 19,255,8,5,0,0,0,15,255,0,4,4,3,0,19,255,0,4,15,0,0,0,255,8,5,255,0,8,15,8,0,0,0
       defb 255,8,4,0,0,0,8,15,255,0,16,15,255,0,12,15,255,0,16,15,255,0,12,15,255,0,16,15,255,0,12,15,255,0,8
       defb 19,0,4,255,0,5,15,255,0,12,15,255,0,5,5,0,19,255,8,5,0,0,0,29,0,0,0,255,8,6,0,0,0
       defb 28,0,0,0,255,8,5,29,27,29,27,27,255,0,7,29,27,27,29,29,27,255,0,7,27,28,27,28,29,255,0,38,5,5
       defb 3,4,3,5,4,5,5,3,4,3,5,255,0,9,255,8,30,255,15,30
       defb 255,0,90,19,0,0,0,19,255,0,21,19,0,0,0,255,8,10,255,0,11,255,8,9,29,28,27,29,27,28,29,28,27
       defb 15,3,255,0,10,15,27,28,29,27,27,28,28,29,15,255,0,8,8,8,8,255,0,7,8,8,15,255,0,7,15,15,255,0,8
       defb 28,27,28,255,0,7,27,29,27,255,0,7,15,15,255,0,28,15,15,255,0,11,19,0,0,3,255,0,13,15,15,255,0,11
       defb 255,8,6,255,0,11,15,15,255,0,11,27,27,29,27,29,27,255,0,11,15,15,255,0,28,15,15,255,0,9,19,255,0,13
       defb 19,255,0,4,15,15,255,0,7,255,8,5,255,0,4,255,8,9,0,0,0,15,15,255,0,7,28,27,29,28,15,255,0,4
       defb 15,27,27,28,27,29,27,29,27,0,0,0,15,15,255,0,11,15,255,0,4,15,255,0,11,15,15,0,0,0,4,0,0
       defb 19,255,0,4,15,255,0,4,15,0,19,255,0,9,15,255,8,30,255,15,30
       defb 255,0,21,30,255,0,51,19,255,0,15,19,255,0,11,255,8,7,255,0,8,255,8,4,255,0,34,19,4,255,0,24,255,8,6
       defb 255,0,24,27,28,27,28,28,15,255,0,4,5,4,5,4,0,19,255,0,19,15,255,8,13,255,0,9,19,3,255,0,5
       defb 255,27,11,255,0,11,8,8,8,255,0,24,5,3,8,8,27,29,255,0,22,5,4,8,8,8,27,255,0,12,15,255,8,14
       defb 27,28,0,0,0,19,255,0,9,15,255,0,13,15,255,0,5,8,255,0,7,8,8,15,255,0,13,15,255,0,79,19,0,0,0
       defb 4,4,255,0,7,4,0,19,255,8,30,255,15,30
       defb 255,0,90,19,255,0,11,4,0,19,4,4,4,255,0,11,19,255,8,4,0,0,0,255,8,11,0,0,0,255,8,9,255,15,4
       defb 0,0,0,255,15,11,0,0,0,255,15,9,255,0,60,4,4,4,0,19,255,0,16,19,4,4,255,0,5,19,255,8,5
       defb 255,0,14,255,8,5,255,0,5,8,255,0,19,255,15,5,255,0,5,15,255,0,63,19,255,0,4,3,255,0,5,4,5
       defb 255,0,11,4,4,0,255,8,9,0,0,0,255,8,6,255,0,4,255,8,8,255,15,9,0,0,0,255,15,6,255,0,4
       defb 255,15,8,255,0,42,3,3,4,3,4,3,3,255,0,8,19,0,0,255,8,30,255,15,30
       defb 255,0,104,19,255,0,27,255,8,5,255,0,27,15,255,0,28,19,15,255,0,18,19,255,0,6,255,8,4,15,255,8,4
       defb 255,0,14,8,8,255,0,9,15,255,0,9,8,8,255,0,9,8,8,255,0,7,15,255,0,7,8,8,255,0,6,19,255,0,6
       defb 8,8,255,0,5,15,255,0,5,8,8,255,0,7,19,8,8,255,0,7,8,8,0,0,0,15,0,0,0,8,8,255,0,8
       defb 8,8,255,0,14,15,255,0,15,19,255,0,13,15,255,0,14,19,255,8,7,255,0,7,15,255,0,8,255,8,7,255,0,13
       defb 8,8,8,255,0,48,19,255,0,24,19,255,8,30,255,15,30
       defb 255,0,30,19,255,0,28,19,8,8,8,255,0,5,255,8,14,255,0,5,8,8,8,255,0,10,30,30,255,0,78,19,255,0,12
       defb 3,0,19,255,0,6,3,3,0,19,255,0,4,255,8,8,255,0,5,255,8,5,255,0,4,255,8,6,255,0,82,19,255,0,28
       defb 255,8,4,255,0,71,19,3,3,3,255,0,18,3,3,0,19,0,0,255,8,7,255,0,10,255,8,11,255,0,44,19,255,0,15
       defb 255,8,30,255,15,30
       defb 255,0,30,19,255,0,28,19,255,8,6,255,0,21,8,8,8,255,0,42,19,0,3,4,3,3,5,255,0,20,255,8,10
       defb 255,0,7,19,255,0,5,19,255,0,21,255,8,6,0,0,8,4,4,255,0,28,8,8,255,0,30,8,8,255,0,23,19
       defb 255,0,6,8,8,0,19,255,0,19,8,8,255,0,7,8,8,255,0,10,255,8,9,255,0,30,19,0,0,0,4,255,0,25
       defb 255,8,14,255,0,93,4,5,3,255,0,9,19,255,8,30,255,15,30
       defb 255,0,132,15,8,8,0,0,0,255,8,5,255,0,4,8,8,8,255,0,12,15,255,0,17,19,0,0,0,5,5,255,0,6
       defb 15,19,255,0,12,4,5,4,3,255,8,9,0,0,0,255,8,7,255,0,7,255,8,4,255,0,18,15,8,255,0,28,15,15
       defb 8,4,255,0,10,19,255,0,14,19,15,15,15,8,255,0,10,255,8,7,0,0,0,255,8,10,255,0,21,15,255,0,16
       defb 19,255,0,12,15,19,255,0,10,4,255,0,4,255,8,11,0,0,255,8,13,0,0,0,8,255,0,13,15,255,0,29,15
       defb 255,0,29,15,19,255,0,14,5,255,8,30,255,15,30
       defb 255,0,30,19,255,0,29,8,8,255,0,95,3,255,0,21,19,255,8,8,255,0,13,15,255,8,8,255,15,8,255,0,11
       defb 19,3,15,255,0,17,8,8,255,0,8,8,8,8,255,0,52,4,3,4,255,0,12,19,255,0,7,255,8,16,255,0,5
       defb 8,8,255,0,10,255,15,10,255,0,21,15,15,255,0,23,4,4,3,4,3,3,15,255,0,8,19,0,0,0,4,255,0,8
       defb 255,8,8,15,255,0,4,255,8,9,255,0,16,15,255,0,28,19,15,19,0,0,0,4,255,0,11,4,255,8,12,255,15,6
       defb 255,8,12,255,15,30
       defb 255,0,30,255,15,31,255,0,28,15,15,255,0,28,15,15,255,0,28,15,15,8,8,8,255,0,22,8,8,15,15,15,255,0,28
       defb 15,15,255,0,28,15,15,255,0,28,15,15,255,8,4,255,0,20,255,8,4,15,15,255,0,28,15,15,255,0,28,15,15
       defb 255,0,28,15,15,255,0,28,15,15,255,8,7,255,0,14,255,8,7,15,15,255,0,28,15,15,255,0,28,15,15,255,0,28
       defb 15,255,8,30,255,15,30
       defb 255,0,124,26,255,0,21,26,255,0,6,7,7,255,0,21,255,7,4,255,0,12,4,4,0,26,255,0,26,255,7,7,255,0,44
       defb 26,255,0,4,4,4,255,0,19,26,0,0,255,7,8,255,0,16,255,7,4,255,0,60,26,255,0,8,3,4,255,0,10
       defb 4,4,255,0,6,26,255,7,12,255,0,8,255,7,10,255,0,75,4,4,255,0,13,255,10,30,255,7,30
       defb 255,0,13,255,7,4,255,0,26,255,7,4,255,0,26,255,7,4,255,0,26,255,7,4,255,0,13,26,0,0,0,4,4
       defb 255,0,7,255,7,4,255,0,5,3,3,255,0,5,26,255,10,9,255,0,4,255,7,4,255,0,4,255,10,9,0,0,0
       defb 7,255,0,9,255,7,4,255,0,9,7,255,0,6,7,255,0,8,4,255,7,4,4,0,26,255,0,6,7,0,0,0,26
       defb 0,0,7,0,0,0,255,10,16,0,0,0,7,0,0,26,10,0,0,7,255,0,22,7,0,0,10,0,0,0,7,255,0,22
       defb 7,255,0,6,7,0,0,255,10,18,0,0,7,255,0,5,10,7,255,0,9,255,7,4,255,0,9,7,7,255,0,5,7
       defb 255,0,9,255,7,4,255,0,7,26,0,7,255,0,6,255,10,4,255,0,6,255,7,4,0,0,255,10,8,0,0,0,10
       defb 255,0,12,255,7,4,255,0,12,10,255,0,13,255,7,4,255,0,23,4,0,26,255,7,4,0,26,255,0,10,4,255,10,30
       defb 255,7,30
       defb 255,0,127,3,255,0,16,3,0,0,0,26,0,0,0,255,10,7,255,0,14,255,10,6,255,0,5,7,255,0,21,7,255,0,6
       defb 26,7,26,255,0,13,4,255,0,4,26,0,7,26,255,0,5,10,7,255,10,7,255,0,5,255,10,9,7,10,255,0,6
       defb 7,255,0,12,7,255,0,8,7,255,0,7,7,255,0,8,26,0,0,0,7,255,0,8,7,0,0,0,10,0,0,0,7
       defb 255,0,7,255,10,6,255,0,8,7,0,0,10,255,0,4,7,255,0,21,7,255,0,7,7,255,0,21,7,255,0,7,7
       defb 0,0,0,4,255,0,12,26,4,0,0,0,7,255,0,11,255,10,15,255,0,19,255,7,6,255,0,20,4,3,4,4,255,7,6
       defb 0,26,255,0,10,255,10,30,255,7,30
       defb 255,0,22,30,30,255,0,76,3,3,4,0,0,0,26,255,0,12,26,255,0,10,255,10,11,255,0,7,10,10,255,0,14
       defb 7,7,7,255,0,13,26,0,0,4,4,5,255,0,8,7,7,7,255,0,7,4,255,0,4,26,255,10,9,255,0,5,7,7,7
       defb 255,0,7,255,10,6,255,0,13,4,7,7,7,26,255,0,23,255,10,4,7,255,10,4,255,0,100,26,255,0,7,4,255,0,14
       defb 4,255,0,6,10,10,10,255,0,4,255,10,7,255,0,4,255,10,7,0,0,0,10,10,255,0,9,7,7,7,255,0,8
       defb 7,7,7,255,0,16,7,7,7,255,0,8,7,7,7,255,0,13,4,4,4,7,7,7,4,3,4,0,0,0,26,0,7,7,7
       defb 0,26,255,0,5,255,10,30,255,7,30
       defb 0,0,0,30,255,0,86,10,10,10,255,0,5,255,10,4,26,255,0,7,255,10,9,7,255,0,12,10,255,0,4,4,4
       defb 10,255,0,23,255,10,6,255,0,41,10,10,10,255,0,57,4,4,0,26,255,0,17,4,255,10,5,0,0,0,255,10,9
       defb 4,255,0,10,4,10,255,0,17,10,4,255,0,8,26,10,255,0,19,10,255,0,8,10,255,0,9,26,0,0,0,26,255,0,7
       defb 255,10,8,255,0,7,255,10,4,0,0,255,10,4,255,0,92,4,255,0,12,5,4,255,0,6,26,255,10,30,7,7,255,10,7
       defb 7,10,255,7,5,255,10,4,255,7,10
       defb 7,255,0,7,7,255,0,5,30,0,0,0,7,255,0,10,7,7,255,0,7,7,255,0,9,7,255,0,10,7,7,255,0,6
       defb 26,7,255,0,9,7,255,0,10,7,7,26,255,0,4,7,7,7,255,0,9,7,255,0,9,26,7,7,10,255,0,14,10,10,10
       defb 255,0,8,10,10,10,255,0,53,4,255,0,11,10,10,10,26,255,0,13,255,10,4,255,0,11,10,10,10,255,0,10,10,10,10
       defb 255,0,14,10,255,0,14,10,255,0,14,10,255,0,14,10,255,0,7,26,0,0,255,4,4,10,4,4,4,255,0,8,4,4,4
       defb 10,26,255,0,6,255,10,11,255,0,8,255,10,9,255,0,32,26,255,0,28,26,255,10,13,255,0,7,255,10,10,255,0,46
       defb 3,5,5,4,3,4,5,4,5,5,4,5,3,3,255,10,30,255,7,30
       defb 255,0,56,26,0,0,0,10,10,10,255,0,15,255,10,10,255,0,4,7,255,0,29,10,10,10,255,0,29,7,255,0,13
       defb 4,4,0,0,26,255,0,11,7,10,10,10,255,0,8,255,10,7,255,0,23,7,255,0,5,7,0,0,0,4,0,0,26
       defb 255,0,12,255,10,4,7,255,0,5,7,0,0,0,255,10,4,255,0,11,4,7,255,0,9,7,255,0,16,10,10,10,7
       defb 255,0,8,26,7,26,255,0,15,7,255,0,10,10,10,7,10,10,10,255,0,8,4,255,10,4,7,255,0,12,7,255,0,10
       defb 10,10,10,255,0,11,4,0,0,0,26,7,255,0,6,26,255,0,16,255,10,6,7,255,0,4,10,10,10,255,0,22,7
       defb 255,0,29,7,255,0,19,4,255,0,7,26,0,7,4,255,0,5,26,255,10,30,255,7,30
       defb 255,0,59,26,255,10,7,255,0,17,255,10,7,255,0,28,10,10,255,0,28,10,10,255,4,4,255,0,5,4,255,10,9
       defb 255,0,9,255,10,6,0,0,0,26,10,10,255,0,9,10,10,26,255,0,4,4,4,10,255,0,8,10,10,255,0,12,255,10,8
       defb 255,0,30,26,255,0,28,26,255,10,4,255,0,21,255,10,6,0,0,0,10,10,255,0,18,10,10,0,0,0,10,10,255,0,5
       defb 10,10,4,4,255,0,10,4,4,10,10,255,0,5,10,10,255,0,7,10,10,255,0,10,10,10,255,0,7,10,10,10,255,0,8
       defb 10,10,255,0,6,10,10,255,0,9,10,10,255,0,26,255,10,4,255,0,28,10,10,255,0,23,4,4,0,0,26,255,10,31
       defb 255,7,30
       defb 255,0,11,7,0,0,30,30,0,0,7,255,0,22,7,255,0,6,7,255,0,22,7,255,0,6,7,255,0,10,26,255,0,11
       defb 7,26,255,0,4,26,7,255,0,9,255,10,4,255,0,9,7,10,255,0,4,10,7,255,0,41,26,255,0,4,4,4,255,0,15
       defb 4,255,0,6,26,255,10,11,255,0,8,255,10,11,255,0,11,10,255,0,6,10,255,0,11,26,255,0,11,10,255,0,4
       defb 10,255,0,11,26,255,10,6,255,0,18,255,10,6,255,0,5,10,10,10,255,0,14,10,10,10,255,0,13,4,255,0,12
       defb 4,255,0,8,26,255,0,8,255,10,4,255,0,4,255,10,4,255,0,9,255,10,4,255,0,22,255,10,4,0,0,0,10,10
       defb 255,0,20,10,10,255,0,57,5,4,4,5,4,4,255,10,30,255,7,30
       defb 255,10,19,7,255,10,11,255,0,28,10,10,255,0,28,10,10,255,0,28,10,10,255,0,28,10,10,255,0,28,10,10,0
       defb 7,255,0,24,7,0,10,10,255,0,28,10,10,255,0,28,10,10,255,0,28,10,10,0,0,0,7,255,0,18,7,255,0,5
       defb 10,10,255,0,28,10,10,255,0,28,10,10,255,0,28,10,10,255,0,8,7,255,0,7,7,255,0,11,10,10,255,0,28
       defb 10,10,255,0,28,10,10,255,0,28,255,10,61
       defb 255,0,120,19,0,0,4,255,0,11,4,0,0,0,4,0,0,0,4,255,0,5,19,255,24,5,255,0,8,255,24,17,255,0,30
       defb 19,255,0,29,24,24,255,0,9,255,24,14,255,0,29,25,255,0,11,3,3,255,0,16,25,0,0,0,24,24,255,0,5
       defb 25,255,24,10,255,0,8,25,255,0,10,25,255,0,18,25,255,0,9,19,25,255,0,13,4,0,0,24,24,25,19,255,0,7
       defb 24,24,25,255,0,7,255,24,7,255,0,4,25,24,24,255,0,8,25,255,0,13,25,255,0,4,25,255,0,10,25,255,0,13
       defb 25,255,0,4,25,255,0,8,3,4,25,3,3,4,3,255,0,8,19,25,19,0,0,4,25,255,0,4,19,255,24,60
       defb 255,0,47,30,255,0,72,19,0,0,0,5,0,0,0,5,255,0,14,4,0,0,0,4,0,19,255,24,16,0,0,0,255,24,11
       defb 255,0,30,19,255,0,29,255,24,13,255,0,29,25,255,0,6,255,24,4,255,0,4,24,24,24,255,0,11,19,25,19,255,0,5
       defb 25,255,0,19,24,24,24,25,24,24,255,0,4,25,255,0,6,4,255,0,15,25,255,0,6,25,255,0,5,24,24,255,0,15
       defb 25,255,0,13,25,255,0,7,4,4,255,0,5,19,25,255,0,6,4,4,255,0,4,19,25,255,0,7,255,24,8,25,0,0,0
       defb 255,24,10,25,255,0,49,3,4,5,5,4,5,4,5,5,4,5,3,4,3,255,24,30,255,25,30
       defb 255,0,30,19,255,0,23,19,255,0,5,24,24,24,255,0,14,255,24,8,25,255,0,29,25,255,0,12,4,255,0,4,4
       defb 255,0,11,25,255,0,4,255,24,21,255,0,4,25,24,255,0,28,25,255,0,28,24,25,255,0,29,25,255,0,29,25,0,0,0
       defb 24,255,0,7,4,4,255,0,4,4,255,0,5,4,0,0,0,19,0,25,255,0,9,255,24,20,25,255,0,15,25,255,0,18
       defb 4,4,255,0,8,19,25,19,255,0,15,4,4,255,24,7,0,0,0,255,24,7,255,0,7,255,24,6,255,0,16,25,255,0,29
       defb 25,255,0,23,255,4,6,25,19,0,0,4,255,0,8,19,255,24,30,255,25,30
       defb 255,0,11,25,255,0,6,25,255,0,22,25,255,0,6,25,255,0,11,19,255,0,10,25,255,0,6,25,19,255,0,10,255,24,5
       defb 255,0,6,25,19,255,0,5,24,24,24,0,0,255,24,5,255,0,13,24,24,24,255,0,9,25,255,0,29,25,255,0,5
       defb 19,255,0,23,25,0,0,255,24,4,19,255,0,6,3,255,0,10,255,24,5,25,255,0,6,255,24,13,255,0,5,25,255,0,29
       defb 25,255,0,10,19,255,0,18,25,255,0,4,255,24,7,255,0,18,25,255,0,29,25,255,0,15,4,255,0,4,4,255,0,8
       defb 25,4,255,0,9,19,255,24,13,255,0,5,255,24,12,255,0,78,255,4,12,255,24,30,255,25,30
       defb 255,25,4,255,0,14,255,24,4,0,0,0,255,25,8,255,0,5,24,4,19,255,0,7,24,0,0,24,255,0,5,255,25,5
       defb 255,0,6,255,24,4,255,0,6,255,24,4,255,0,6,25,25,25,255,0,28,25,25,0,0,0,19,255,0,19,19,255,0,4
       defb 25,25,0,0,0,24,24,255,0,8,19,4,255,0,8,24,24,0,0,0,25,25,255,0,4,24,24,255,0,5,255,24,6
       defb 255,0,5,24,24,255,0,4,25,25,255,0,5,24,24,4,255,0,13,24,24,255,0,4,19,25,25,24,24,255,0,4,24,24
       defb 255,0,11,4,24,24,255,0,4,24,24,25,25,255,0,7,24,24,255,0,10,24,24,255,0,7,25,25,19,255,0,7,24,24
       defb 0,0,0,24,24,0,0,0,24,24,255,0,8,25,25,24,24,255,0,24,24,24,25,25,255,0,28,25,25,255,0,28,25,25
       defb 4,255,0,10,255,24,6,255,0,10,19,25,25,24,4,255,0,8,24,255,0,6,24,255,0,8,4,24,25,25,25,24,255,0,24
       defb 24,255,25,5,24,255,0,6,255,4,10,255,0,6,24,255,25,7,255,24,22,255,25,34
       defb 255,0,60,19,255,0,28,19,255,24,5,255,0,21,255,24,4,255,0,41,4,0,0,19,0,0,4,255,0,16,25,24,255,0,4
       defb 255,24,9,255,0,5,24,25,0,0,0,24,0,0,0,25,24,255,0,20,24,25,255,0,6,25,255,0,22,25,255,0,6
       defb 25,255,0,22,25,255,0,6,25,19,255,0,16,4,0,0,0,19,25,255,0,6,255,24,9,255,0,7,255,24,9,255,0,62
       defb 19,255,0,14,4,255,0,10,4,0,0,0,255,24,4,255,0,4,255,24,14,255,0,4,255,24,4,255,0,37,255,4,16
       defb 255,0,6,19,255,24,60
       defb 0,0,25,255,0,13,25,255,0,11,25,0,0,0,25,255,0,13,25,255,0,11,25,0,0,0,24,0,0,0,24,24,255,0,5
       defb 25,24,24,25,255,0,6,24,24,24,0,0,24,255,0,8,25,255,0,5,25,255,0,9,25,255,0,4,25,255,0,8,25
       defb 255,0,4,4,25,255,0,9,25,255,0,4,25,255,0,8,25,255,24,5,25,255,0,6,4,0,19,25,255,0,4,25,0,0,0
       defb 25,24,24,0,0,24,255,0,5,24,0,0,0,24,25,255,24,5,0,0,0,24,24,0,0,0,25,255,0,9,19,25,255,0,4
       defb 25,255,0,4,25,255,0,8,25,255,0,9,24,25,255,0,4,25,255,0,11,19,0,25,255,0,10,25,255,0,4,25,19
       defb 255,0,10,255,24,5,0,0,25,255,0,5,24,24,0,0,0,255,24,9,255,0,10,25,4,255,0,4,25,255,0,9,25
       defb 255,0,13,25,24,24,0,0,0,25,255,0,9,25,255,0,6,19,255,0,6,25,255,0,5,25,255,0,9,25,255,0,4
       defb 19,0,24,24,255,0,4,24,24,4,255,0,4,24,0,0,0,24,24,255,4,4,25,0,0,0,24,24,255,0,8,25,24,24,24
       defb 0,0,25,255,0,4,25,255,24,4,25,255,0,13,25,255,0,10,25,255,0,16,4,4,25,4,4,4,255,0,7,25,19
       defb 255,0,7,4,0,0,255,24,60
       defb 255,0,45,19,4,4,255,0,15,25,24,255,0,7,255,24,6,255,0,6,24,24,25,255,0,6,25,255,0,22,25,255,0,6
       defb 25,255,0,21,19,25,255,0,6,25,24,24,255,0,15,4,4,24,24,24,25,0,0,0,24,0,0,25,255,0,5,24,24,24
       defb 255,0,7,255,24,4,0,0,0,25,0,0,24,0,0,0,25,255,0,5,3,3,4,4,255,0,5,4,4,255,0,6,25
       defb 255,0,6,25,255,0,4,255,24,6,0,0,0,24,24,24,255,0,6,25,255,0,6,25,255,0,7,25,25,255,0,5,25,25
       defb 255,0,5,19,25,255,0,5,24,25,24,255,0,5,19,25,25,0,0,19,0,0,25,25,255,0,5,24,25,24,255,0,5
       defb 25,255,0,5,24,24,25,25,0,0,24,0,0,25,25,24,255,0,5,25,255,0,6,25,255,0,7,25,25,0,24,24,24
       defb 0,25,25,255,0,6,25,255,0,5,19,25,255,0,7,25,25,255,24,6,25,255,0,6,25,19,0,0,255,24,6,255,0,5
       defb 25,255,24,7,25,255,0,4,255,24,4,255,0,13,255,24,9,255,0,25,25,255,0,29,25,255,0,14,255,24,60
       defb 24,24,255,0,26,255,24,4,255,0,26,255,24,4,19,255,0,24,19,255,24,8,0,0,0,24,24,24,255,0,7,24,24
       defb 255,0,4,255,24,7,255,0,8,25,4,4,255,0,6,25,255,0,8,255,24,4,255,0,8,25,24,24,24,255,0,5,25
       defb 255,0,8,255,24,4,255,0,8,25,255,0,8,25,255,0,8,255,24,4,255,4,4,0,0,0,19,25,255,0,8,25,19
       defb 255,0,4,4,4,4,255,24,13,255,0,7,255,24,14,255,0,26,255,24,4,255,0,26,255,24,4,255,0,12,4,255,0,12
       defb 19,255,24,17,0,0,0,255,24,14,255,0,9,24,255,0,5,24,255,0,10,255,24,4,255,0,8,19,24,255,0,4,19
       defb 24,19,255,0,9,255,24,4,255,0,7,24,24,24,0,0,0,255,24,5,255,0,8,255,24,4,255,0,26,255,24,4,4,4
       defb 255,0,19,255,4,5,255,24,62
       defb 255,25,31,255,0,28,25,25,255,0,28,25,25,255,0,28,255,25,4,255,0,24,255,25,4,255,0,28,25,25,255,0,28
       defb 25,25,255,0,28,255,25,5,255,0,22,255,25,5,255,0,28,25,25,255,0,28,25,25,255,0,28,255,25,6,255,0,20
       defb 255,25,6,255,0,28,25,25,255,0,28,255,25,9,255,0,15,255,25,8,255,0,28,25,25,255,0,28,255,25,61
numsc  defb 30
nmedat defb 0,0,152,5,2,12,120,80,2,10,40,135,2,10,48,85,6,2,152,5,8,29,152,145,4,32,96,35,4,32,48,115,255
       defb 2,10,40,15,0,0,152,10,2,12,120,120,5,27,40,75,6,2,152,10,8,29,40,145,255
       defb 0,0,152,5,2,12,104,80,5,27,72,110,6,2,152,5,8,28,56,5,4,32,56,105,2,10,72,45,2,10,152,115,255
       defb 2,10,40,125,0,0,152,5,2,12,40,45,5,27,80,50,4,26,80,65,6,2,152,5,8,28,120,5,2,10,120,120,255
       defb 0,0,152,5,2,12,120,125,2,12,120,30,5,27,32,100,5,27,32,40,6,2,152,5,8,28,64,80,2,23,152,100,255
       defb 0,0,152,5,2,12,128,110,5,27,64,95,6,2,152,5,2,12,24,60,4,32,136,55,8,29,152,145,2,23,152,120,2,10,64,20,255
       defb 0,0,152,5,4,26,88,95,5,27,96,75,6,2,152,5,2,12,96,130,8,29,56,145,2,23,120,40,255
       defb 0,0,152,15,2,12,40,115,2,10,96,70,4,26,48,55,5,27,104,140,6,2,152,15,8,29,40,145,2,23,120,25,255
       defb 0,0,152,15,2,12,56,25,2,10,96,50,5,27,32,75,4,26,96,25,6,2,152,15,8,28,56,5,2,12,152,115,2,10,56,120,255
       defb 0,0,152,25,2,12,80,135,2,12,80,20,3,33,104,85,6,34,120,85,6,2,152,25,255
       defb 0,0,152,20,2,14,88,20,2,16,120,30,5,27,104,85,4,26,48,55,6,2,152,20,8,29,48,145,255
       defb 0,0,152,20,2,14,96,65,2,16,152,105,5,27,32,105,5,27,32,55,6,2,152,20,8,29,48,145,255
       defb 0,0,152,15,2,16,48,30,2,14,128,90,2,14,152,115,5,27,32,145,5,27,96,45,6,2,152,15,8,29,152,145,255
       defb 0,0,152,15,2,14,120,95,2,16,40,80,5,27,24,50,6,2,152,15,4,32,88,115,8,29,152,145,255
       defb 0,0,152,10,2,16,112,80,2,14,32,130,5,27,80,80,6,2,152,10,8,29,32,145,4,32,56,20,255
       defb 0,0,152,20,2,14,128,25,2,16,104,135,5,27,56,100,6,2,152,20,8,29,40,85,4,32,40,75,255
       defb 0,0,152,15,2,16,24,110,2,14,152,90,5,27,104,85,6,2,152,15,8,28,24,5,255
       defb 0,0,152,15,2,16,48,70,5,27,88,75,6,2,152,15,8,28,24,5,2,23,152,115,255
       defb 0,0,152,10,2,16,64,125,2,14,88,10,5,27,72,75,6,2,152,10,4,32,96,75,8,29,120,145,255
       defb 0,0,152,15,3,35,96,80,6,2,152,15,6,36,112,80,255
       defb 5,27,48,55,2,20,152,75,2,18,72,105,2,18,96,60,0,0,152,10,6,2,152,10,8,28,152,130,255
       defb 2,20,72,30,0,0,152,10,6,2,152,10,4,32,80,90,5,27,88,115,8,29,80,145,255
       defb 2,20,48,30,2,18,24,100,2,18,152,120,0,0,152,10,6,2,152,10,8,28,48,5,255
       defb 2,20,72,15,2,20,152,60,2,18,120,120,0,0,152,10,6,2,152,10,4,32,136,80,8,28,112,5,255
       defb 5,27,104,125,5,27,96,30,2,18,120,80,0,0,152,30,6,2,152,30,8,28,72,10,255
       defb 5,27,88,80,2,20,128,60,2,18,96,45,0,0,152,25,6,2,152,25,8,29,128,145,255
       defb 2,20,88,110,0,0,152,15,6,2,152,15,8,29,24,75,5,27,88,50,5,27,120,130,255
       defb 2,20,24,70,2,18,152,100,4,26,56,80,0,0,152,25,6,2,152,25,5,27,48,45,8,28,88,25,255
       defb 0,0,152,25,6,2,152,25,2,20,104,40,5,27,40,80,8,28,104,15,2,23,104,115,255
       defb 3,37,96,85,0,0,152,25,6,38,112,85,6,2,152,25,255
NUMOBJ equ 17
objdta equ $
       defb 192,129,66,192,129,129,66,66,66,3,2,128,192,129,66,192
       defb 192,129,66,192,129,129,66,64,128,192,192,64,192,192,192,192
       defb 129,129,64,64,129,128,64,64,192,128,64,192,129,128,64,64
       defb 128,128,64,64,192,128,64,192,192,128,64,192,64,0,0,128
       defb 255,40,125,255,40,125
       defb 0,4,8,0,0,4,8,0,0,4,8,0,0,4,8,0
       defb 12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12
       defb 12,12,12,12,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 255,24,5,255,24,5
       defb 0,4,8,0,0,4,8,0,0,4,8,0,0,4,8,0
       defb 0,4,8,0,3,6,8,0,12,12,12,12,12,12,12,12
       defb 12,12,12,12,3,6,9,3,2,4,8,0,0,4,8,0
       defb 0,4,8,0,0,4,8,0,0,4,8,0,0,4,8,0
       defb 255,24,5,255,24,5
       defb 0,0,0,0,0,0,0,0,17,0,0,34,51,35,19,51
       defb 51,35,19,51,51,35,19,51,0,1,3,0,51,35,19,51
       defb 51,35,19,51,51,35,19,51,51,35,19,51,51,35,19,51
       defb 51,35,19,51,51,35,19,51,51,35,19,51,51,35,19,51
       defb 255,24,5,255,24,5
       defb 0,0,0,0,0,0,0,0,51,2,1,51,34,35,19,17
       defb 34,35,19,17,34,17,34,17,17,0,0,34,0,34,17,0
       defb 0,17,34,0,1,1,0,0,35,35,35,51,35,35,35,51
       defb 35,35,35,51,35,35,35,34,35,51,35,34,1,0,0,0
       defb 255,24,5,255,24,5
       defb 0,41,41,0,20,60,60,40,40,41,41,0,40,41,41,0
       defb 20,60,60,40,0,41,41,40,0,41,41,40,20,60,60,40
       defb 0,40,40,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 30,24,5,30,24,5
       defb 1,0,1,2,0,2,1,2,0,1,2,0,0,1,2,0
       defb 0,2,1,0,1,0,0,2,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,24,5,0,24,5
       defb 0,0,22,40,0,0,40,60,0,0,40,40,60,60,40,60
       defb 60,60,40,60,0,0,40,41,0,0,40,60,0,0,20,40
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 254,24,5,254,24,5
       defb 204,204,204,204,136,0,0,70,136,204,204,70,136,204,204,70
       defb 136,204,204,70,137,3,3,70,204,204,204,204,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 255,24,5,255,24,5
       defb 0,0,0,0,0,0,0,0,4,12,12,0,4,12,12,8
       defb 4,12,12,8,4,12,12,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 255,24,5,255,24,5
       defb 0,0,0,0,4,0,0,8,12,8,4,12,12,8,4,12
       defb 2,0,1,3,12,8,4,12,12,8,4,12,12,8,4,12
       defb 12,8,4,12,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 255,24,5,255,24,5
       defb 0,12,12,0,4,6,1,8,9,4,1,6,9,4,8,6
       defb 9,0,0,6,4,2,1,8,0,12,12,0,0,0,0,0
       defb 0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 255,24,5,255,24,5
       defb 4,8,6,8,12,9,6,12,12,9,6,12,4,12,12,8
       defb 0,12,12,0,0,4,8,0,0,4,8,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 255,24,5,255,24,5
       defb 0,4,8,0,0,4,8,0,0,4,8,0,0,4,8,0
       defb 4,12,12,8,4,12,12,8,4,12,12,8,4,12,12,8
       defb 4,12,12,8,4,12,12,8,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 255,24,5,255,24,5
       defb 0,4,8,0,0,4,8,0,0,4,8,0,0,4,8,0
       defb 0,4,8,0,0,4,8,0,0,4,8,0,12,12,12,12
       defb 12,12,12,12,0,4,8,0,0,4,8,0,0,4,8,0
       defb 0,4,8,0,0,4,8,0,0,4,8,0,0,4,8,0
       defb 255,24,5,255,24,5
       defb 0,0,0,0,0,0,0,0,0,0,0,0,48,16,32,48
       defb 48,16,32,48,48,16,32,48,48,16,32,48,0,0,0,0
       defb 48,16,32,48,48,16,32,48,48,16,32,48,48,16,32,48
       defb 48,16,32,48,48,16,32,48,48,16,32,48,48,16,32,48
       defb 255,24,5,255,24,5
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 255,24,5,255,24,5
palett equ $
       defb 0,2,6,8,18,20,24,26,0,3,15,5,24,11,16,25
font   equ $
       defb 0,0,0,0,0,0,0,0
       defb 0,32,32,32,32,0,32,0
       defb 80,80,0,0,0,0,0,0
       defb 0,80,248,80,248,80,0,0
       defb 32,112,192,96,48,224,32,0
       defb 0,216,216,248,248,112,32,0
       defb 32,32,32,32,248,248,248,248
       defb 216,216,0,216,216,216,216,216
       defb 248,0,112,112,112,0,248,0
       defb 0,64,32,32,32,32,64,0
       defb 0,32,248,112,248,32,0,0
       defb 0,64,64,224,64,64,0,0
       defb 0,0,0,0,0,0,32,64
       defb 0,0,0,224,0,0,0,0
       defb 0,0,0,0,0,0,64,0
       defb 0,32,32,64,64,128,128,0
       defb 0,96,144,176,208,144,96,0
       defb 0,64,192,64,64,64,224,0
       defb 0,96,144,16,96,128,240,0
       defb 0,96,144,32,16,144,96,0
       defb 0,32,96,160,240,32,32,0
       defb 0,240,128,224,16,16,224,0
       defb 0,96,128,224,144,144,96,0
       defb 0,240,16,32,32,64,64,0
       defb 0,96,144,96,144,144,96,0
       defb 0,96,144,144,112,16,96,0
       defb 0,0,64,0,0,64,0,0
       defb 0,0,64,0,0,64,128,0
       defb 0,0,32,64,128,64,32,0
       defb 0,0,128,64,32,64,128,0
       defb 0,0,0,224,0,224,0,0
       defb 0,224,16,32,64,0,64,0
       defb 0,96,144,176,176,128,112,0
       defb 0,96,144,144,240,144,144,0
       defb 0,224,144,224,144,144,224,0
       defb 0,96,144,128,128,144,96,0
       defb 0,192,160,144,144,144,224,0
       defb 0,240,128,224,128,128,240,0
       defb 0,240,128,224,128,128,128,0
       defb 0,96,144,128,176,144,112,0
       defb 0,144,144,240,144,144,144,0
       defb 0,224,64,64,64,64,224,0
       defb 0,48,16,16,16,144,96,0
       defb 0,144,160,192,160,144,144,0
       defb 0,128,128,128,128,128,240,0
       defb 0,136,216,168,136,136,136,0
       defb 0,144,208,208,176,176,144,0
       defb 0,96,144,144,144,144,96,0
       defb 0,224,144,144,224,128,128,0
       defb 0,96,144,144,176,144,112,0
       defb 0,224,144,160,192,160,144,0
       defb 0,112,128,64,32,16,224,0
       defb 0,248,32,32,32,32,32,0
       defb 0,144,144,144,144,144,96,0
       defb 0,136,136,80,80,32,32,0
       defb 0,136,136,136,168,168,80,0
       defb 0,144,144,96,96,144,144,0
       defb 0,136,136,80,32,32,32,0
       defb 0,240,16,32,64,128,240,0
       defb 0,96,64,64,64,64,96,0
       defb 0,128,128,64,64,32,32,0
       defb 0,96,32,32,32,32,96,0
       defb 64,160,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,248
       defb 48,64,64,224,64,64,240,0
       defb 0,224,16,112,144,144,112,0
       defb 0,128,128,224,144,144,224,0
       defb 0,0,96,128,128,144,96,0
       defb 0,16,16,112,144,144,112,0
       defb 0,0,96,144,240,128,112,0
       defb 0,48,64,224,64,64,64,0
       defb 0,0,112,144,144,112,16,96
       defb 0,128,128,128,224,144,144,0
       defb 0,64,0,192,64,64,224,0
       defb 0,32,0,96,32,32,32,192
       defb 0,128,144,160,192,160,144,0
       defb 0,64,64,64,64,64,32,0
       defb 0,0,80,168,168,136,136,0
       defb 0,0,224,144,144,144,144,0
       defb 0,0,96,144,144,144,96,0
       defb 0,0,224,144,144,224,128,128
       defb 0,0,112,144,144,112,16,16
       defb 0,0,96,128,128,128,128,0
       defb 0,0,112,128,96,16,224,0
       defb 0,64,224,64,64,64,32,0
       defb 0,0,144,144,144,144,96,0
       defb 0,0,136,136,80,80,32,0
       defb 0,0,136,136,136,168,80,0
       defb 0,0,144,144,96,144,144,0
       defb 0,0,144,144,144,112,16,224
       defb 0,0,240,32,64,128,240,0
       defb 0,96,64,192,64,64,96,0
       defb 0,64,64,64,64,64,64,0
       defb 0,192,64,96,64,64,192,0
       defb 0,80,160,0,0,0,0,0
       defb 112,136,32,64,64,32,136,112
jtab   equ $
       defb 99
keys   defb 67,69,34,27,47,71,63,64,65,57,56
plsnd  ret
ibloc  jp initsc
rbloc  ret
eop    equ $
